package com.qizhen.healsphere;

import com.qizhen.healsphere.util.FileCommonUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xwpf.usermodel.*;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class WordSplitter {

    private static final String ZHANG_PATTERN = "^第[零一二三四五六七八九十百千万0123456789]+章\\s+[^\\n]+";
    public void splitWordDocument(String inputFilePath, String outputDirectory,String pattern) throws IOException {
        System.out.println(inputFilePath+"==="+outputDirectory+"==="+pattern);
        XWPFDocument document = new XWPFDocument(new FileInputStream(inputFilePath));
        StringBuilder currentChapter = new StringBuilder();
        String currentChapterTitle = null;
        Pattern compile = Pattern.compile(pattern);
        for (IBodyElement element : document.getBodyElements()) {
            if (element instanceof XWPFParagraph) {
                XWPFParagraph paragraph = (XWPFParagraph) element;
                String text = paragraph.getText();
                Matcher matcher = compile.matcher(text);
                boolean isAppendTitle = false;
                if (matcher.find()) {
                    String tempTitle = matcher.group();
                    //标题一样的话，不保存
                    if (currentChapterTitle == null) {
                        currentChapterTitle = tempTitle;
                        isAppendTitle = true;
                    } else if (currentChapterTitle != null && !trim(currentChapterTitle).equals(trim(tempTitle))) {
                        save(currentChapterTitle, currentChapter.toString(), outputDirectory);
                        currentChapter.setLength(0); // Clear the StringBuilder
                        currentChapterTitle = tempTitle;
                        isAppendTitle = true;
                    }
                }
                String trimText = trim(text);
                if (StringUtils.isNotEmpty(text) && (isAppendTitle || !trim(currentChapterTitle).equals(trimText))
                        && !"本章数字资源".equals(trimText)) {
                    currentChapter.append(text).append("\n");
                }

            } /*else if (element instanceof XWPFTable) {
                // Handle tables if necessary
                currentChapter.append(element.toString()).append("\n");
            }*/
        }
        save(currentChapterTitle, currentChapter.toString(), outputDirectory);
        document.close();
    }

    private void save(String chapterTitle, String chapterContents, String outputDirectory) throws IOException {
        if (StringUtils.isEmpty(chapterTitle)) {
            return;
        }
        String fileName = FileCommonUtils.sanitizeFileName(chapterTitle);
        String fileNameEXt = fileName + ".txt";
        outputDirectory += fileName + "\\";
        File outputFile = new File(outputDirectory, fileNameEXt);
        // 检查目录是否存在，如果不存在则创建目录
        File directory = new File(outputDirectory);
        if (!directory.exists()) {
            boolean isCreated = directory.mkdirs();
            if (isCreated) {
                System.out.println("目录已创建: " + outputDirectory);
            } else {
                System.out.println("目录创建失败: " + outputDirectory);
                return;
            }
        }
        try (BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile))) {
            writer.write(chapterContents);
        }
    }

    private String trim(String text) {
        return text.replaceAll("\\s+", "");
    }


    public static void removeHeadersAndFooters(String inputFilePath, String outputFilePath) {
        try (FileInputStream fis = new FileInputStream(inputFilePath);
             XWPFDocument document = new XWPFDocument(fis)) {

            // 删除所有页眉
            for (XWPFHeader header : document.getHeaderList()) {
                List<XWPFParagraph> paragraphs = header.getParagraphs();
                for (int i = paragraphs.size() - 1; i >= 0; i--) {
                    header.removeParagraph(paragraphs.get(i));
                }
            }

            // 删除所有页脚
            for (XWPFFooter footer : document.getFooterList()) {
                List<XWPFParagraph> paragraphs = footer.getParagraphs();
                for (int i = paragraphs.size() - 1; i >= 0; i--) {
                    footer.removeParagraph(paragraphs.get(i));
                }
            }

            // 保存修改后的文档
            try (FileOutputStream fos = new FileOutputStream(outputFilePath)) {
                document.write(fos);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static void main(String[] args) {
        WordSplitter wordSplitter1 = new WordSplitter();
        String inputFilePath = "E:\\project\\vscode\\急诊与灾难医学（第4版）.docx";
        String outputDirectory = "E:\\project\\vscode\\《急诊与灾难医学（第4版）》\\";
        try {
            //removeHeadersAndFooters(inputFilePath, outputFilePath);
            wordSplitter1.splitWordDocument(inputFilePath, outputDirectory,ZHANG_PATTERN);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}