package com.qizhen.healsphere.facade;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.qizhen.healsphere.common.exception.CommonErrorCode;
import com.qizhen.healsphere.common.exception.CommonException;
import com.qizhen.healsphere.repository.mapper.entity.PresetInfo;
import com.qizhen.healsphere.repository.neo4j.BaseNodeRepository;
import com.qizhen.healsphere.service.impl.KgServiceImpl;
import com.qizhen.healsphere.web.dto.*;
import com.qizhen.healsphere.web.param.KgQuery;
import com.qizhen.healsphere.web.param.KgTree;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicLong;

/**
 * @Description: 用户日志业务层
 * @author: gaodm
 * @time: 2018/8/6 9:11
 */
@Component
public class KgFacade extends KgServiceImpl {
    @Autowired
    private PresetInfoFacade presetInfoFacade;
    @Autowired
    private BaseNodeRepository baseNodeRepository;


    public List<NodeDTO> getNode(KgQuery kgQuery){

        return baseNodeRepository.findNodeByNameLike(kgQuery.getLabelName(), kgQuery.getInputStr());
    }

    public GraphLabelDTO getGraphFac(KgQuery kgQuery) {
        GraphLabelDTO graphLabelDTO = new GraphLabelDTO();
        List<CategorieDTO> categories = new ArrayList<>();
        List<GNodeDTO> node = new ArrayList<>();
        List<LinkDTO> links = new ArrayList<>();
        List<GraphDTO> res = this.getGraph(kgQuery);
        if (CollUtil.isEmpty(res)) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS);
        } else {
            Map<String,Object> itemStyleMap = new HashMap<>();
            itemStyleMap.put("display",true);
            long nodeId = 0;
            categories.add(new CategorieDTO("中心词"));
            categories.add(new CategorieDTO("关系"));
            Map<String, Integer> cMap = new HashMap<>();
            cMap.put("中心词", 0);
            cMap.put("关系", 1);

            GraphDTO graphDTO = res.get(0);
            GNodeDTO gNodeDTO
                    = new GNodeDTO(graphDTO.getLabel(), 0, graphDTO.getName(), nodeId, "circle", 50,graphDTO.getProperties(),graphDTO.getId(),itemStyleMap);
            nodeId++;
            node.add(gNodeDTO);

            if (CollUtil.isNotEmpty(graphDTO.getENodeRSDTOS())) {
                Integer rsId = 2;
                for (BaseNodeRSDTO baseNodeRSDTO : graphDTO.getENodeRSDTOS()) {
                    if (null == cMap.get(baseNodeRSDTO.getRType())) {
                        cMap.put(baseNodeRSDTO.getRType(), rsId);
                        categories.add(new CategorieDTO(baseNodeRSDTO.getRType()));
                        rsId++;
                    }
                    GNodeDTO nNodeDTO
                            = new GNodeDTO(graphDTO.getLabel(), 1, "", nodeId, "diamond", 10,graphDTO.getProperties(),graphDTO.getId(),itemStyleMap);
                    node.add(nNodeDTO);
                    links.add(new LinkDTO(gNodeDTO.getName(), nNodeDTO.getName(), baseNodeRSDTO.getRType(),baseNodeRSDTO.getRType()));
                    nodeId++;
                    if (CollUtil.isNotEmpty(baseNodeRSDTO.getENodeDTOS())) {
                        for (NextNodeDTO baseNodeDTO : baseNodeRSDTO.getENodeDTOS()) {
                            Map<String,Object> childrenItemStyleMap = new HashMap<>();
                            String symbol = "circle";
                            childrenItemStyleMap.put("display", true);
//                            if (baseNodeDTO.getPCount() == 0) {
//                                childrenItemStyleMap.put("color", "#808080");
//                            }
                            if (baseNodeDTO.getPCount() == 0) {
                                childrenItemStyleMap.put("display", false);
                            }
                            GNodeDTO eNodeDTO
                                    = new GNodeDTO(baseNodeDTO.getLabel(), cMap.get(baseNodeRSDTO.getRType()),
                                    baseNodeDTO.getName(), nodeId, symbol, 28, baseNodeDTO.getProperties(),baseNodeDTO.getId(),childrenItemStyleMap);
                            nodeId++;
                            node.add(eNodeDTO);
                            links.add(new LinkDTO(nNodeDTO.getName(), eNodeDTO.getName(), "",baseNodeRSDTO.getRType()));
                        }
                    }
                }
            }
            graphLabelDTO.setCategories(categories);
            graphLabelDTO.setNode(node);
            graphLabelDTO.setLinks(links);
        }

        return graphLabelDTO;
    }

    public TreeDTO getTreeFac(KgTree kgTree) {
        TreeDTO treeDTO = new TreeDTO();
        if ((kgTree.getType() == 1
                && (kgTree.getSubType() == 1 || kgTree.getSubType() == 2 || kgTree.getSubType() == 3))
                || (kgTree.getType() == 2 && (kgTree.getSubType() == 0 || kgTree.getSubType() == 1))
                || (kgTree.getType() == 3 && (kgTree.getSubType() == 0))
                || (kgTree.getType() == 4 && (kgTree.getSubType() == 0))
                || (kgTree.getType() == 5 && (kgTree.getSubType() == 0))
                || (kgTree.getType() == 6 && (kgTree.getSubType() == 0))) {
            // KgQueryVO kgQueryVO = new KgQueryVO();
            // kgQueryVO.setLabelName("诊断依据");
            // kgQueryVO.setInputStr("诊断依据");
            // String tree = this.getTree(kgQueryVO);
            PresetInfo presetInfo = presetInfoFacade.getPresetInfo((kgTree.getType() * 10 + kgTree.getSubType()));
            String tree = "";
            if (null != presetInfo){
                tree = presetInfo.getContent();
            }
            if (StrUtil.isBlank(tree)) {
                throw new CommonException(CommonErrorCode.NOT_EXISTS);
            } else {
                treeDTO = getTreeDTO(tree);
            }
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR);
        }
        return treeDTO;
    }

    private TreeDTO getTreeDTO(String tree) {
        TreeDTO treeDTO = new TreeDTO();
        List<TreeNodeDTO> nodes = new ArrayList<>();
        Tree treeObj = JSONUtil.toBean(tree, Tree.class);
        if (null != treeObj) {
            // 使用 AtomicLong 来生成唯一ID
            AtomicLong idGenerator = new AtomicLong(0);
            
            // 存储父节点的完整路径到ID的映射
            Map<String, Long> pathIdMap = new HashMap<>();
            
            // 根节点特殊处理
            long rootId = idGenerator.getAndIncrement();
            pathIdMap.put("根节点", rootId);
            
            // 从根节点开始构建树
            addTree(nodes, treeObj, pathIdMap, "根节点", idGenerator, "根节点");
        }

        treeDTO.setNodes(nodes);
        return treeDTO;
    }

    private void addTree(List<TreeNodeDTO> nodes, Tree treeObj, Map<String, Long> pathIdMap, 
                        String parentName, AtomicLong idGenerator, String parentPath) {
        // 构建当前节点的完整路径
        String currentPath = parentPath + "/" + treeObj.getName();
        
        // 总是生成新的ID
        Long currentId = idGenerator.getAndIncrement();
        // 保存当前节点路径和ID的映射
        pathIdMap.put(currentPath, currentId);
        
        // 获取父节点ID
        Long parentId = pathIdMap.get(parentPath);
        
        // 判断是否为叶子节点
        boolean isLeaf = CollUtil.isEmpty(treeObj.getSNode());
        
        // 设置图标
        String icon = isLeaf ? "/images/node.png" : "/images/icon.png";
        String iconOpen = isLeaf ? "" : "/images/iconOpen.png";
        String iconClose = isLeaf ? "" : "/images/iconClose.png";

        // 创建树节点
        TreeNodeDTO node = TreeNodeDTO.builder()
            .id(currentId.intValue())
            .pId(parentId.intValue())
            .name(treeObj.getName())
            .icon(icon)
            .iconOpen(iconOpen)
            .iconClose(iconClose)
            .build();
        
        nodes.add(node);

        // 递归处理子节点
        if (!isLeaf) {
            for (Tree childTree : treeObj.getSNode()) {
                addTree(nodes, childTree, pathIdMap, treeObj.getName(), idGenerator, currentPath);
            }
        }
    }

}
