package com.qizhen.healsphere.service.impl;

import com.qizhen.healsphere.repository.neo4j.BaseNodeRepository;
import com.qizhen.healsphere.repository.neo4j.BaseRelationshipRepository;
import com.qizhen.healsphere.service.EntityService;
import com.qizhen.healsphere.service.RelationshipService;
import com.qizhen.healsphere.web.vo.EntityVO;
import com.qizhen.healsphere.web.vo.RelationshipNameVO;
import com.qizhen.healsphere.web.vo.RelationshipVO;
import com.qizhen.healsphere.web.vo.UpdateRelationTypeVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class RelationshipServiceImpl implements RelationshipService {

    @Autowired
    BaseRelationshipRepository relationshipRepository;


    @Override
    public Boolean createRelationship(List<RelationshipVO> relationshipList) {
        try {
            relationshipList.stream().forEach(relationship ->
                    relationshipRepository.createRelationship(
                            relationship.getStartLabel(),
                            relationship.getStartId(),
                            relationship.getEndLabel(),
                            relationship.getEndId(),
                            relationship.getRelationshipType(),
                            relationship.getProperty()
                    )
            );
            // 如果所有关系都成功创建，则返回 true
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public Boolean deleteRelationship(List<RelationshipVO> relationshipList) {
        try {
            relationshipList.stream().forEach(relationship ->
                    relationshipRepository.deleteRelationship(
                            relationship.getStartLabel(),
                            relationship.getStartId(),
                            relationship.getEndLabel(),
                            relationship.getEndId(),
                            relationship.getRelationshipType()
                    )
            );
            // 如果所有关系都成功删除，则返回 true
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    @Override
    public List<String> findRelationshipType(RelationshipNameVO relationshipName) {

        return relationshipRepository.findRelationshipNames(relationshipName.getRelationshipType());
    }

    @Override
    public Boolean updateRelationshipType(List<UpdateRelationTypeVO> updateRelationshipTypeList) {
        try {
            updateRelationshipTypeList.stream().forEach(relationship ->
                    relationshipRepository.updateRelationship(
                            relationship.getOldRelationshipType(),
                            relationship.getNewRelationshipType()
                    )
            );
            // 如果名称都成功更新，则返回 true
            return true;
        } catch (Exception e) {
            return false;
        }
    }

}
