package com.qizhen.healsphere.util;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class SentenceSplitter {

    /**
     * 将一段话中的每句话拆分出来，支持中英文混合。
     *
     * @param text 输入的文本
     * @return 拆分后的句子列表
     */
    public static List<String> splitSentences(String text) {
        List<String> sentences = new ArrayList<>();
        // 定义正则表达式，匹配中英文句子分隔符
        String regex = "[。！？]+";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(text);

        int start = 0;
        while (matcher.find()) {
            int end = matcher.end();
            // 包含句子分隔符和可能的换行符
            String sentence = text.substring(start, end).trim();
            // 检查句子分隔符后面是否有换行符
            if (end < text.length() && text.charAt(end) == '\n') {
                sentence += "\n";
            }
            if (!sentence.isEmpty()) {
                sentences.add(sentence);
            }
            start = end;
        }

        // 添加最后一段未匹配到分隔符的内容，保留可能的换行符
        if (start < text.length()) {
            String lastSentence = text.substring(start).trim();
            if (!lastSentence.isEmpty()) {
                sentences.add(lastSentence);
            }
        }

        return sentences;
    }

    public static void main(String[] args) {
        // 测试用例
        String input = "（一）热度与热程\n" +
                "通常将发热程度分为：①低热：37.3～38℃;②中等度热：38.1～39℃;③高热：39.1～41℃;④超高 热：41℃以上。\n" +
                "热程是指发热病程持续的时间。通常按发热持续的时间将其分为急性发热和长期发热，而急性 发热在急诊最为常见。";
        List<String> result = splitSentences(input);

        // 输出结果
        for (String sentence : result) {
            System.out.println(sentence);
            System.out.println("=========");
        }
    }
}