package com.qizhen.healsphere.web;

import com.qizhen.healsphere.service.EntityService;
import com.qizhen.healsphere.web.dto.RespDTO;
import com.qizhen.healsphere.web.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("/entity")
@Api(value = "实体API", tags = { "实体API" })
public class EntityController {

    @Autowired
    EntityService entityService;

    @ApiOperation(value = "批量新增实体",
            notes = "")
    @RequestMapping(value = "/createEntity",method = RequestMethod.POST)
    @ResponseBody
    public RespDTO<List<Map<String, Object>>> createEntity(@RequestBody List<CreateEntityVO> createEntityList) {

        List<Map<String, Object>> list = entityService.createEntity(createEntityList);
        return RespDTO.onSuc(list);
    }


    @ApiOperation(value = "批量删除实体",
            notes = "")
    @RequestMapping(value = "/deleteEntity",method = RequestMethod.POST)
    @ResponseBody
    public RespDTO<Boolean> deleteEntity(@RequestBody List<Long> ids) {

        Boolean flag = entityService.deleteEntity(ids);
        return RespDTO.onSuc(flag);
    }


    @ApiOperation(value = "批量修改实体名称",
            notes = "")
    @RequestMapping(value = "/updateEntityName",method = RequestMethod.POST)
    @ResponseBody
    public RespDTO<List<Map<String, Object>>> updateEntityName(@RequestBody List<UpdateEntityVO> updateEntityList) {

        List<Map<String, Object>> list = entityService.updateEntityName(updateEntityList);
        return RespDTO.onSuc(list);
    }


    @ApiOperation(value = "实体列表查询（根据标签和名字模糊查询实体）",
            notes = "")
    @RequestMapping(value = "/findEntityListByName",method = RequestMethod.POST)
    @ResponseBody
    public RespDTO<List<Map<String, Object>>> findEntityListByName(@RequestBody CreateEntityVO entity) {

        List<Map<String, Object>> list = entityService.findEntityListByName(entity.getLabel(), entity.getName());
        return RespDTO.onSuc(list);
    }


    @ApiOperation(value = "实体详情查询（根据id查询实体）",
            notes = "")
    @RequestMapping(value = "/findEntity",method = RequestMethod.POST)
    @ResponseBody
    public RespDTO<Map<String, Object>> findEntity(@RequestBody FindEntityVO id) {

        Map<String, Object> entity = entityService.findEntityById(id);
        return RespDTO.onSuc(entity);
    }

    @ApiOperation(value = "实体合并",
            notes = "")
    @RequestMapping(value = "/mergeEntity",method = RequestMethod.POST)
    @ResponseBody
    public RespDTO<Map<String, Object>> mergeEntity(@RequestBody MergeEntityVO entityVO) {

        Map<String, Object> entity = entityService.mergeEntity(entityVO);
        return RespDTO.onSuc(entity);
    }
}
