package com.qizhen.healsphere;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.qizhen.healsphere.common.ai.BaidubceUtil;
import com.qizhen.healsphere.common.ai.Knowlege;
import com.qizhen.healsphere.common.ai.QizhenAssistant;
import com.qizhen.healsphere.repository.neo4j.entity.BaseEntity;
import com.qizhen.healsphere.service.EntityService;
import com.qizhen.healsphere.service.RelationshipService;
import com.qizhen.healsphere.web.vo.CreateEntityVO;
import com.qizhen.healsphere.web.vo.RelationshipVO;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@RunWith(SpringRunner.class)
@ComponentScan(basePackages = {"com.qizhen.healsphere.model","com.qizhen.healsphere.repository"})
@SpringBootTest
public class DataWriteTest30zc {
    @Autowired
    RelationshipService relationshipService;
    @Autowired
    EntityService entityService;


    private static String diseaseStr = "霍乱" +
            ",伤寒" +
            ",副伤寒"+
            ",细菌性痢疾" +
            ",肠阿米巴病" +
            ",感染性腹泻" +
            ",急性肠炎" +
            ",婴儿腹泻" +
            ",肺结核" +
            ",结核性胸膜炎" +
            ",原发性肺结核" +
            ",结核性脑膜炎" +
            ",骨结核" +
            ",颈椎结核" +
            ",胸椎结核" +
            ",腰椎结核" +
            ",脊柱结核" +
            ",骶髂关节结核" +
            ",肾结核" +
            ",子宫内膜结核" +
            ",附睾结核" +
            ",颈淋巴结结核" +
            ",肠结核" +
            ",结核性腹膜炎" +
            ",鼠疫" +
            ",钩端螺旋体病" +
            ",麻风" +
            ",皮肤白喉" +
            ",白喉" +
            ",猩红热" +
            ",流行性脑脊髓膜炎" +
            ",先天性梅毒" +
            ",梅毒" +
            ",淋病" +
            ",流行性斑疹伤寒" +
            ",狂犬病" +
            ",流行性乙型脑炎" +
            ",登革热" +
            ",手足口病" +
            ",乙型病毒性肝炎" +
            ",慢性病毒性肝炎" +
            ",艾滋病" +
            ",疟疾" +
            ",内脏利什曼病" +
            ",黑热病" +
            ",埃及血吸虫病" +
            ",血吸虫病" +
            ",肺血吸虫病" +
            ",棘球蚴病" +
            ",丝虫病" +
            ",流行性感冒";
    @Test
    public void writeNeo4j() {
        String startLabel = "疾病";
        String propertyStr = "并发症\t的并发症有哪些疾病";
        String[] properties = propertyStr.split(",");
        workbook = new HSSFWorkbook();//这里也可以设置sheet的Name
        String fileName = "";//
        HSSFSheet sheet = workbook.createSheet(fileName);
        int rows = 0;
        for(String property:properties){
            String[] split = property.split("\t");
            if(split.length<2){
                continue;
            }
            String endLabel = split[0];
            List<Knowlege> data = getData(BaidubceUtil.getAccessToken(), property);
            if(!CollectionUtils.isEmpty(data)) {

                for (Knowlege temp:data) {
                    HSSFRow row = sheet.createRow(rows++);
                    row.createCell(0).setCellValue(temp.getEntity() == null ? "" : temp.getEntity());
                    row.createCell(1).setCellValue(temp.getProperty() == null ? "" : temp.getProperty());
                    row.createCell(2).setCellValue(temp.getValue() == null ? "" : temp.getValue());
                    row.createCell(3).setCellValue(temp.getQuestion() == null ? "" : temp.getQuestion());
                    row.createCell(4).setCellValue(temp.getAnswer() == null ? "" : temp.getAnswer());
                    row.createCell(5).setCellValue(temp.getChunk() == null ? "" : temp.getChunk());
                    row.createCell(6).setCellValue(temp.getRefenrece() == null ? "" : temp.getRefenrece());
                }
                save(fileName);

                for (Knowlege temp:data) {
                    String value = temp.getValue();
                    if(StringUtils.isBlank(value)){
                        continue;
                    }
                    try{
                        JSONArray jsonArray = JSONArray.parseArray(value);
                        BaseEntity startEntity = createNoExists(startLabel, temp.getEntity());
                        long startId = startEntity.getId();
                        List<RelationshipVO> relationshipList = new ArrayList<>();
                        for(int i=0;i<jsonArray.size();i++){
                            String name = jsonArray.getString(i);
                            if(StringUtils.isEmpty(name)){
                                continue;
                            }
                            BaseEntity endEntity =  createNoExists(endLabel, name);
                            Long endId = endEntity.getId();
                            RelationshipVO relationshipVO = new RelationshipVO();
                            relationshipVO.setStartId(startId);
                            relationshipVO.setEndId(endId);
                            relationshipVO.setStartLabel(startLabel);
                            relationshipVO.setEndLabel(endLabel);
                            relationshipVO.setRelationshipType(startLabel+"相关"+endLabel);
                            relationshipList.add(relationshipVO);
                        }
                        if(!CollectionUtils.isEmpty(relationshipList)) {
                            System.out.println( relationshipService.createRelationship(relationshipList));
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    private BaseEntity createNoExists(String labelName, String name) {
        BaseEntity nodeByName = entityService.findNodeByName(labelName, name);
        if(Objects.nonNull(nodeByName)){//节点不存在
            return nodeByName;
        }
        CreateEntityVO createEntity = new CreateEntityVO();
        createEntity.setName(name);
        createEntity.setLabel(labelName);
        return entityService.create(createEntity);
    }

    public static void main(String[] args) {
        String accessToken = BaidubceUtil.getAccessToken();
        String propertyStr = "隔离期\t的隔离期或隔离观察期是多长时间" +
                ",接触者检疫\t的接触者检疫措施有哪些";

        saveExel(propertyStr, accessToken,"3.1传染");
    }

    static HSSFWorkbook workbook;

    static Long totalCount = 0l;
    static Long successCount = 0l;
    static Long failCount = 0l;
    static Long unkonwCount = 0l;

    private static void saveExel(String propertyStr, String accessToken,String fileName) {
        String[] properties = propertyStr.split(",");
        workbook = new HSSFWorkbook();//这里也可以设置sheet的Name
        HSSFSheet sheet = workbook.createSheet(fileName);
        ;//工作表
        int rows = 0;
        for(String property:properties){
            List<Knowlege> data = getData(accessToken, property);
            if(!CollectionUtils.isEmpty(data)) {
                for (Knowlege temp:data) {
                    HSSFRow row = sheet.createRow(rows++);
                    row.createCell(0).setCellValue(temp.getEntity() == null ? "" : temp.getEntity());
                    row.createCell(1).setCellValue(temp.getProperty() == null ? "" : temp.getProperty());
                    row.createCell(2).setCellValue(temp.getValue() == null ? "" : temp.getValue());
                    row.createCell(3).setCellValue(temp.getQuestion() == null ? "" : temp.getQuestion());
                    row.createCell(4).setCellValue(temp.getAnswer() == null ? "" : temp.getAnswer());
                    row.createCell(5).setCellValue(temp.getChunk() == null ? "" : temp.getChunk());
                    row.createCell(6).setCellValue(temp.getRefenrece() == null ? "" : temp.getRefenrece());
                }
            }
            String successRate = String.format("%.2f", Double.valueOf(successCount)/ Double.valueOf(totalCount));
            String unkonwRate = String.format("%.2f", Double.valueOf(unkonwCount)/ Double.valueOf(totalCount));
            sheet.getRow(rows-1).createCell(7).setCellValue(successRate);
            sheet.getRow(rows-1).createCell(8).setCellValue(unkonwRate);
            save(fileName);
            totalCount = 0l;
            successCount = 0l;
            failCount = 0l;
            unkonwCount = 0l;
        }
    }

    private static synchronized void save(String fileName) {
        try {
            fileName="C:\\Users\\17664\\Desktop\\"+fileName+System.currentTimeMillis()+".xlsx";
            //文档输出
            FileOutputStream out = new FileOutputStream(new File(fileName));
            workbook.write(out);
            out.close();
            System.out.println(fileName + "存储完毕");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static List<Knowlege> getData(String accessToken,String property) {
        List<Knowlege> list = new ArrayList<>();
        try {
            BaidubceUtil baidubceUtil = new BaidubceUtil();

            String[] diseases = diseaseStr.split(",");
            String appId= "38392143-27fb-4eeb-94dc-9d1be79157d4";
            for (int i=0;i<diseases.length;i++) {
                try {
                    String disease = diseases[i];
                    disease = disease.trim();
                    String[] split = property.split("\t");
                    if(split.length<2){
                        continue;
                    }
                    String quetionParty = split[1];
                    String relation = split[0];
                    String question = disease + quetionParty + "？";

                    Map<String, String> result = QizhenAssistant.getChatResponse(question, QizhenAssistant.getConversationId(appId),appId);
                    String answer = result.get("answer");
                    String references = result.get("references");
                    String defaultReferences = result.get("defaultReferences");
                    String chatResponse = "";
                    if (!("failed".equals(answer) || answer.contains(QizhenAssistant.noAnswer))) {
                        String format = "你是专门处理医学领域文本的关系抽取专家。你将在指定的文本中抽取其中“"+disease+"的"+relation+"”。\n" +
                                "\n" +
                                "#要求\n" +
                                "1、抽取的结果将以JSON数组的形式呈现。每个抽取的“"+relation+"”高度简洁、高度概括，不要要描述性的文字，文字尽量保持在12个字符以内！\n" +
                                "\n" +
                                "#示例1\n" +
                                "以抽取“主要症状”为例\n" +
                                "文本：\n" +
                                "臭汗症的主要症状有全身或局部多汗且有臭味^[1][3][4]^。\n" +
                                "\n" +
                                "局部性臭汗症多发于大汗腺所在部位，如腋窝、腹股沟、足部、肛周、外阴、脐部及女性乳房下等处，以足部、腋部臭汗症最为多见。腋窝臭汗症俗称狐臭，是一种特殊的刺鼻臭味。足部臭汗症常与足部多汗伴发，有刺鼻的臭味^[1][4]^。\n" +
                                "\n" +
                                "全身性臭汗症为一种与种族有关的生理现象，也可见于卫生习惯不良者，服食某些食物（如葱、蒜、芥末）或某些药物（如麝香）后，在个别人中可产生臭汗^[4]^。\n" +
                                "\n" +
                                "输出：[\"全身或局部多汗且有臭味\",\"局部性臭汗症\",\"腋窝臭汗症\",\"足部臭汗症常\",\"全身性臭汗症\"]\n" +
                                "\n" +
                                "#示例2\n" +
                                "以抽取“是否传染病”为例\n" +
                                "文本：\n" +
                                "**是**^[2][4][6]^。\n" +
                                "\n" +
                                "输出：[\"是\"]\n\n"+
                                "2、没有可抽取的“"+relation+"”，则返回空json数组。\n" +
                                "\n" +
                                "本次抽取的文本如下：\n\n";

                        String zhiling = format+ answer;
                        System.out.println(zhiling);
                        chatResponse = baidubceUtil.getChatResponse(zhiling, accessToken);
                        chatResponse = filte(chatResponse);
                        JSONArray jsonResult = new JSONArray();
                        if(!StringUtils.isBlank(chatResponse)){
                            try{
                                JSONArray jsonArray = JSONArray.parseArray(chatResponse);
                                JSONArray referenceJA = JSONArray.parseArray(references);
                                for(int t=0;t<jsonArray.size();t++){
                                    String name = jsonArray.getString(t);
                                    JSONObject temp = new JSONObject();
                                    temp.put("name", name);
                                    JSONArray tempReferenceJA = new JSONArray();
                                    for(int r=0;r<referenceJA.size();r++){
                                        JSONObject referenceJO = referenceJA.getJSONObject(r);
                                        String content = referenceJO.getString("content");
                                        String contentFilted = content.replaceAll("\\s+", "");
                                        if(contentFilted.contains(name)){
                                            //JSONObject clone = referenceJO.clone();
                                            tempReferenceJA.add(referenceJO.getString("title"));
                                        }
                                    }

                                    String llmQuestion = "你是一个资深的医学专家，请用“是”、“否”和“不确定”回答用户的问题回答。\n" +
                                            "\n" +
                                            "#要求\n" +
                                            "1、只回答“是”、“否”和“不确定”，不要有额外的信息。\n" +
                                            "\n" +
                                            "#示例\n" +
                                            "用户输入：急性上呼吸道感染的治疗药物是否包括“利巴韦林”？\n" +
                                            "输出：是\n\n" +
                                            "请回答："+disease+"的"+relation+"是否包括“"+name+"”？";
                                    String llmAnswer = BaidubceUtil.getChatResponse(llmQuestion, accessToken);
                                    totalCount++;
                                    temp.put("LLM-question",llmQuestion);
                                    temp.put("LLM-answer",llmAnswer);
                                    if(!StringUtils.isEmpty(llmAnswer) && llmAnswer.length()<7) {
                                        if (llmAnswer.contains("是")) {
                                            successCount++;
                                        } else if (llmAnswer.contains("否")) {
                                            failCount++;
                                        }
                                    }else{
                                        unkonwCount++;
                                    }

                                    temp.put("reference", tempReferenceJA);
                                    if(tempReferenceJA.size()<1){
                                        temp.put("defaultReferences", defaultReferences);
                                    }
                                    jsonResult.add(temp);
                                }
                            }catch (Exception e){
                                System.out.println("######"+chatResponse);
                                e.printStackTrace();
                                continue;
                            }
                        }
                        addNode(disease, relation, chatResponse,jsonResult.toJSONString(), answer, question, result, list);
                    }else {
                        addNode(disease, relation, "","", answer, question, result, list);
                    }
                }catch (Exception e){
                    System.out.println("抽取三元组失败！");
                    e.printStackTrace();
                }
            }
            return list;
        } catch (Exception e) {
            System.out.println("未知错误！");
            e.printStackTrace();
        }finally {
            return list;
        }
    }

    private static void addNode(String disease, String property, String chatResponse,String refences, String answer, String question, Map<String, String> result, List<Knowlege> list) {
        Knowlege knowlege = new Knowlege();
        knowlege.setEntity(disease);
        knowlege.setProperty(property);
        knowlege.setValue(chatResponse);
        knowlege.setAnswer(answer);
        knowlege.setQuestion(question);
        knowlege.setChunk(result.get("references"));
        knowlege.setRefenrece(refences);
        list.add(knowlege);
    }

    private static String filte(String chatResponse) {
        if (chatResponse.startsWith("```json")) {
            chatResponse = chatResponse.substring(7);
        }
        if (chatResponse.endsWith("```")) {
            chatResponse = chatResponse.substring(0, chatResponse.length() - 3);
        }
        return chatResponse;
    }
}

