package com.qizhen.healsphere;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.qizhen.healsphere.config.Neo4jUtil;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.neo4j.driver.v1.Session;
import org.neo4j.driver.v1.StatementResult;
import org.neo4j.driver.v1.Values;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.HashMap;
import java.util.Map;

@RunWith(SpringRunner.class)
@ComponentScan(basePackages = {"com.qizhen.healsphere"})
@SpringBootTest
public class DiagbotDisease2Neo4jTest {

    @Autowired
    private Neo4jUtil neo4jUtil;

    private static final String BASE_URL = "http://173.18.12.194:88/api/cdssman/klDisease";
    private static final String AUTHORIZATION_HEADER = "Bearer eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VyX2lkIjozMTUsInVzZXJfbmFtZSI6ImNkc3N6c2s2Iiwic2NvcGUiOlsic2VydmljZSJdLCJleHAiOjE3NDEzMzQ4NzcsImF1dGhvcml0aWVzIjpbIi91c2VyL21pZGlmeVBhc3N3b3JkO0FMTCIsIi91c2VyL2dldFVzZXJPcmdNZW51O0FMTCJdLCJqdGkiOiJiZTFjZTZkOS1iNTgwLTQ1YzItOGU4MC0wMWVkYmIxYjU0NjEiLCJjbGllbnRfaWQiOiJ1YWEtc2VydmljZSJ9.Zfsem6ogTT17-kkD_towsvSwGHL_NOaFWOj6FHVjFVqmabCHl47v9S1DTRcd5f9J46EvrbxxggaPisnYQHw3etW_UmG4NzdwRFaHeTOik1_ropYFK0kUnB9sBkpIO1KxnLYQxIjliUDO7Hv1Bk_egmpvLr-Sbb_yMKTinLwnZt7TPgYFffVLHU46PFXzGW8dbMk9ZSlhCzLQtDEUkcU23lYyrHz-0-gzn0TipFrlbgLHM99JrY7HW5bDqXjjHEQbwnbLM-pgxX7-doR-WdyPzUnF_CCho2G4J6-cdi3ONPJxmjMJ1qrotrl2vOYOrwgJQ-Q6ROX9SzU3ynWqeEbUPQ";
    private final CloseableHttpClient httpClient = HttpClients.createDefault();;
    private final ObjectMapper objectMapper = new ObjectMapper();

    @Test
    public void mergeData() {
        try {
            processAllDiseases();
            System.out.println("数据处理完成");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void processAllDiseases() {
        try {
            for (int page = 1; page <= 221; page++) {
                String pageData = fetchDiseasePage(page);
                JsonNode pageNode = objectMapper.readTree(pageData);
                JsonNode records = pageNode.path("data").path("records");

                for (JsonNode record : records) {
                    String disName = record.path("disName").asText();
                    processDisease(disName);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String fetchDiseasePage(int page) throws Exception {
        String json = String.format("{\"disName\": \"\",\"current\": %d,\"size\": 10}", page);

        HttpPost request = new HttpPost(BASE_URL + "/getDiseasePage");
        request.setHeader("Content-Type", "application/json");
        request.setHeader("Authorization", AUTHORIZATION_HEADER);
        request.setEntity(new StringEntity(json));

        return EntityUtils.toString(httpClient.execute(request).getEntity());
    }

    private void processDisease(String disName) {
        try {
            String detailData = fetchDiseaseDetail(disName);
            JsonNode detailNode = objectMapper.readTree(detailData);
            JsonNode diseaseDetail = detailNode.path("data").path("diseaseDetailDTO");

            try (Session session = neo4jUtil.getSession()) {
                // 创建疾病节点
                createDiseaseNode(session, disName,diseaseDetail);

                // 处理所有相关数据
                JsonNode datas = diseaseDetail.path("datas");
                for (JsonNode data : datas) {
                    String relationshipType = data.path("rname").asText();
                    JsonNode details = data.path("details");

                    for (JsonNode detail : details) {
                        String libName = detail.path("libName").asText();
                        String symptomId = detail.path("id").asText();
                        if (libName != null && !libName.isEmpty()) {
                            createSymptomRelationship(session, disName, relationshipType, libName, symptomId);
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String fetchDiseaseDetail(String disName) throws Exception {
        String json = String.format("{\"disName\": \"%s\"}", disName);

        Map<String,String> map = new HashMap<>();
        map.put("disName",disName);
        json = JSONUtil.toJsonStr(map);

        String body = HttpUtil.createPost(BASE_URL + "/getDetail")
                .header("Content-Type", "application/json")
                .header("Authorization", AUTHORIZATION_HEADER).body(json).execute().body();
        HttpPost request = new HttpPost(BASE_URL + "/getDetail");
        request.setHeader("Content-Type", "application/json");
        request.setHeader("Authorization", AUTHORIZATION_HEADER);
        request.setHeader("User-Agent","PostmanRuntime-ApipostRuntime/1.1.0");
        request.setHeader("Accept","*/*");
        request.setHeader("Accept-Encoding","gzip, deflate, br");
        request.setEntity(new StringEntity(json));

        return EntityUtils.toString(httpClient.execute(request).getEntity());
//        return body;
    }

    private void createDiseaseNode(Session session, String disName, JsonNode diseaseDetail) {
        // 先检查疾病节点是否存在
        String checkQuery = "MATCH (d:`疾病` {name: $name}) RETURN count(d) as count";
        StatementResult result = session.run(checkQuery, Values.parameters("name", disName));
        long count = result.single().get("count").asLong();

        if (count == 0) {
            String query = "CREATE (d:`疾病` {name: $name, source_type: $sourceType, source_id: $sourceId})";
            String sourceId = diseaseDetail.path("sid").asText();
            session.run(query, Values.parameters(
                    "name", disName,
                    "sourceType", "diagbotcloud",
                    "sourceId", sourceId));
        }
    }

    private void createSymptomRelationship(Session session, String disName, String relationshipType, String symptomName, String symptomId) {
        // 从relationshipType中提取标签名称
        String labelName = relationshipType.contains("疾病相关") ?
                relationshipType.substring(relationshipType.indexOf("疾病相关") + 4) : relationshipType;

        // 先检查症状节点是否存在
        String checkQuery = "MATCH (s:" + labelName + " {name: $symptomName}) RETURN count(s) as count";
        StatementResult result = session.run(checkQuery, Values.parameters("symptomName", symptomName));
        long count = result.single().get("count").asLong();

        String query;
        if (count == 0) {
            // 创建新的症状节点
            query = "MATCH (d:Disease {name: $disName}) " +
                    "CREATE (s:" + labelName + " {name: $symptomName, source_type: $sourceType, source_id: $sourceId}) " +
                    "CREATE (d)-[r:%s]->(s)";
        } else {
            // 仅创建关系
            query = "MATCH (d:Disease {name: $disName}), (s:" + labelName + " {name: $symptomName}) " +
                    "CREATE (d)-[r:%s]->(s)";
        }

        // 将关系类型转换为有效的Neo4j关系类型（移除空格和特殊字符）
        String validRelationType = relationshipType.toUpperCase().replaceAll("[^A-Z0-9]+", "_");

        session.run(String.format(query, validRelationType),
                Values.parameters(
                        "disName", disName,
                        "symptomName", symptomName,
                        "sourceType", "diagbotcloud",
                        "sourceId", symptomId));
    }

    public static void main(String[] args) {
        // 1. 定义请求地址和参数
        String url = "http://173.18.12.194:88/api/cdssman/klDisease/getDetail";
//        String jsonBody = "{\"disName\": \"急性上呼吸道感染\"}";
        String token = "Bearer eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VyX2lkIjozMTUsInVzZXJfbmFtZSI6ImNkc3N6c2s2Iiwic2NvcGUiOlsic2VydmljZSJdLCJleHAiOjE3NDEzMzQ4NzcsImF1dGhvcml0aWVzIjpbIi91c2VyL21pZGlmeVBhc3N3b3JkO0FMTCIsIi91c2VyL2dldFVzZXJPcmdNZW51O0FMTCJdLCJqdGkiOiJiZTFjZTZkOS1iNTgwLTQ1YzItOGU4MC0wMWVkYmIxYjU0NjEiLCJjbGllbnRfaWQiOiJ1YWEtc2VydmljZSJ9.Zfsem6ogTT17-kkD_towsvSwGHL_NOaFWOj6FHVjFVqmabCHl47v9S1DTRcd5f9J46EvrbxxggaPisnYQHw3etW_UmG4NzdwRFaHeTOik1_ropYFK0kUnB9sBkpIO1KxnLYQxIjliUDO7Hv1Bk_egmpvLr-Sbb_yMKTinLwnZt7TPgYFffVLHU46PFXzGW8dbMk9ZSlhCzLQtDEUkcU23lYyrHz-0-gzn0TipFrlbgLHM99JrY7HW5bDqXjjHEQbwnbLM-pgxX7-doR-WdyPzUnF_CCho2G4J6-cdi3ONPJxmjMJ1qrotrl2vOYOrwgJQ-Q6ROX9SzU3ynWqeEbUPQ"; // 完整token见原CURL

        JSONObject json = new JSONObject();
        json.set("disName", "急性上呼吸道感染");
        String jsonBody = JSONUtil.toJsonStr(json);

        // 2. 构建并发送POST请求
        HttpResponse response = HttpRequest.post(url)
                .header("Authorization", token)  // 设置鉴权头[4](@ref)[5](@ref)
                .header("Content-Type", "application/json")  // 设置内容类型[4](@ref)[12](@ref)
                .body(jsonBody)  // 设置JSON请求体[4](@ref)[12](@ref)
                .timeout(5000)  // 设置超时时间（可选）
                .execute();

        // 3. 处理响应结果
        if (response.isOk()) {
            System.out.println("响应状态：" + response.getStatus());
            System.out.println("响应内容：" + response.body());
        } else {
            System.out.println("请求失败，状态码：" + response.getStatus());
        }
    }
}
