package com.qizhen.healsphere;

import com.alibaba.fastjson.JSONArray;
import com.qizhen.healsphere.common.ai.BaidubceUtil;
import com.qizhen.healsphere.common.ai.Knowlege;
import com.qizhen.healsphere.repository.neo4j.entity.BaseEntity;
import com.qizhen.healsphere.service.EntityService;
import com.qizhen.healsphere.service.RelationshipService;
import com.qizhen.healsphere.web.vo.CreateEntityVO;
import com.qizhen.healsphere.web.vo.RelationshipVO;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.util.CollectionUtils;

import java.io.*;
import java.util.*;

/**
 * 危险值
 */
@RunWith(SpringRunner.class)
@ComponentScan(basePackages = {"com.qizhen.healsphere.model", "com.qizhen.healsphere.repository"})
@SpringBootTest
public class JianyanWxzTest {
    @Autowired
    RelationshipService relationshipService;
    @Autowired
    EntityService entityService;
    private static int maxCount= 10;
    private static String directoryPath  = "C:\\Users\\17664\\Desktop\\邵逸夫医院爬取数据\\检验";
    static HSSFWorkbook workbook;
    static String startLabel = "检验";
    @Test
    public void writeNeo4j() throws Exception {
        String propertys = "英文名称,商品名称,成份,性状,适应症,不良反应";
        workbook = new HSSFWorkbook();//这里也可以设置sheet的Name
        String accessToken = BaidubceUtil.getAccessToken();
        String[] split = propertys.split(",");
        for(String property:split) {
            HSSFSheet sheet = workbook.createSheet(property);
            List<Knowlege> knowleges = saveExecl(property, accessToken, sheet);
            saveNeo4j(property, knowleges);
        }
    }

    public static void main(String[] args) throws Exception {
        String propertys = "危急值";
        workbook = new HSSFWorkbook();//这里也可以设置sheet的Name
        String accessToken = BaidubceUtil.getAccessToken();
        String[] split = propertys.split(",");
        for(String property:split) {
            HSSFSheet sheet = workbook.createSheet(property);
            saveExecl(property,accessToken, sheet);
        }
    }

    private void saveNeo4j(String property, List<Knowlege> knowleges) {
        for (Knowlege temp: knowleges) {
            String value = temp.getValue();
            if(StringUtils.isBlank(value)){
                continue;
            }
            try{
                JSONArray jsonArray = JSONArray.parseArray(value);
                BaseEntity startEntity = createNoExists(startLabel, temp.getEntity());
                long startId = startEntity.getId();
                List<RelationshipVO> relationshipList = new ArrayList<>();
                for(int i=0;i<jsonArray.size();i++){
                    String name = jsonArray.getString(i);
                    if(StringUtils.isEmpty(name)){
                        continue;
                    }
                    if("商品名称".equals(property)){
                        property = "商品名";
                    }
                    BaseEntity endEntity =  createNoExists(startLabel+ property, name);
                    Long endId = endEntity.getId();
                    RelationshipVO relationshipVO = new RelationshipVO();
                    relationshipVO.setStartId(startId);
                    relationshipVO.setEndId(endId);
                    relationshipVO.setStartLabel(startLabel);
                    relationshipVO.setEndLabel(startLabel+ property);
                    relationshipVO.setRelationshipType(startLabel+"相关"+ property);
                    relationshipList.add(relationshipVO);
                }
                if(!CollectionUtils.isEmpty(relationshipList)) {
                    System.out.println(relationshipService.createRelationship(relationshipList));
                }
            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }

    private BaseEntity createNoExists(String labelName, String name) {
        BaseEntity nodeByName = entityService.findNodeByName(labelName, name);
        if(Objects.nonNull(nodeByName)){//节点不存在
            return nodeByName;
        }
        CreateEntityVO createEntity = new CreateEntityVO();
        createEntity.setName(name);
        createEntity.setLabel(labelName);
        return entityService.create(createEntity);
    }


    public static String readTxtFile(File txtFile) {
        StringBuilder content = new StringBuilder();
        try (BufferedReader reader = new BufferedReader(new FileReader(txtFile))) {
            String line;
            // 逐行读取文件内容
            while ((line = reader.readLine()) != null) {
                // 输出读取到的每一行内容
                content.append(line+"\n");
            }
        } catch (IOException e) {
            System.out.println("读取文件时发生错误: " + e.getMessage());
        }
        return content.toString();
    }

    private static List<Knowlege> saveExecl(String property, String accessToken, HSSFSheet sheet) throws Exception {
        int curCount = 0;
        List<Knowlege> result = new ArrayList<>();
        File directory = new File(directoryPath);
        // 获取目录下的所有文件和子目录
        File[] files = directory.listFiles();

        // 检查files是否为null，避免NullPointerException
        if (files != null) {
            // 循环遍历目录下的每个文件和子目录
            for (File file : files) {
                // 检查文件是否是.txt文件
                if (file.isFile() && file.getName().endsWith(".txt")) {
                    // 读取.txt文件的内容
                    String s = readTxtFile(file);
                    String name = file.getName().split("_")[1].replace(".txt","");
                    if(!s.contains(property)){
                        continue;
                    }
                    Knowlege values = getValues(s, property, accessToken);

                    HSSFRow writeRow = sheet.createRow(curCount);
                    name = (name == null ? "" : name);
                    String answer = (values.getAnswer() == null ? "" : values.getAnswer());
                    String question = (values.getQuestion() == null ? "" : values.getQuestion());
                    writeRow.createCell(0).setCellValue(name);
                    writeRow.createCell(1).setCellValue(answer);
                    writeRow.createCell(2).setCellValue(s == null ? "" : s);
                    writeRow.createCell(3).setCellValue(question == null ? "" : question);
                    Knowlege knowlege = new Knowlege();
                    knowlege.setEntity(name);

                    knowlege.setProperty(property);
                    knowlege.setValue(answer);

                    result.add(knowlege);
                    curCount++;
                    if (maxCount > 0) {
                        if (curCount >= maxCount) {
                            break;
                        }
                    }
                }
            }
        } else {
            System.out.println("指定的目录不存在或无法访问: " + directoryPath);
        }
        save(property);
        return result;
    }

    private static Knowlege getValues(String zhaiyao, String property, String accessToken) {
        if("是否空腹".equals(property)){
            property ="标本要求是否空腹";
        }
        if (StringUtils.isEmpty(zhaiyao)) {
            return null;
        }
        String format = "你是医学检验领域的专家。你将在指定的文本中抽取其中“" + property + "”。\n" +
                "\n" +
                "#要求\n" +
                "1、如果危险值上限有多个值，取值最大的；如果危险值下限有多个值，取值最小的。\n" +
                "2、没有可抽取的“\" + property + \"”，则返回空json数组。\n" +
                "\n" +
                "#示例1\n" +
                "文本：\n" +
                "危急值:\n" +
                "1．危急值下限：平均数：1.87，中位数：2.00\n" +
                "\n" +
                "2．危急值上限：平均数：38.08，中位数：30.00\n" +
                "\n" +
                "输出：[\"危急值下限：1.87\",\"危急值上限：38.08\"]\n\n" +
                "#示例2\n" +
                "文本：\n" +
                "\n" +
                "危急值:\n" +
                "血钾：＜3.0mmol/L或＞6.0mmol/L" +
                "\n" +
                "输出：[\"血钾：＜3.0mmol/L或＞6.0mmol/L\"]\n\n" +
                "本次抽取的文本如下：\n\n";

        String zhiling = format + zhaiyao;
        System.out.println(zhiling);
        String chatResponse = BaidubceUtil.getChatResponse(zhiling, accessToken);
        chatResponse = filte(chatResponse);
        Knowlege knowlege = new Knowlege();
        knowlege.setQuestion(format);
        knowlege.setAnswer(chatResponse);
        return knowlege;
    }

    private static synchronized void save(String fileName) {
        try {
            fileName="C:\\Users\\17664\\Desktop\\"+fileName+System.currentTimeMillis()+".xlsx";
            //文档输出
            FileOutputStream out = new FileOutputStream(new File(fileName));
            workbook.write(out);
            out.close();
            System.out.println(fileName + "存储完毕");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static String filte(String chatResponse) {
        if (chatResponse.startsWith("```json")) {
            chatResponse = chatResponse.substring(7);
        }
        if (chatResponse.endsWith("```")) {
            chatResponse = chatResponse.substring(0, chatResponse.length() - 3);
        }
        return chatResponse;
    }

    private void typeHandle(String property) {
        int curCount = 0;
        File directory = new File(directoryPath);
        // 获取目录下的所有文件和子目录
        File[] files = directory.listFiles();
        List<Knowlege> result = new ArrayList<>();
        // 检查files是否为null，避免NullPointerException
        if (files != null) {
            workbook = new HSSFWorkbook();//这里也可以设置sheet的Name
            HSSFSheet sheet = workbook.createSheet(property);
            // 循环遍历目录下的每个文件和子目录
            for (File file : files) {
                // 检查文件是否是.txt文件
                if (file.isFile() && file.getName().endsWith(".txt")) {
                    // 读取.txt文件的内容
                    String name = file.getName().split("_")[1].replace(".txt","");
                    HSSFRow writeRow = sheet.createRow(curCount);
                    name = (name == null ? "" : name);
                    String answer ="";
                    if("检验项大类".equals(property)) {
                        answer = jianyanMap.get(name);
                        if (StringUtils.isBlank(answer)) {
                            if (name.length() < 3) {
                                answer = "[]";
                            } else {
                                for (String temp : jianyanSplit) {
                                    if (temp.contains(name)) {
                                        answer = temp.split("\t")[0];
                                        break;
                                    }
                                }
                            }
                        }
                    }else if("化验项大类".equals(property)){
                        answer = huayanMap.get(name);
                        if (StringUtils.isBlank(answer)) {
                            if (name.length() < 3) {
                                answer = "[]";
                            } else {
                                for (String temp : huayanSplit) {
                                    if (temp.contains(name)) {
                                        answer = temp.split("\t")[0];
                                        break;
                                    }
                                }
                            }
                        }
                    }else{
                        System.out.println("property值不正确");
                        return;
                    }

                    if(StringUtils.isBlank(answer)){
                        answer = "[]";
                    }else{
                        answer = "[\""+answer+"\"]";
                    }
                    writeRow.createCell(0).setCellValue(name);
                    writeRow.createCell(1).setCellValue(answer);
                    Knowlege knowlege = new Knowlege();
                    knowlege.setEntity(name);

                    knowlege.setProperty(property);
                    knowlege.setValue(answer);

                    result.add(knowlege);
                    curCount++;
                    if (maxCount > 0) {
                        if (curCount >= maxCount) {
                            break;
                        }
                    }
                }
            }
        } else {
            System.out.println("指定的目录不存在或无法访问: " + directoryPath);
        }
        save(property);
        saveNeo4j(property, result);
    }

    static String[] huayanSplit;
    static Map<String,String> huayanMap = new HashMap<String,String>();
    static String[] jianyanSplit;
    static Map<String,String> jianyanMap = new HashMap<String,String>();
    static {
        StringBuilder huayan = new StringBuilder();
        huayan.append("临床血液检验\t11-去氢-血栓烷B<sub>2</sub>测定" +
                ",临床血液检验\t6-酮-前列腺素F1α测定" +
                ",临床血液检验\tα<sub>2</sub>-抗纤溶酶活性测定" +
                ",临床血液检验\tα<sub>2</sub>-抗纤溶酶抗原测定" +
                ",临床血液检验\tα<sub>2</sub>-纤溶酶抑制物活性测定" +
                ",临床血液检验\tα<sub>2</sub>-纤溶酶抑制物抗原测定" +
                ",临床血液检验\tδ-氨基乙酰丙酸" +
                ",临床血液检验\tδ-氨基乙酰丙酸脱氢酶测定" +
                ",临床血液检验\t常用细胞化学染色" +
                ",临床血液检验\t细胞免疫化学染色技术——碱性磷酸酶-抗碱性磷酸酶法" +
                ",临床血液检验\t细胞免疫化学染色技术——链霉素抗生物素-生物素法" +
                ",临床血液检验\t血浆纤维蛋白原片段<sub>1+2</sub>测定——ELISA法" +
                ",临床血液检验\tABO基因分型技术" +
                ",临床血液检验\tABO新生儿溶血病血型血清学检查——患儿血样检查" +
                ",临床血液检验\tABO血型鉴定" +
                ",临床血液检验\tABO亚型检测方法" +
                ",临床血液检验\tDonath-Landsteiner试验" +
                ",临床血液检验\tELISA法定量测定D-二聚体" +
                ",临床血液检验\tHbF酸洗脱试验" +
                ",临床血液检验\tIgG血型同种抗体筛查试验" +
                ",临床血液检验\tIgM血型同种抗体筛查试验" +
                ",临床血液检验\tMN血型定型" +
                ",临床血液检验\tP1Pk血型定型" +
                ",临床血液检验\tProCGlobal试验" +
                ",临床血液检验\tRh新生儿溶血病血型血清学检查——产妇血清学检查" +
                ",临床血液检验\tRh新生儿溶血病血型血清学检查——婴儿血清学检查" +
                ",临床血液检验\tRh血型定型" +
                ",临床血液检验\tSchumm试验" +
                ",临床血液检验\tTEG血小板图实验（ADP途径）" +
                ",临床血液检验\tTEG血小板图实验（花生四烯酸途径）" +
                ",临床血液检验\t阿司匹林耐量试验" +
                ",临床血液检验\t白细胞分类计数" +
                ",临床血液检验\t白细胞计数" +
                ",临床血液检验\t白细胞免疫表型分析" +
                ",临床血液检验\t白细胞形态学检查" +
                ",临床血液检验\t靶形红细胞" +
                ",临床血液检验\t变性珠蛋白小体试验" +
                ",临床血液检验\t丙酮酸激酶活性测定" +
                ",临床血液检验\t丙酮酸激酶荧光斑点试验" +
                ",临床血液检验\t丙型肝炎病毒抗体免疫球蛋白G" +
                ",临床血液检验\t不稳定血红蛋白试验" +
                ",临床血液检验\t产前母体内IgG类抗-A（抗-B）效价检测" +
                ",临床血液检验\t产前筛查试验" +
                ",临床血液检验\t产前筛查血标本采集" +
                ",临床血液检验\t沉淀反应" +
                ",临床血液检验\t出血时间测定" +
                ",临床血液检验\t促红细胞生成素受体测定" +
                ",临床血液检验\t醋酸萘酯酶染色和氟化钠抑制试验" +
                ",临床血液检验\t蛋白C活性测定" +
                ",临床血液检验\t蛋白C抗原测定" +
                ",临床血液检验\t蛋白S抗原测定" +
                ",临床血液检验\t单核细胞计数" +
                ",临床血液检验\t胆色素原脱氨基酶测定" +
                ",临床血液检验\t单特异性抗球蛋白试验" +
                ",临床血液检验\t丁酸萘酯酶染色" +
                ",临床血液检验\t非特异性酯酶染色" +
                ",临床血液检验\t复钙交叉时间测定" +
                ",临床血液检验\t复钙交叉试验" +
                ",临床血液检验\t肝素定量测定" +
                ",临床血液检验\t高铁血红蛋白还原试验" +
                ",临床血液检验\t骨髓病理学检查" +
                ",临床血液检验\t骨髓细胞形态学检验" +
                ",临床血液检验\t骨髓有核细胞计数" +
                ",临床血液检验\t亨氏小体测定" +
                ",临床血液检验\t红细胞变形性测定" +
                ",临床血液检验\t红细胞表面电荷测定" +
                ",临床血液检验\t红细胞比积" +
                ",临床血液检验\t红细胞丙酮酸激酶活性测定" +
                ",临床血液检验\t红细胞卟啉测定" +
                ",临床血液检验\t红细胞沉降率测定" +
                ",临床血液检验\t红细胞电泳时间测定" +
                ",临床血液检验\t红细胞孵育渗透脆性试验" +
                ",临床血液检验\t红细胞交叉配血试验" +
                ",临床血液检验\t红细胞计数" +
                ",临床血液检验\t红细胞聚集性测定" +
                ",临床血液检验\t红细胞聚集指数" +
                ",临床血液检验\t红细胞镰变试验" +
                ",临床血液检验\t红细胞膜蛋白电泳" +
                ",临床血液检验\t红细胞膜蛋白电泳分析" +
                ",临床血液检验\t红细胞膜缺陷的检验——红细胞自身溶血试验" +
                ",临床血液检验\t红细胞膜缺陷的基因检测" +
                ",临床血液检验\t红细胞内游离原卟啉测定" +
                ",临床血液检验\t红细胞平均指数" +
                ",临床血液检验\t红细胞破坏部位测定" +
                ",临床血液检验\t红细胞葡萄糖-6-磷酸脱氢酶活性测定" +
                ",临床血液检验\t红细胞渗透脆性试验" +
                ",临床血液检验\t红细胞寿命检验" +
                ",临床血液检验\t红细胞体积分布宽度" +
                ",临床血液检验\t红细胞形态学检查" +
                ",临床血液检验\t红细胞血型抗体鉴定" +
                ",临床血液检验\t红细胞血型配型测定" +
                ",临床血液检验\t红细胞游离原卟啉测定" +
                ",临床血液检验\t红细胞直径" +
                ",临床血液检验\t红细胞自身溶血试验及其纠正试验" +
                ",临床血液检验\t活化部分凝血活酶时间测定" +
                ",临床血液检验\t活化蛋白C抵抗试验" +
                ",临床血液检验\t活化凝血时间测定" +
                ",临床血液检验\t浆细胞形态学" +
                ",临床血液检验\t间接抗人球蛋白试验" +
                ",临床血液检验\t简易凝血活酶生成试验及纠正试验" +
                ",临床血液检验\t巨大型血小板" +
                ",临床血液检验\t巨核细胞计数" +
                ",临床血液检验\t聚凝胺试验" +
                ",临床血液检验\t抗红细胞自身抗体测定" +
                ",临床血液检验\t抗坏血酸-氰化物试验" +
                ",临床血液检验\t抗碱血红蛋白测定" +
                ",临床血液检验\t抗凝血酶-Ⅲ" +
                ",临床血液检验\t抗凝血酶活性测定" +
                ",临床血液检验\t抗凝血酶抗原测定" +
                ",临床血液检验\t抗球蛋白交叉配血试验" +
                ",临床血液检验\t抗心磷脂抗体测定" +
                ",临床血液检验\t可溶性纤维蛋白单体复合物测定" +
                ",临床血液检验\t可溶性血管内皮细胞蛋白C受体测定" +
                ",临床血液检验\t狼疮抗凝物的检测" +
                ",临床血液检验\t狼疮抗凝物质Lupo试验、Lucor试验" +
                ",临床血液检验\t泪液分泌型免疫球蛋白A" +
                ",临床血液检验\t冷凝集试验" +
                ",临床血液检验\t冷凝集素试验" +
                ",临床血液检验\t冷热溶血试验" +
                ",临床血液检验\t冷热双相溶血试验" +
                ",临床血液检验\t冷溶血试验" +
                ",临床血液检验\t氯乙酸ASD萘酚酯酶(CE)染色" +
                ",临床血液检验\t轮状病毒抗原的检测" +
                ",临床血液检验\t毛细血管脆性试验" +
                ",临床血液检验\t梅毒血清试验" +
                ",临床血液检验\t玫瑰花环试验" +
                ",临床血液检验\t内皮素-1测定" +
                ",临床血液检验\t尿卟啉原脱羧基酶测定" +
                ",临床血液检验\t尿含铁血黄素试验" +
                ",临床血液检验\t尿激酶型纤溶酶原激活剂活性测定" +
                ",临床血液检验\t尿激酶型纤溶酶原激活剂抗原测定" +
                ",临床血液检验\t尿激酶型纤溶酶原激活物活性测定" +
                ",临床血液检验\t尿液1-脱氨基-8-右旋精氨酸加压素测定" +
                ",临床血液检验\t尿液纤维蛋白（原）降解产物" +
                ",临床血液检验\t尿液游离血红蛋白检测" +
                ",临床血液检验\t凝血常规检查" +
                ",临床血液检验\t凝血酶-抗凝血酶复合物测定" +
                ",临床血液检验\t凝血酶测定" +
                ",临床血液检验\t凝血酶调节蛋白活性测定（TM:A）" +
                ",临床血液检验\t凝血酶调节蛋白抗原测定（TM:Ag）" +
                ",临床血液检验\t凝血酶激活的纤溶抑制剂活性测定" +
                ",临床血液检验\t凝血酶时间" +
                ",临床血液检验\t凝血酶时间延长的纠正试验" +
                ",临床血液检验\t凝血酶原片段1＋2测定" +
                ",临床血液检验\t凝血因子Ⅱ、Ⅴ、Ⅶ和Ⅹ的活性测定(一期法)" +
                ",临床血液检验\t凝血因子ⅧA、ⅧB亚基抗原测定" +
                ",临床血液检验\t凝血因子Ⅷ定性试验" +
                ",临床血液检验\t凝血因子Ⅷ抑制物测定" +
                ",临床血液检验\t凝血因子Ⅷ、Ⅸ、Ⅺ、Ⅻ的活性测定(一期法)" +
                ",临床血液检验\t凝血因子Ⅹa抑制试验" +
                ",临床血液检验\t凝血因子Ⅹ促凝活性测定" +
                ",临床血液检验\t凝血因子ⅩⅢ定性试验" +
                ",临床血液检验\t平均红细胞体积" +
                ",临床血液检验\t平均红细胞血红蛋白含量" +
                ",临床血液检验\t平均红细胞血红蛋白浓度" +
                ",临床血液检验\t贫血原因检测" +
                ",临床血液检验\t葡萄糖-6-磷酸脱氢酶活性测定" +
                ",临床血液检验\t葡萄糖-6-磷酸脱氢酶荧光斑点试验" +
                ",临床血液检验\t全血/血浆黏度测定" +
                ",临床血液检验\t去二甲基-血栓素B<sub>2</sub>、11-去氢-血栓素B<sub>2</sub>测定" +
                ",临床血液检验\t热变性试验" +
                ",临床血液检验\t热放散试验" +
                ",临床血液检验\t热溶血试验" +
                ",临床血液检验\t瑞斯托霉素诱导的血小板凝集试验" +
                ",临床血液检验\t乳胶凝集法定性测定D-二聚体" +
                ",临床血液检验\t弱D型鉴定" +
                ",临床血液检验\t伤寒、副伤寒血清凝集试验" +
                ",临床血液检验\t蛇毒因子溶血试验" +
                ",临床血液检验\t释放试验" +
                ",临床血液检验\t嗜碱性点彩红细胞计数" +
                ",临床血液检验\t嗜碱性粒细胞计数" +
                ",临床血液检验\t嗜酸细胞阳离子蛋白的测定" +
                ",临床血液检验\t嗜酸性粒细胞计数" +
                ",临床血液检验\t嗜异性凝集试验" +
                ",临床血液检验\t嗜中性分叶核粒细胞数" +
                ",临床血液检验\t酸化甘油溶血试验" +
                ",临床血液检验\t酸化溶血试验（Ham试验）" +
                ",临床血液检验\t酸性磷酸酶染色" +
                ",临床血液检验\t苏丹黑B染色" +
                ",临床血液检验\t髓过氧化物酶染色" +
                ",临床血液检验\t肽链裂解试验" +
                ",临床血液检验\t碳氧血红蛋白检测" +
                ",临床血液检验\t特殊情况下ABO血型的检测" +
                ",临床血液检验\t体外血栓形成监测" +
                ",临床血液检验\t同种血小板抗体检测" +
                ",临床血液检验\t唾液A，B，H，Le<sup>a</sup>，Le<sup>b</sup>血型物质检测方法" +
                ",临床血液检验\t唾液中ABH血型物质测定" +
                ",临床血液检验\tvWF瑞斯托霉素辅因子测定" +
                ",临床血液检验\t外周血T细胞计数" +
                ",临床血液检验\t外周血细胞涂片形态学检查" +
                ",临床血液检验\t网织红细胞计数" +
                ",临床血液检验\t网织红细胞生成指数" +
                ",临床血液检验\t微量补体溶血敏感试验" +
                ",临床血液检验\t维生素B<sub>12</sub>吸收试验" +
                ",临床血液检验\t温氏法血细胞比容测定" +
                ",临床血液检验\t无创产前检测" +
                ",临床血液检验\t纤溶酶-抗纤溶酶复合物测定" +
                ",临床血液检验\t纤溶酶原活性测定" +
                ",临床血液检验\t纤溶酶原激活抑制剂-1活性测定" +
                ",临床血液检验\t纤溶酶原激活抑制剂-1抗原测定" +
                ",临床血液检验\t纤溶系统检测" +
                ",临床血液检验\t纤维蛋白肽A测定" +
                ",临床血液检验\t纤维蛋白原测定" +
                ",临床血液检验\t纤维蛋白（原）降解产物检测" +
                ",临床血液检验\t献血者健康检查方法" +
                ",临床血液检验\t细胞分子生物学检验" +
                ",临床血液检验\t细胞过碘酸Schiff (糖原)染色" +
                ",临床血液检验\t细胞甲苯胺蓝染色" +
                ",临床血液检验\t细胞铁染色法" +
                ",临床血液检验\t细胞遗传学检验" +
                ",临床血液检验\t新生儿疾病筛查样本采集" +
                ",临床血液检验\t新生儿溶血症筛查" +
                ",临床血液检验\t新生儿血型血清学检查" +
                ",临床血液检验\t新生儿周围血像和骨髓像特点" +
                ",临床血液检验\t锌原卟啉测定" +
                ",临床血液检验\t血沉方程K值" +
                ",临床血液检验\t血管内皮细胞功能试验" +
                ",临床血液检验\t血管性血友病因子抗原测定" +
                ",临床血液检验\t血管性血友病因子瑞斯托霉素辅因子测定" +
                ",临床血液检验\t血红蛋白A2测定" +
                ",临床血液检验\t血红蛋白C检测" +
                ",临床血液检验\t血红蛋白F测定" +
                ",临床血液检验\t血红蛋白F碱变性试验" +
                ",临床血液检验\t血红蛋白F酸洗脱试验" +
                ",临床血液检验\t血红蛋白H包涵体检查" +
                ",临床血液检验\t血红蛋白S测定" +
                ",临床血液检验\t血红蛋白S溶解度试验" +
                ",临床血液检验\t血红蛋白测定" +
                ",临床血液检验\t血红蛋白醋酸纤维膜电泳" +
                ",临床血液检验\t血红蛋白电泳" +
                ",临床血液检验\t血红蛋白尿测定" +
                ",临床血液检验\t血红蛋白区带电泳分析" +
                ",临床血液检验\t血红蛋白热不稳定试验" +
                ",临床血液检验\t血红蛋白溶解度试验" +
                ",临床血液检验\t血红蛋白组分色谱分析" +
                ",临床血液检验\t血红素检验" +
                ",临床血液检验\t血红素结合蛋白测定" +
                ",临床血液检验\t血浆D-二聚体测定" +
                ",临床血液检验\t血浆P-选择素测定" +
                ",临床血液检验\t血浆卟啉测定" +
                ",临床血液检验\t血浆蛋白C和蛋白S检测" +
                ",临床血液检验\t血浆单个凝血因子促凝血活性测定" +
                ",临床血液检验\t血浆肝素浓度测定" +
                ",临床血液检验\t血浆激肽释放酶原测定" +
                ",临床血液检验\t血浆抗凝血酶检测" +
                ",临床血液检验\t血浆硫酸鱼精蛋白副凝固试验" +
                ",临床血液检验\t血浆黏度测定" +
                ",临床血液检验\t血浆凝血活化分子标志物检测" +
                ",临床血液检验\t血浆凝血酶-抗凝血酶复合物测定" +
                ",临床血液检验\t血浆凝血酶原时间测定" +
                ",临床血液检验\t血浆凝血因子抑制物检测" +
                ",临床血液检验\t血浆凝血因子Ⅶ活性测定" +
                ",临床血液检验\t血浆糖蛋白特异性自身抗体测定（改良MAIPA法）" +
                ",临床血液检验\t血浆纤溶酶原检测" +
                ",临床血液检验\t血浆纤溶抑制物检测" +
                ",临床血液检验\t血浆纤维蛋白原测定——凝血酶法" +
                ",临床血液检验\t血浆纤维蛋白原定量" +
                ",临床血液检验\t血浆因子ⅫⅠ定性试验" +
                ",临床血液检验\t血浆游离血红蛋白测定" +
                ",临床血液检验\t血浆组织型纤溶酶原激活物及其抑制物检测" +
                ",临床血液检验\t血浆、尿液去甲基-6-酮-前列腺素F1α测定" +
                ",临床血液检验\t血块或血浆凝块溶解时间测定" +
                ",临床血液检验\t血块收缩试验" +
                ",临床血液检验\t血清补体C3裂解产物" +
                ",临床血液检验\t血清和红细胞叶酸测定" +
                ",临床血液检验\t血清结合珠蛋白测定" +
                ",临床血液检验\t血清内因子阻断抗体测定" +
                ",临床血液检验\t血清黏度测定" +
                ",临床血液检验\t血清凝血酶原时间测定" +
                ",临床血液检验\t血清铁测定" +
                ",临床血液检验\t血清铁四项" +
                ",临床血液检验\t血清维生素B<sub>12</sub>测定" +
                ",临床血液检验\t血清纤维蛋白(原)降解产物定性试验" +
                ",临床血液检验\t血清转铁蛋白受体测定" +
                ",临床血液检验\t血清、尿液2-甲基柠檬酸测定" +
                ",临床血液检验\t血清、尿液胱硫醚测定" +
                ",临床血液检验\t血清、尿液甲基丙二酸测定" +
                ",临床血液检验\t血容量测定" +
                ",临床血液检验\t血栓前体蛋白测定" +
                ",临床血液检验\t血栓三项" +
                ",临床血液检验\t血栓素B<sub>2</sub>检测" +
                ",临床血液检验\t血小板ATP释放试验" +
                ",临床血液检验\t血小板P-选择素测定" +
                ",临床血液检验\t血小板促凝活性测定" +
                ",临床血液检验\t血小板第3因子有效性测定" +
                ",临床血液检验\t血小板第四因子测定" +
                ",临床血液检验\t血小板钙流的测定" +
                ",临床血液检验\t血小板功能试验" +
                ",临床血液检验\t血小板交叉配血试验" +
                ",临床血液检验\t血小板计数" +
                ",临床血液检验\t血小板聚集试验" +
                ",临床血液检验\t血小板颗粒膜糖蛋白CD62P测定" +
                ",临床血液检验\t血小板颗粒膜糖蛋白CD63测定" +
                ",临床血液检验\t血小板膜糖蛋白检测" +
                ",临床血液检验\t血小板黏附试验" +
                ",临床血液检验\t血小板平均容积" +
                ",临床血液检验\t血小板平均体积" +
                ",临床血液检验\t血小板寿命测定" +
                ",临床血液检验\t血小板糖蛋白测定" +
                ",临床血液检验\t血小板糖蛋白Ⅱb、Ⅲa自身抗体测定" +
                ",临床血液检验\t血小板体积分布宽度" +
                ",临床血液检验\t血小板微颗粒测定" +
                ",临床血液检验\t血小板相关补体C4测定" +
                ",临床血液检验\t血小板相关免疫球蛋白测定" +
                ",临床血液检验\t血小板形态学检查" +
                ",临床血液检验\t血小板血型检测" +
                ",临床血液检验\t血小板压积" +
                ",临床血液检验\t血小板自身抗体测定" +
                ",临床血液检验\t血细胞比容测定" +
                ",临床血液检验\t血细胞膜CD55/CD59表型分析" +
                ",临床血液检验\t血细胞组织化学染色" +
                ",临床血液检验\t血型的基因分型" +
                ",临床血液检验\t血型鉴定" +
                ",临床血液检验\t血型抗体效价测定" +
                ",临床血液检验\t血型血清学——抗球蛋白试验" +
                ",临床血液检验\t血型血清——吸收试验" +
                ",临床血液检验\t血液常规检查" +
                ",临床血液检验\t血液分析仪检验技术" +
                ",临床血液检验\t血液流变学检查" +
                ",临床血液检验\t血液酸碱度测定" +
                ",临床血液检验\t血液粘度测定" +
                ",临床血液检验\t血液肿瘤分子遗传学检测" +
                ",临床血液检验\t盐水介质交叉配血试验" +
                ",临床血液检验\t叶酸/维生素B<sub>12</sub>治疗性试验" +
                ",临床血液检验\t叶酸吸收试验" +
                ",临床血液检验\t异丙醇沉淀试验" +
                ",临床血液检验\t异丙醇试验" +
                ",临床血液检验\t异常红细胞形态检查" +
                ",临床血液检验\t乙醇凝胶试验" +
                ",临床血液检验\t乙醚放散法" +
                ",临床血液检验\t婴ABO血型不合新生儿溶血病——孕妇IgG抗A（B）检测" +
                ",临床血液检验\t意外抗体筛查与鉴定" +
                ",临床血液检验\t乙酰胆碱酯酶测定" +
                ",临床血液检验\t乙型肝炎核心抗原" +
                ",临床血液检验\t异型淋巴细胞测定" +
                ",临床血液检验\t有核红细胞测定" +
                ",临床血液检验\t游离肝素时间测定" +
                ",临床血液检验\t游离试验" +
                ",临床血液检验\t优球蛋白溶解时间" +
                ",临床血液检验\t造血干细胞计数" +
                ",临床血液检验\t造血祖细胞培养" +
                ",临床血液检验\t早孕期母外周血筛查试验" +
                ",临床血液检验\t正常血浆APTT纠正试验" +
                ",临床血液检验\t正常血浆PT纠正试验" +
                ",临床血液检验\t蔗糖溶血试验" +
                ",临床血液检验\t蔗糖水溶血试验" +
                ",临床血液检验\t直接抗人球蛋白试验" +
                ",临床血液检验\t中性杆状核粒细胞" +
                ",临床血液检验\t中性粒细胞测定" +
                ",临床血液检验\t中性粒细胞的分离与功能检测" +
                ",临床血液检验\t中性粒细胞碱性磷酸酶染色" +
                ",临床血液检验\t中性粒细胞计数" +
                ",临床血液检验\t中孕期母外周血筛查试验" +
                ",临床血液检验\t转铁蛋白饱和度测定" +
                ",临床血液检验\t珠蛋白基因突变检测" +
                ",临床血液检验\t珠蛋白肽链分析" +
                ",临床血液检验\t珠蛋白肽链聚丙烯酰胺凝胶电泳分析" +
                ",临床血液检验\t柱凝集法交叉配血试验" +
                ",临床血液检验\t自身红细胞抗体" +
                ",临床血液检验\t总血红蛋白测定" +
                ",临床血液检验\t组织型纤溶酶原激活剂活性测定" +
                ",临床血液检验\t组织型纤溶酶原激活剂抗原测定" +
                ",临床血液检验\t组织因子促凝活性测定" +
                ",临床血液检验\t组织因子途径抑制物活性测定" +
                ",临床体液检验\t1小时尿沉渣计数" +
                ",临床体液检验\t24小时尿卟啉测定" +
                ",临床体液检验\t24小时尿蛋白定量" +
                ",临床体液检验\t24小时尿量测定" +
                ",临床体液检验\t24小时尿磷测定" +
                ",临床体液检验\t24小时尿锰测定" +
                ",临床体液检验\t24小时尿钼测定" +
                ",临床体液检验\t24小时尿铜测定" +
                ",临床体液检验\t24小时尿硒测定" +
                ",临床体液检验\t24小时尿液儿茶酚胺测定" +
                ",临床体液检验\t24小时尿液环磷酸腺苷" +
                ",临床体液检验\t24小时尿液环磷酸腺苷测定" +
                ",临床体液检验\t24小时尿液肌酸测定" +
                ",临床体液检验\t24小时尿液生长激素测定" +
                ",临床体液检验\t浆膜腔积液一般检验" +
                ",临床体液检验\t（神经）梅毒脑脊液检测" +
                ",临床体液检验\t艾迪斯计数法" +
                ",临床体液检验\t白细胞酯酶检验" +
                ",临床体液检验\t板层小体计数" +
                ",临床体液检验\t鼻腔分泌物的生化检查" +
                ",临床体液检验\t不孕不育的免疫学检查" +
                ",临床体液检验\t仓鼠卵-精子穿透试验" +
                ",临床体液检验\t虫卵计数法" +
                ",临床体液检验\t穿刺液常规检查" +
                ",临床体液检验\t穿刺液培养" +
                ",临床体液检验\t促胰酶素-促胰液素试验" +
                ",临床体液检验\t蛋白质定量测定" +
                ",临床体液检验\t胆色素原测定" +
                ",临床体液检验\t定量法尿液有形成分检查" +
                ",临床体液检验\t定性法尿液有形成分检查" +
                ",临床体液检验\t粪便白细胞" +
                ",临床体液检验\t粪便标本采集" +
                ",临床体液检验\t粪便卟啉测定" +
                ",临床体液检验\t粪便常规检查" +
                ",临床体液检验\t粪便胆红素测定" +
                ",临床体液检验\t粪便淀粉颗粒" +
                ",临床体液检验\t粪便红细胞" +
                ",临床体液检验\t粪便酵母菌" +
                ",临床体液检验\t粪便肌肉纤维的定性试验" +
                ",临床体液检验\t粪便寄生虫检查" +
                ",临床体液检验\t粪便理学检查" +
                ",临床体液检验\t粪便黏液" +
                ",临床体液检验\t粪便脓液" +
                ",临床体液检验\t粪便pH测定" +
                ",临床体液检验\t粪便吞噬细胞" +
                ",临床体液检验\t粪便显微镜检查" +
                ",临床体液检验\t粪便细菌培养" +
                ",临床体液检验\t粪便颜色" +
                ",临床体液检验\t粪便胰弹性蛋白酶检测" +
                ",临床体液检验\t粪便隐血试验" +
                ",临床体液检验\t粪便有形成分显微镜检查" +
                ",临床体液检验\t粪便脂肪检查" +
                ",临床体液检验\t粪便脂肪平衡试验" +
                ",临床体液检验\t粪便中性脂肪测定" +
                ",临床体液检验\t粪胆素测定" +
                ",临床体液检验\t粪胆素原测定" +
                ",临床体液检验\t粪食物残渣" +
                ",临床体液检验\t粪脂肪苏丹染色" +
                ",临床体液检验\t附睾功能检验" +
                ",临床体液检验\t腹水常规检查" +
                ",临床体液检验\t甘氨酰脯氨酸二肽氨基肽酶测定" +
                ",临床体液检验\t宫颈5%醋酸染色肉眼观察" +
                ",临床体液检验\t宫颈5%复方碘染色肉眼观察" +
                ",临床体液检验\t宫颈分泌物淋病奈瑟菌培养" +
                ",临床体液检验\t宫颈分泌物沙眼衣原体快速抗原检测" +
                ",临床体液检验\t宫颈分泌物涂片革兰染色显微镜检查" +
                ",临床体液检验\t宫颈黏液检验" +
                ",临床体液检验\t宫腔细菌培养" +
                ",临床体液检验\t关节腔积液常规检查" +
                ",临床体液检验\t汗液电解质检测" +
                ",临床体液检验\t汗液氯化物测定" +
                ",临床体液检验\t滑膜液标本采集与一般性状检验" +
                ",临床体液检验\t滑膜液补体检测" +
                ",临床体液检验\t滑膜液蛋白质检测" +
                ",临床体液检验\t滑膜液结晶检查" +
                ",临床体液检验\t滑膜液抗核抗体检测" +
                ",临床体液检验\t滑膜液类风湿因子检测" +
                ",临床体液检验\t滑膜液黏蛋白凝块形成试验" +
                ",临床体液检验\t滑膜液葡萄糖检测" +
                ",临床体液检验\t滑膜液乳酸检测" +
                ",临床体液检验\t混合抗球蛋白反应试验" +
                ",临床体液检验\t碱负荷试验" +
                ",临床体液检验\t浆膜腔积液C-反应蛋白测定" +
                ",临床体液检验\t浆膜腔积液癌胚抗原检测" +
                ",临床体液检验\t浆膜腔积液病原生物学检查" +
                ",临床体液检验\t浆膜腔积液采集与理学检验" +
                ",临床体液检验\t浆膜腔积液淀粉酶检测" +
                ",临床体液检验\t浆膜腔积液流式细胞分析技术" +
                ",临床体液检验\t浆膜腔积液黏蛋白定性试验" +
                ",临床体液检验\t浆膜腔积液葡萄糖检测" +
                ",临床体液检验\t浆膜腔积液溶菌酶检测" +
                ",临床体液检验\t浆膜腔积液乳酸脱氢酶检测" +
                ",临床体液检验\t浆膜腔积液酸碱度测定" +
                ",临床体液检验\t浆膜腔积液腺苷脱氨酶检测" +
                ",临床体液检验\t浆膜腔积液纤维蛋白（原）降解产物测定" +
                ",临床体液检验\t浆膜腔积液显微镜检查" +
                ",临床体液检验\t浆膜腔积液纤维连接蛋白测定" +
                ",临床体液检验\t浆膜腔积液细胞学检查" +
                ",临床体液检验\t浆膜腔积液有形成分检查" +
                ",临床体液检验\t浆膜腔积液脂蛋白电泳" +
                ",临床体液检验\t浆膜腔积液总蛋白定量检测" +
                ",临床体液检验\t精浆的生物化学检查" +
                ",临床体液检验\t精浆果糖测定" +
                ",临床体液检验\t精浆镁测定" +
                ",临床体液检验\t精浆免疫球蛋白测定" +
                ",临床体液检验\t精浆免疫抑制物质测定" +
                ",临床体液检验\t精浆柠檬酸测定" +
                ",临床体液检验\t精浆酸性磷酸酶测定" +
                ",临床体液检验\t精浆锌测定" +
                ",临床体液检验\t精浆中性α-葡萄糖苷酶测定" +
                ",临床体液检验\t精浆γ-GT测定" +
                ",临床体液检验\t精浆γ-谷氨酰基转肽酶测定" +
                ",临床体液检验\t精囊腺功能检验" +
                ",临床体液检验\t精液白细胞检验" +
                ",临床体液检验\t精液标本收集与精液理学检验" +
                ",临床体液检验\t精液常规检查" +
                ",临床体液检验\t精液弹性蛋白酶检验" +
                ",临床体液检验\t精液凋亡细胞检验" +
                ",临床体液检验\t精液pH测定" +
                ",临床体液检验\t精液乳酸脱氢酶X测定" +
                ",临床体液检验\t精液液化时间" +
                ",临床体液检验\t精子-宫颈黏液相互作用试验" +
                ",临床体液检验\t精子穿卵试验" +
                ",临床体液检验\t精子顶体反应及顶体酶检验" +
                ",临床体液检验\t精子顶体酶活性测定" +
                ",临床体液检验\t精子顶体完整率检验" +
                ",临床体液检验\t精子核检验" +
                ",临床体液检验\t精子活动力测定" +
                ",临床体液检验\t精子活动率测定" +
                ",临床体液检验\t精子计数" +
                ",临床体液检验\t精子膜功能检验" +
                ",临床体液检验\t精子凝集现象检查和有形成分检查" +
                ",临床体液检验\t精子爬高试验" +
                ",临床体液检验\t精子尾部低渗肿胀试验" +
                ",临床体液检验\t精子形态观察" +
                ",临床体液检验\t精子运动速度" +
                ",临床体液检验\t禁水加压素试验" +
                ",临床体液检验\t禁水试验" +
                ",临床体液检验\t抗精子抗体免疫珠试验" +
                ",临床体液检验\t泪液pH检测" +
                ",临床体液检验\t泪液溶菌酶检测" +
                ",临床体液检验\t泪液乳铁蛋白检测" +
                ",临床体液检验\t流式尿沉渣分析" +
                ",临床体液检验\t李文生试验" +
                ",临床体液检验\t氯化铵负荷试验" +
                ",临床体液检验\t卵磷脂小体测定" +
                ",临床体液检验\t卵磷脂／鞘磷脂比值测定" +
                ",临床体液检验\t脑脊液5-羟色胺测定" +
                ",临床体液检验\t脑脊液5-羟吲哚乙酸测定" +
                ",临床体液检验\t脑脊液C-反应蛋白测定" +
                ",临床体液检验\t脑脊液IgA测定" +
                ",临床体液检验\t脑脊液IgG测定" +
                ",临床体液检验\t脑脊液IgM测定" +
                ",临床体液检验\t脑脊液Tau蛋白测定" +
                ",临床体液检验\t脑脊液氨基酸总量测定" +
                ",临床体液检验\t脑脊液氨基酸组分" +
                ",临床体液检验\t脑脊液丙氨酸氨基转移酶测定" +
                ",临床体液检验\t脑脊液病原体形态学检查" +
                ",临床体液检验\t脑脊液常规检查" +
                ",临床体液检验\t脑脊液蛋白质定性试验" +
                ",临床体液检验\t脑脊液的寡克隆电泳检测" +
                ",临床体液检验\t脑脊液电泳" +
                ",临床体液检验\t脑脊液多巴胺测定" +
                ",临床体液检验\t脑脊液甘氨酸测定" +
                ",临床体液检验\t脑脊液高香草酸测定" +
                ",临床体液检验\t脑脊液假胆碱酯酶测定" +
                ",临床体液检验\t脑脊液肌酸激酶测定" +
                ",临床体液检验\t脑脊液理学检查" +
                ",临床体液检验\t脑脊液氯化物测定" +
                ",临床体液检验\t脑脊液免疫球蛋白 G和白蛋白比率测定" +
                ",临床体液检验\t脑脊液葡萄糖测定" +
                ",临床体液检验\t脑脊液醛缩酶测定" +
                ",临床体液检验\t脑脊液乳酸检测" +
                ",临床体液检验\t脑脊液乳酸脱氢酶及同工酶测定" +
                ",临床体液检验\t脑脊液色氨酸试验" +
                ",临床体液检验\t脑脊液酸性磷酸酶测定" +
                ",临床体液检验\t脑脊液髓鞘碱性蛋白测定" +
                ",临床体液检验\t脑脊液特异性梅毒螺旋体抗体" +
                ",临床体液检验\t脑脊液天冬氨酸氨基转移酶测定" +
                ",临床体液检验\t脑脊液腺苷酸激酶测定" +
                ",临床体液检验\t脑脊液腺苷脱氨酶测定" +
                ",临床体液检验\t脑脊液显微镜检查" +
                ",临床体液检验\t脑脊液细胞计数" +
                ",临床体液检验\t脑脊液细菌培养标本采集与处理" +
                ",临床体液检验\t脑脊液隐血试验" +
                ",临床体液检验\t脑脊液乙酰胆碱测定" +
                ",临床体液检验\t脑脊液乙酰胆碱酯酶测定" +
                ",临床体液检验\t脑脊液有形成分检查——细胞学检查" +
                ",临床体液检验\t脑脊液总蛋白" +
                ",临床体液检验\t脑脊液β-葡萄糖苷酶测定" +
                ",临床体液检验\t尿17-酮类固醇测定" +
                ",临床体液检验\t尿T-H糖蛋白测定" +
                ",临床体液检验\t尿苯丙酮定性试验" +
                ",临床体液检验\t尿本周蛋白定性试验" +
                ",临床体液检验\t尿沉渣白细胞测定" +
                ",临床体液检验\t尿沉渣定量检查" +
                ",临床体液检验\t尿沉渣红细胞测定" +
                ",临床体液检验\t尿沉渣上皮细胞和异形细胞" +
                ",临床体液检验\t尿沉渣显微镜检查" +
                ",临床体液检验\t尿蛋白定性试验" +
                ",临床体液检验\t尿胆红素定性试验" +
                ",临床体液检验\t尿胆原定性试验" +
                ",临床体液检验\t尿胱氨酸定性试验" +
                ",临床体液检验\t尿含铁血黄素定性试验" +
                ",临床体液检验\t尿红细胞位相留取" +
                ",临床体液检验\t尿红细胞形态分析" +
                ",临床体液检验\t尿结石成份分析" +
                ",临床体液检验\t尿肌红蛋白定性试验" +
                ",临床体液检验\t尿酪氨酸定性试验" +
                ",临床体液检验\t尿量测定" +
                ",临床体液检验\t尿流率测定" +
                ",临床体液检验\t尿氯测定" +
                ",临床体液检验\t尿浓缩试验" +
                ",临床体液检验\t尿培养" +
                ",临床体液检验\t尿葡萄糖定性试验" +
                ",临床体液检验\t尿妊娠试验" +
                ",临床体液检验\t尿乳糜定性试验" +
                ",临床体液检验\t尿三杯试验" +
                ",临床体液检验\t尿三氯化铁试验" +
                ",临床体液检验\t尿渗量测定" +
                ",临床体液检验\t尿渗量和自由水清除率测定" +
                ",临床体液检验\t尿酸化功能测定" +
                ",临床体液检验\t尿酮体定性试验" +
                ",临床体液检验\t尿同型半胱氨酸测定" +
                ",临床体液检验\t尿微量白蛋白检测" +
                ",临床体液检验\t尿液5-羟吲哚乙酸测定" +
                ",临床体液检验\t尿液N-乙酰-β-D-氨基葡萄糖苷酶测定" +
                ",临床体液检验\t尿液N’-甲基尼克酰胺测定" +
                ",临床体液检验\t尿液SDS-PAGE" +
                ",临床体液检验\t尿液Tamm-Horsfall蛋白测定" +
                ",临床体液检验\t尿液氨基酸测定" +
                ",临床体液检验\t尿液白蛋白测定" +
                ",临床体液检验\t尿液白细胞" +
                ",临床体液检验\t尿液白细胞酯酶测定" +
                ",临床体液检验\t尿液半乳糖苷酶" +
                ",临床体液检验\t尿液比密测定" +
                ",临床体液检验\t尿液丙氨酸氨基肽酶测定" +
                ",临床体液检验\t尿液比重测定" +
                ",临床体液检验\t尿液草酸测定" +
                ",临床体液检验\t尿液常规检查" +
                ",临床体液检验\t尿液成体干细胞" +
                ",临床体液检验\t尿液蛋白选择性指数" +
                ",临床体液检验\t尿液蛋白质测定" +
                ",临床体液检验\t尿液碘测定" +
                ",临床体液检验\t尿液淀粉酶测定" +
                ",临床体液检验\t尿液分泌型IgA检测" +
                ",临床体液检验\t尿液氟测定" +
                ",临床体液检验\t尿液钙测定" +
                ",临床体液检验\t尿液干化学分析" +
                ",临床体液检验\t尿液铬测定" +
                ",临床体液检验\t尿液胱氨酸测定" +
                ",临床体液检验\t尿液胱抑素C检测" +
                ",临床体液检验\t尿液钴测定" +
                ",临床体液检验\t尿液还原糖试验" +
                ",临床体液检验\t尿液红细胞" +
                ",临床体液检验\t尿液钾测定" +
                ",临床体液检验\t尿液碱性磷酸酶测定" +
                ",临床体液检验\t尿液结晶测定" +
                ",临床体液检验\t尿液来源的诱导多潜能干细胞" +
                ",临床体液检验\t尿液亮氨酸氨基肽酶测定" +
                ",临床体液检验\t尿液铝测定" +
                ",临床体液检验\t尿液氯化物测定" +
                ",临床体液检验\t尿液镁测定" +
                ",临床体液检验\t尿液锰测定" +
                ",临床体液检验\t尿液免疫球蛋白测定" +
                ",临床体液检验\t尿液钼测定" +
                ",临床体液检验\t尿液钠测定" +
                ",临床体液检验\t尿液尿胆原测定" +
                ",临床体液检验\t尿液镍测定" +
                ",临床体液检验\t尿液柠檬酸测定" +
                ",临床体液检验\t尿液浓缩稀释试验" +
                ",临床体液检验\t尿液葡萄糖测定" +
                ",临床体液检验\t尿液溶菌酶测定" +
                ",临床体液检验\t尿液乳清酸测定" +
                ",临床体液检验\t尿液乳酸脱氢酶测定" +
                ",临床体液检验\t尿液上皮细胞" +
                ",临床体液检验\t尿液酸碱度测定" +
                ",临床体液检验\t尿液铜测定" +
                ",临床体液检验\t尿液维生素B<sub>1</sub>测定" +
                ",临床体液检验\t尿液维生素C测定" +
                ",临床体液检验\t尿液无机磷测定" +
                ",临床体液检验\t尿液酰基肉碱测定" +
                ",临床体液检验\t尿液细胞学检查" +
                ",临床体液检验\t尿液硒测定" +
                ",临床体液检验\t尿液细菌培养" +
                ",临床体液检验\t尿液细菌学检查" +
                ",临床体液检验\t尿液锌测定" +
                ",临床体液检验\t尿液颜色和透明度测定" +
                ",临床体液检验\t尿液亚硝酸盐测定" +
                ",临床体液检验\t尿液游离免疫球蛋白轻链测定" +
                ",临床体液检验\t尿液有形成分分析仪检验" +
                ",临床体液检验\t尿液有形成分检查的推荐参考方法" +
                ",临床体液检验\t尿液有形成分显微镜检查方法" +
                ",临床体液检验\t尿液真菌培养" +
                ",临床体液检验\t尿液中病毒感染细胞及其包涵体" +
                ",临床体液检验\t尿液转铁蛋白测定" +
                ",临床体液检验\t尿液总蛋白测定" +
                ",临床体液检验\t尿液α<sub>2</sub>-巨球蛋白测定" +
                ",临床体液检验\t尿液β-葡萄糖醛酸苷酶测定" +
                ",临床体液检验\t尿液β<sub>2</sub>-微球蛋白测定" +
                ",临床体液检验\t尿液γ-谷氨酰基转肽酶测定" +
                ",临床体液检验\t尿隐血（血红蛋白）定性测定" +
                ",临床体液检验\t尿有机酸测定" +
                ",临床体液检验\t尿游离皮质醇" +
                ",临床体液检验\t尿β<sub>2</sub>-微球蛋白" +
                ",临床体液检验\t尿γ-谷氨酰基转移酶测定" +
                ",临床体液检验\t尿Ⅰ型胶原交联氨基末端肽测定" +
                ",临床体液检验\t泡沫稳定性指数" +
                ",临床体液检验\t前列腺功能检验" +
                ",临床体液检验\t前列腺液LDH5／LDH1检验" +
                ",临床体液检验\t前列腺液标本收集及理学检验" +
                ",临床体液检验\t前列腺液高敏C反应蛋白" +
                ",临床体液检验\t前列腺液检查" +
                ",临床体液检验\t前列腺液柠檬酸检测" +
                ",临床体液检验\t前列腺液酸碱度测定" +
                ",临床体液检验\t前列腺液酸性磷酸酶测定" +
                ",临床体液检验\t前列腺液锌检测" +
                ",临床体液检验\t前列腺液有形成分分析" +
                ",临床体液检验\t全自动尿10项干化学常规分析（URISYS 2400型）" +
                ",临床体液检验\t染色法尿液有形成分检查" +
                ",临床体液检验\t绒毛采集" +
                ",临床体液检验\t乳糜尿检查" +
                ",临床体液检验\t乳头溢液的细胞学检查" +
                ",临床体液检验\t生精细胞检验" +
                ",临床体液检验\t肾功能检查" +
                ",临床体液检验\t生长抑素测定" +
                ",临床体液检验\t生殖内分泌激素检验" +
                ",临床体液检验\t肾血流量测定" +
                ",临床体液检验\t十二指肠液胆红素测定" +
                ",临床体液检验\t十二指肠液胆汁酸测定" +
                ",临床体液检验\t十二指肠液胰蛋白酶检测" +
                ",临床体液检验\t十二指肠液γ-谷氨酰基转肽酶测定" +
                ",临床体液检验\t十二指肠引流液常规检查" +
                ",临床体液检验\t酸负荷试验" +
                ",临床体液检验\t随机尿儿茶酚胺测定" +
                ",临床体液检验\t随机尿肌酐测定" +
                ",临床体液检验\t痰培养" +
                ",临床体液检验\t碳酸氢离子重吸收排泌试验" +
                ",临床体液检验\t痰液标本的采集与及理学检查" +
                ",临床体液检验\t痰液分泌型IgA检测" +
                ",临床体液检验\t痰液检查" +
                ",临床体液检验\t痰液乳酸脱氢酶测定" +
                ",临床体液检验\t痰液细胞涂片制作" +
                ",临床体液检验\t痰液细胞学检查" +
                ",临床体液检验\t痰液有形成分分析" +
                ",临床体液检验\t唾液蛋白质测定" +
                ",临床体液检验\t唾液淀粉酶测定" +
                ",临床体液检验\t唾液电解质测定" +
                ",临床体液检验\t唾液毒物分析" +
                ",临床体液检验\t唾液非蛋白氮类物质测定" +
                ",临床体液检验\t唾液激素测定" +
                ",临床体液检验\t唾液黏蛋白测定" +
                ",临床体液检验\t唾液溶菌酶测定" +
                ",临床体液检验\t唾液有机成分" +
                ",临床体液检验\t唾液游离皮质醇测定" +
                ",临床体液检验\t唾液治疗药物监测" +
                ",临床体液检验\t外阴阴道假丝酵母菌病检验" +
                ",临床体液检验\t微量白蛋白尿测定" +
                ",临床体液检验\t胃酸分泌量检测" +
                ",临床体液检验\t胃液常规检查" +
                ",临床体液检验\t胃液尿素检测" +
                ",临床体液检验\t胃液气味" +
                ",临床体液检验\t胃液乳酸检测" +
                ",临床体液检验\t胃液锌检测" +
                ",临床体液检验\t胃液隐血试验" +
                ",临床体液检验\t无精子因子检验" +
                ",临床体液检验\t线索细胞检查（湿片法）" +
                ",临床体液检验\t线索细胞检查（阴道菌群革兰氏染色法）" +
                ",临床体液检验\t细菌性阴道病检验" +
                ",临床体液检验\t胸腔积液微生物学检查" +
                ",临床体液检验\t胸腔积液β<sub>2</sub>-微球蛋白" +
                ",临床体液检验\t胸水常规检查" +
                ",临床体液检验\t胸（腹）水及心包积液常规检查" +
                ",临床体液检验\t血浆、尿液渗透压测定" +
                ",临床体液检验\t血清、24小时尿液尿素氮测定" +
                ",临床体液检验\t羊水氨基酸测定" +
                ",临床体液检验\t羊水采集" +
                ",临床体液检验\t羊水促黄体生成素测定" +
                ",临床体液检验\t羊水催乳素测定" +
                ",临床体液检验\t羊水促肾上腺皮质激素测定" +
                ",临床体液检验\t羊水胆红素测定" +
                ",临床体液检验\t羊水钙测定" +
                ",临床体液检验\t羊水睾酮测定" +
                ",临床体液检验\t羊水钾测定" +
                ",临床体液检验\t羊水碱性磷酸酶测定" +
                ",临床体液检验\t羊水甲状腺素测定" +
                ",临床体液检验\t羊水肌酐测定" +
                ",临床体液检验\t羊水磷脂酰甘油测定" +
                ",临床体液检验\t羊水氯测定" +
                ",临床体液检验\t羊水卵磷脂测定" +
                ",临床体液检验\t羊水卵泡刺激素测定" +
                ",临床体液检验\t羊水镁测定" +
                ",临床体液检验\t羊水钠测定" +
                ",临床体液检验\t羊水皮质醇测定" +
                ",临床体液检验\t羊水前列腺素E测定" +
                ",临床体液检验\t羊水前列腺素F测定" +
                ",临床体液检验\t羊水人绒毛膜促性腺激素测定" +
                ",临床体液检验\t羊水人胎盘生乳素测定" +
                ",临床体液检验\t羊水乳铁蛋白测定" +
                ",临床体液检验\t羊水生长激素测定" +
                ",临床体液检验\t羊水铁测定" +
                ",临床体液检验\t羊水铁蛋白测定" +
                ",临床体液检验\t羊水维生素A测定" +
                ",临床体液检验\t羊水维生素B<sub>12</sub>测定" +
                ",临床体液检验\t羊水维生素D测定" +
                ",临床体液检验\t羊水维生素E测定" +
                ",临床体液检验\t羊水血型物质测定" +
                ",临床体液检验\t羊水胰岛素测定<b></b>" +
                ",临床体液检验\t羊水乙酰胆碱酯酶测定" +
                ",临床体液检验\t羊水孕酮测定" +
                ",临床体液检验\t羊水转铁蛋白测定" +
                ",临床体液检验\t羊水总蛋白测定" +
                ",临床体液检验\t羊水γ-谷氨酰基转移酶测定" +
                ",临床体液检验\t咽拭子细菌培养及药敏试验" +
                ",临床体液检验\t阴道分泌物胺试验" +
                ",临床体液检验\t阴道分泌物的采集" +
                ",临床体液检验\t阴道分泌物干化学酶学检验" +
                ",临床体液检验\t阴道分泌物检查" +
                ",临床体液检验\t阴道分泌物pH测定" +
                ",临床体液检验\t阴道分泌物湿片显微镜检查" +
                ",临床体液检验\t阴道分泌物涂片革兰染色显微镜检查" +
                ",临床体液检验\t阴道过氧化氢试验" +
                ",临床体液检验\t阴道活检" +
                ",临床体液检验\t阴道毛滴虫病检验" +
                ",临床体液检验\t阴道清洁度评估" +
                ",临床体液检验\t阴道脱落细胞检验" +
                ",临床体液检验\t阴道唾液酸酐酶检验" +
                ",临床体液检验\t阴道真菌检验" +
                ",临床体液检验\t阴道支原体感染检验" +
                ",临床体液检验\t荧光偏振试验" +
                ",临床体液检验\t支气管肺泡灌洗液检验" +
                ",临床体液检验\t昼夜尿比密试验和3h尿比密试验" +
                ",临床化学检验\t11-脱氧皮质醇测定" +
                ",临床化学检验\t11β-羟化酶测定" +
                ",临床化学检验\t17-羟孕酮测定" +
                ",临床化学检验\t17α-羟化酶测定" +
                ",临床化学检验\t1，25-二羟维生素D<sub>3</sub>测定" +
                ",临床化学检验\t21-羟化酶测定" +
                ",临床化学检验\t21-羟化酶抗体测定" +
                ",临床化学检验\t24h尿液电解质测定" +
                ",临床化学检验\t24小时尿18-羟皮质酮测定" +
                ",临床化学检验\t24小时尿促肾上腺皮质激素测定" +
                ",临床化学检验\t24小时尿碘测定" +
                ",临床化学检验\t24小时尿多巴胺测定" +
                ",临床化学检验\t24小时尿氟测定" +
                ",临床化学检验\t24小时尿钾测定" +
                ",临床化学检验\t24小时尿锰测定" +
                ",临床化学检验\t24小时尿钼测定" +
                ",临床化学检验\t24小时尿钠测定" +
                ",临床化学检验\t24小时尿镍测定" +
                ",临床化学检验\t24小时尿铜测定" +
                ",临床化学检验\t24小时尿维生素B<sub>2</sub>测定" +
                ",临床化学检验\t24小时尿维生素B<sub>5</sub>测定" +
                ",临床化学检验\t24小时尿维生素C测定" +
                ",临床化学检验\t24小时尿硒测定" +
                ",临床化学检验\t24小时尿液儿茶酚胺测定" +
                ",临床化学检验\t24小时尿液生长激素测定" +
                ",临床化学检验\t24小时尿总雌三醇测定" +
                ",临床化学检验\t24小时尿（血）18-羟皮质醇测定" +
                ",临床化学检验\t3-甲氧基去甲肾上腺素测定" +
                ",临床化学检验\t3β-羟类固醇脱氢酶测定" +
                ",临床化学检验\t5'-核苷酸酶测定" +
                ",临床化学检验\t5-羟色胺释放试验" +
                ",临床化学检验\t5′-核苷酸磷酸二酯酶" +
                ",临床化学检验\t9α-氟氢可的松试验" +
                ",临床化学检验\t<sup>14</sup>C-氨基比林呼气试验" +
                ",临床化学检验\tα-L-岩藻糖苷酶测定" +
                ",临床化学检验\tα-半乳糖苷酶测定" +
                ",临床化学检验\tα-羟丁酸脱氢酶测定" +
                ",临床化学检验\tα<sub>1</sub>-酸性糖蛋白测定" +
                ",临床化学检验\tβ-N-乙酰氨基葡萄糖苷酶测定" +
                ",临床化学检验\tβ-氨基己糖苷酶测定" +
                ",临床化学检验\tβ-淀粉样蛋白测定" +
                ",临床化学检验\tβ-内啡肽测定" +
                ",临床化学检验\tγ-谷氨酰基转移酶同工酶测定" +
                ",临床化学检验\tδ-胆红素测定" +
                ",临床化学检验\tⅠ型胶原吡啶交联终肽测定" +
                ",临床化学检验\tⅠ型胶原交联C末端肽、N末端肽测定" +
                ",临床化学检验\tⅠ型胶原交联羧基末端肽测定" +
                ",临床化学检验\tⅠ型前胶原前肽测定" +
                ",临床化学检验\tⅠ型前胶原羧基、氨基前肽测定" +
                ",临床化学检验\tⅠ型前胶原肽测定" +
                ",临床化学检验\tⅣ型胶原7S片段测定" +
                ",临床化学检验\tⅣ型胶原NC1片段测定" +
                ",临床化学检验\tⅣ型胶原测定" +
                ",临床化学检验\t阿扑吗啡试验" +
                ",临床化学检验\t噻嗪类利尿药兴奋试验" +
                ",临床化学检验\t糖化血红蛋白测定——酶法" +
                ",临床化学检验\t血清氯化物测定——电量分析法" +
                ",临床化学检验\t血清氯化物测定——硫氰酸汞比色法" +
                ",临床化学检验\t血清氯化物测定——酶法测定" +
                ",临床化学检验\t血清总钙测定——甲基麝香草酚蓝比色法" +
                ",临床化学检验\tACTH兴奋试验" +
                ",临床化学检验\tCRH-LVP兴奋试验" +
                ",临床化学检验\tC肽检测" +
                ",临床化学检验\tGuthrie试验" +
                ",临床化学检验\tICA512自身抗体测定" +
                ",临床化学检验\tL-γ-谷氨酰基转移酶测定" +
                ",临床化学检验\tLDL／HDL比值" +
                ",临床化学检验\tLHRH垂体兴奋试验" +
                ",临床化学检验\tN-乙酰-β-D-氨基葡萄糖苷酶同工酶测定" +
                ",临床化学检验\tN<sup>5</sup>-甲基四氢叶酸-同型半胱氨酸甲基转移酶测定" +
                ",临床化学检验\tN<sup>5</sup>，N<sup>10</sup>-甲烯四氢叶酸还原酶测定" +
                ",临床化学检验\tPaigen试验" +
                ",临床化学检验\tP物质测定" +
                ",临床化学检验\tTRH兴奋试验" +
                ",临床化学检验\tTRH／LHRH兴奋试验" +
                ",临床化学检验\tTT<sub>4</sub>／TBG比值" +
                ",临床化学检验\t阿米卡星血药浓度测定" +
                ",临床化学检验\t阿米替林血药浓度测定" +
                ",临床化学检验\t白蛋白／球蛋白比值" +
                ",临床化学检验\t半胱氨酸测定" +
                ",临床化学检验\t半乳糖-1-磷酸尿苷酰转移酶测定" +
                ",临床化学检验\t半乳糖耐量试验" +
                ",临床化学检验\t半乳糖凝集素-3测定" +
                ",临床化学检验\t半乳糖血症个体化检测" +
                ",临床化学检验\t备解素因子B测定" +
                ",临床化学检验\t苯巴比妥血药浓度测定" +
                ",临床化学检验\t苯丙氨酸测定" +
                ",临床化学检验\t苯妥英血药浓度测定" +
                ",临床化学检验\t便携式血糖仪床旁血糖监测" +
                ",临床化学检验\t标准碳酸氢盐测定" +
                ",临床化学检验\t吡啶酚、脱氧吡啶酚测定" +
                ",临床化学检验\t吡啶斯的明（PD）与左旋多巴（L-dopa）联合兴奋试验" +
                ",临床化学检验\t丙氨酸氨基转移酶测定" +
                ",临床化学检验\t丙氨酸氨基转移酶同工酶测定" +
                ",临床化学检验\t丙氨酸测定" +
                ",临床化学检验\t丙吡胺血药浓度测定" +
                ",临床化学检验\t丙米嗪血药浓度测定" +
                ",临床化学检验\t丙咪嗪血药浓度测定" +
                ",临床化学检验\t丙酮酸检测" +
                ",临床化学检验\t丙戊酸血药浓度测定" +
                ",临床化学检验\t层粘连蛋白测定" +
                ",临床化学检验\t茶碱血药浓度测定" +
                ",临床化学检验\t超敏雌二醇测定" +
                ",临床化学检验\t超氧化物歧化酶测定" +
                ",临床化学检验\t垂体促性腺激素直接测定" +
                ",临床化学检验\t垂体后叶素注射试验" +
                ",临床化学检验\t垂体激素检验" +
                ",临床化学检验\t垂体腺苷酸环化酶激活肽测定" +
                ",临床化学检验\t触珠蛋白测定" +
                ",临床化学检验\t雌二醇测定" +
                ",临床化学检验\t雌激素撤退试验");

        huayan.append(",临床化学检验\t雌激素受体测定" +
                ",临床化学检验\t雌四醇测定" +
                ",临床化学检验\t雌酮测定" +
                ",临床化学检验\t雌、孕激素联合试验" +
                ",临床化学检验\t促黄体素测定" +
                ",临床化学检验\t催产素酶测定" +
                ",临床化学检验\t催乳素测定" +
                ",临床化学检验\t促甲状腺激素测定" +
                ",临床化学检验\t促甲状腺激素释放激素兴奋试验" +
                ",临床化学检验\t促甲状腺激素受体抗体测定" +
                ",临床化学检验\t促甲状腺素释放激素兴奋试验" +
                ",临床化学检验\t促肾上腺皮质激素和相关肽测定" +
                ",临床化学检验\t促肾上腺皮质激素释放激素测定" +
                ",临床化学检验\t促肾上腺皮质激素释放激素兴奋试验" +
                ",临床化学检验\t促胃液素测定" +
                ",临床化学检验\t粗纤维调节素测定" +
                ",临床化学检验\t促性腺激素释放激素测定" +
                ",临床化学检验\t促性腺激素释放激素兴奋试验" +
                ",临床化学检验\t促胰液素测定" +
                ",临床化学检验\t大剂量地塞米松抑制试验" +
                ",临床化学检验\t蛋氨酸测定" +
                ",临床化学检验\t单胺氧化酶测定" +
                ",临床化学检验\t蛋白结合己糖测定" +
                ",临床化学检验\t胆固醇调节元件结合蛋白测定" +
                ",临床化学检验\t胆固醇裂解酶测定" +
                ",临床化学检验\t胆固醇酯转移蛋白测定" +
                ",临床化学检验\t胆红素代谢功能检测" +
                ",临床化学检验\t胆碱酯酶测定" +
                ",临床化学检验\t胆碱酯酶同工酶测定" +
                ",临床化学检验\t胆囊收缩素测定" +
                ",临床化学检验\t胆石胆固醇测定" +
                ",临床化学检验\t胆石胆红素测定" +
                ",临床化学检验\t胆石胆绿素测定" +
                ",临床化学检验\t胆石钙测定" +
                ",临床化学检验\t胆石磷脂测定" +
                ",临床化学检验\t胆汁胆酸测定" +
                ",临床化学检验\t胆汁钙测定" +
                ",临床化学检验\t胆汁磷脂测定" +
                ",临床化学检验\t胆汁酸测定" +
                ",临床化学检验\t胆汁酸代谢功能检测" +
                ",临床化学检验\t胆汁无机磷测定" +
                ",临床化学检验\t胆汁直接胆红素测定" +
                ",临床化学检验\t胆汁总胆汁酸测定" +
                ",临床化学检验\t胆汁β-葡萄糖醛酸苷酶测定" +
                ",临床化学检验\t淀粉酶测定" +
                ",临床化学检验\t淀粉酶同工酶测定" +
                ",临床化学检验\t靛氰绿试验" +
                ",临床化学检验\t地布卡因指数" +
                ",临床化学检验\t低分子量肝素监测" +
                ",临床化学检验\t低钙试验" +
                ",临床化学检验\t地高辛血药浓度测定" +
                ",临床化学检验\t抵抗素测定" +
                ",临床化学检验\t低磷试验" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定——Friedewald公式计算法" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定——PVS沉淀法" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定——匀相测定法" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定——β-定量法" +
                ",临床化学检验\t低密度脂蛋白受体功能试验" +
                ",临床化学检验\t低密度脂蛋白受体相关蛋白测定" +
                ",临床化学检验\t低密度脂蛋白亚型测定" +
                ",临床化学检验\t地塞米松抑制试验" +
                ",临床化学检验\t动脉-肺泡氧分压比测定" +
                ",临床化学检验\t对乙酰氨基酚血药浓度测定" +
                ",临床化学检验\t多虑平血药浓度测定" +
                ",临床化学检验\t二氢生物喋啶还原酶测定" +
                ",临床化学检验\t二氧化碳结合力测定" +
                ",临床化学检验\t二氧化碳总量测定" +
                ",临床化学检验\t放射免疫分析" +
                ",临床化学检验\t反三碘甲状腺原氨酸测定" +
                ",临床化学检验\t肺表面活性蛋白质D测定" +
                ",临床化学检验\t非高密度脂蛋白胆固醇测定" +
                ",临床化学检验\t非结合胆红素测定" +
                ",临床化学检验\t肺泡-动脉氧分压差测定" +
                ",临床化学检验\t肺泡氧分压测定" +
                ",临床化学检验\t酚红排泄试验" +
                ",临床化学检验\t酚妥拉明试验" +
                ",临床化学检验\t脯氨酸测定" +
                ",临床化学检验\t脯氨酰羟化酶测定" +
                ",临床化学检验\t氟胞嘧啶血药浓度测定" +
                ",临床化学检验\t呋塞米兴奋试验" +
                ",临床化学检验\t改良J-G法血清胆红素测定" +
                ",临床化学检验\t钙耐量试验" +
                ",临床化学检验\t甘氨胆酸测定" +
                ",临床化学检验\t甘氨鹅脱氧胆酸测定" +
                ",临床化学检验\t甘氨酸测定" +
                ",临床化学检验\t肝促凝血酶原酶测定" +
                ",临床化学检验\t刚果红试验" +
                ",临床化学检验\t肝功能常规检查" +
                ",临床化学检验\t肝素辅因子Ⅱ活性测定" +
                ",临床化学检验\t肝纤维化四项" +
                ",临床化学检验\t甘油三酯测定" +
                ",临床化学检验\t肝脂酶测定" +
                ",临床化学检验\t肝组织铜测定" +
                ",临床化学检验\t肝、胆、胰疾病常用实验检测项目" +
                ",临床化学检验\t高尔基体蛋白73测定" +
                ",临床化学检验\t高密度脂蛋白胆固醇测定" +
                ",临床化学检验\t高密度脂蛋白胆固醇测定——超速离心结合选择性沉淀法" +
                ",临床化学检验\t高密度脂蛋白胆固醇测定——硫酸葡聚糖-Mg沉淀法和磷钨酸-镁沉淀法" +
                ",临床化学检验\t高密度脂蛋白胆固醇测定——匀相测定法" +
                ",临床化学检验\t高密度脂蛋白亚型测定" +
                ",临床化学检验\t高敏促甲状腺激素测定" +
                ",临床化学检验\t高渗盐水试验" +
                ",临床化学检验\t高糖基化人绒毛膜促性腺激素测定" +
                ",临床化学检验\t睾酮测定" +
                ",临床化学检验\t骨I型胶原降解产物测定" +
                ",临床化学检验\t瓜氨酸测定" +
                ",临床化学检验\t胱硫醚β合成酶测定" +
                ",临床化学检验\t谷氨酸测定" +
                ",临床化学检验\t谷氨酸脱氢酶测定" +
                ",临床化学检验\t谷氨酸脱羧酶65自身抗体测定" +
                ",临床化学检验\t谷氨酰胺测定" +
                ",临床化学检验\t谷胱甘肽还原酶检测" +
                ",临床化学检验\t谷胱甘肽转移酶测定" +
                ",临床化学检验\t谷胱甘肽转移酶同工酶" +
                ",临床化学检验\t骨碱性磷酸酶测定" +
                ",临床化学检验\t过氯酸钾排泄试验" +
                ",临床化学检验\t过氯酸钾释放试验" +
                ",临床化学检验\t果糖胺检测" +
                ",临床化学检验\t估算的肾小球滤过率" +
                ",临床化学检验\t汗液氯化物测定" +
                ",临床化学检验\t黑色细胞刺激素" +
                ",临床化学检验\t红细胞铬测定" +
                ",临床化学检验\t红细胞钴测定" +
                ",临床化学检验\t红细胞还原型谷胱甘肽测定" +
                ",临床化学检验\t红细胞磷酸甘油酸激酶" +
                ",临床化学检验\t红细胞维生素B<sub>1</sub>测定" +
                ",临床化学检验\t红细胞锌测定" +
                ",临床化学检验\t环孢霉素A测定" +
                ",临床化学检验\t环孢素血药浓度测定" +
                ",临床化学检验\t缓冲碱测定" +
                ",临床化学检验\t环鸟苷酸测定" +
                ",临床化学检验\t火焰光度法血清钠钾测定" +
                ",临床化学检验\t甲氨蝶呤血药浓度测定" +
                ",临床化学检验\t钾负荷试验" +
                ",临床化学检验\t甲功五项" +
                ",临床化学检验\t甲亢平（卡比马唑）试验" +
                ",临床化学检验\t碱负荷试验" +
                ",临床化学检验\t降钙素基因相关肽测定" +
                ",临床化学检验\t渐进性糖基化终末产物测定" +
                ",临床化学检验\t碱剩余测定" +
                ",临床化学检验\t肌氨酸测定" +
                ",临床化学检验\t碱性磷酸酶测定" +
                ",临床化学检验\t碱性磷酸酶同工酶测定" +
                ",临床化学检验\t简易高渗盐水试验" +
                ",临床化学检验\t甲氧基肾上腺素和甲氧基去甲肾上腺素测定" +
                ",临床化学检验\t甲氧氯普胺兴奋试验" +
                ",临床化学检验\t甲状旁腺素测定" +
                ",临床化学检验\t甲状旁腺素相关蛋白测定" +
                ",临床化学检验\t甲状腺刺激免疫球蛋白测定" +
                ",临床化学检验\t甲状腺功能异常筛查试验" +
                ",临床化学检验\t甲状腺激素抗体测定" +
                ",临床化学检验\t甲状腺激素抑制试验" +
                ",临床化学检验\t甲状腺激素自身抗体测定" +
                ",临床化学检验\t甲状腺片抑制甲状腺吸<sup>131</sup>I试验" +
                ",临床化学检验\t甲状腺素结合力测定" +
                ",临床化学检验\t甲状腺微粒体抗体测定" +
                ",临床化学检验\t甲状腺吸<sup>131</sup>I率试验" +
                ",临床化学检验\t基础代谢率测定" +
                ",临床化学检验\t基础体温测定" +
                ",临床化学检验\t极低密度脂蛋白受体测定" +
                ",临床化学检验\t结合胆红素测定" +
                ",临床化学检验\t饥饿试验" +
                ",临床化学检验\t精氨酸测定" +
                ",临床化学检验\t精氨酸酶测定" +
                ",临床化学检验\t精氨酸兴奋试验" +
                ",临床化学检验\t精氨酰琥珀酸裂解酶测定" +
                ",临床化学检验\t经皮脐血采集" +
                ",临床化学检验\t禁水与垂体后叶素联合试验" +
                ",临床化学检验\t肌酸激酶测定" +
                ",临床化学检验\t巨催乳素测定" +
                ",临床化学检验\t菊粉清除率试验" +
                ",临床化学检验\t咖啡因血药浓度测定" +
                ",临床化学检验\t开博通抑制试验" +
                ",临床化学检验\t卡马西平血药浓度测定" +
                ",临床化学检验\t抗蛋白酶3抗体测定" +
                ",临床化学检验\t抗甲状腺球蛋白抗体测定" +
                ",临床化学检验\t抗甲状腺微粒体抗体测定" +
                ",临床化学检验\t抗利尿激素测定" +
                ",临床化学检验\t抗苗勒管激素测定" +
                ",临床化学检验\t抗人绒毛膜促性腺激素抗体测定" +
                ",临床化学检验\t抗生素最低抑／杀菌浓度测定" +
                ",临床化学检验\t抗髓过氧化物酶抗体测定" +
                ",临床化学检验\t抗心肌抗体测定" +
                ",临床化学检验\t卡托普利抑制试验" +
                ",临床化学检验\t可乐定兴奋试验" +
                ",临床化学检验\t可溶性CD40配体测定" +
                ",临床化学检验\t口服葡萄糖耐量试验" +
                ",临床化学检验\t快速滴注钙抑制试验" +
                ",临床化学检验\t奎尼丁血药浓度测定" +
                ",临床化学检验\t赖氨酸测定" +
                ",临床化学检验\t拉莫三嗪血药浓度测定" +
                ",临床化学检验\t酪胺试验" +
                ",临床化学检验\t酪氨酸测定" +
                ",临床化学检验\t酪酪肽测定" +
                ",临床化学检验\t类胆固醇颗粒样残留物测定" +
                ",临床化学检验\t类甘油三酯颗粒样残留物测定" +
                ",临床化学检验\t类胰蛋白酶测定" +
                ",临床化学检验\t冷加压试验" +
                ",临床化学检验\t亮氨酸测定" +
                ",临床化学检验\t利多卡因试验" +
                ",临床化学检验\t利多卡因血药浓度测定" +
                ",临床化学检验\t利伐沙班监测" +
                ",临床化学检验\t磷清除率试验" +
                ",临床化学检验\t磷清除试验" +
                ",临床化学检验\t磷酸己糖异构酶测定" +
                ",临床化学检验\t磷酸盐排泄指数" +
                ",临床化学检验\t磷酸组织胺试验" +
                ",临床化学检验\t磷重吸收试验" +
                ",临床化学检验\t硫酸雌酮测定" +
                ",临床化学检验\t硫酸脱氢表雄酮测定" +
                ",临床化学检验\t硫酸脱氢表雄酮负荷试验" +
                ",临床化学检验\t锂血药浓度测定" +
                ",临床化学检验\t氯丙嗪兴奋试验" +
                ",临床化学检验\t滤过钠排泄分数" +
                ",临床化学检验\t氯米芬负荷试验" +
                ",临床化学检验\t氯压定（可乐定）试验" +
                ",临床化学检验\t卵磷脂胆固醇酯酰转移酶测定" +
                ",临床化学检验\t卵磷脂胆固醇脂酰转移酶测定" +
                ",临床化学检验\t卵泡刺激素测定" +
                ",临床化学检验\t螺内酯（安体舒通）醛固酮比率测定" +
                ",临床化学检验\t麦考酚酸血药浓度测定" +
                ",临床化学检验\t毛细管血总二氧化碳测定" +
                ",临床化学检验\t霉酚酸酯测定" +
                ",临床化学检验\t美替拉酮兴奋试验" +
                ",临床化学检验\t免疫透射比浊法测定ApoAⅠ和ApoB" +
                ",临床化学检验\t免疫透射比浊法测定ApoE" +
                ",临床化学检验\t免疫透射比浊法测定脂蛋白（a）" +
                ",临床化学检验\t泌乳素测定" +
                ",临床化学检验\t泌乳素分泌兴奋试验" +
                ",临床化学检验\t泌乳素分泌抑制试验" +
                ",临床化学检验\t木糖吸收试验" +
                ",临床化学检验\t钠负荷试验" +
                ",临床化学检验\t奈替米星血药浓度测定" +
                ",临床化学检验\t脑啡肽测定" +
                ",临床化学检验\t脑脊液氨基酸谱测定" +
                ",临床化学检验\t脑脊液白蛋白测定" +
                ",临床化学检验\t脑脊液免疫球蛋白G测定" +
                ",临床化学检验\t脑脊液总蛋白测定" +
                ",临床化学检验\t脑钠肽测定" +
                ",临床化学检验\t内生肌酐清除率测定" +
                ",临床化学检验\t内因子测定" +
                ",临床化学检验\t内因子阻滞抗体测定" +
                ",临床化学检验\t黏多糖测定" +
                ",临床化学检验\t尿17-羟类固醇测定" +
                ",临床化学检验\t尿17-生酮皮质类固醇测定" +
                ",临床化学检验\t尿17-酮类固醇测定" +
                ",临床化学检验\t尿3-甲氧基-4-羟苦杏仁酸测定" +
                ",临床化学检验\t鸟氨酸氨基甲酰转移酶测定" +
                ",临床化学检验\t鸟氨酸测定" +
                ",临床化学检验\t尿雌激素／肌酐比值" +
                ",临床化学检验\t尿促性腺激素刺激试验" +
                ",临床化学检验\t尿蛋白电泳" +
                ",临床化学检验\t尿蛋白选择性指数测定" +
                ",临床化学检验\t尿道分泌物检查" +
                ",临床化学检验\t尿结石铵盐测定" +
                ",临床化学检验\t尿结石胞苷测定" +
                ",临床化学检验\t尿结石草酸钙测定" +
                ",临床化学检验\t尿结石胆固醇测定" +
                ",临床化学检验\t尿结石胆红素测定" +
                ",临床化学检验\t尿结石胆绿素测定" +
                ",临床化学检验\t尿结石靛蓝测定" +
                ",临床化学检验\t尿结石钙测定" +
                ",临床化学检验\t尿结石胱氨酸测定" +
                ",临床化学检验\t尿结石磺胺类药物测定" +
                ",临床化学检验\t尿结石黄嘌呤测定" +
                ",临床化学检验\t尿结石磷酸盐测定" +
                ",临床化学检验\t尿结石镁测定" +
                ",临床化学检验\t尿结石尿酸测定" +
                ",临床化学检验\t尿结石尿脂石测定" +
                ",临床化学检验\t尿结石碳酸盐测定" +
                ",临床化学检验\t尿结石铁测定" +
                ",临床化学检验\t尿结石纤维蛋白测定" +
                ",临床化学检验\t尿滤过钠排泄分数测定" +
                ",临床化学检验\t尿氯化物测定" +
                ",临床化学检验\t尿免疫球蛋白G测定" +
                ",临床化学检验\t尿三氯化铁试验" +
                ",临床化学检验\t尿视黄醇结合蛋白测定" +
                ",临床化学检验\t尿香草基苦杏仁酸测定" +
                ",临床化学检验\t尿液17-羟皮质类固醇测定" +
                ",临床化学检验\t尿液17-酮类固醇测定" +
                ",临床化学检验\t尿液17-酮皮质类固醇测定" +
                ",临床化学检验\t尿液5-羟吲哚乙酸测定" +
                ",临床化学检验\t尿液还原糖试验" +
                ",临床化学检验\t尿液浓缩稀释试验" +
                ",临床化学检验\t尿液香草扁桃酸测定" +
                ",临床化学检验\t尿中4-吡哆酸含量测定" +
                ",临床化学检验\t尿转铁蛋白测定" +
                ",临床化学检验\t尿总蛋白测定" +
                ",临床化学检验\t牛磺酸测定" +
                ",临床化学检验\tp24蛋白测定" +
                ",临床化学检验\t嘌呤/嘧啶测定" +
                ",临床化学检验\t苹果酸脱氢酶测定" +
                ",临床化学检验\t皮质醇测定" +
                ",临床化学检验\t皮质醇昼夜节律试验" +
                ",临床化学检验\t皮质类固醇结合蛋白测定" +
                ",临床化学检验\t皮质酮测定" +
                ",临床化学检验\t普乐可复（FK506）测定" +
                ",临床化学检验\t普鲁卡因血药浓度测定" +
                ",临床化学检验\t葡萄糖负荷试验" +
                ",临床化学检验\t葡萄糖脑苷酶测定" +
                ",临床化学检验\t普通血栓弹力图实验" +
                ",临床化学检验\t强啡肽测定" +
                ",临床化学检验\t羟甲基戊二酰辅酶A还原酶测定" +
                ",临床化学检验\t鞘磷脂酶测定" +
                ",临床化学检验\t壳三糖苷酶测定" +
                ",临床化学检验\t庆大霉素血药浓度测定" +
                ",临床化学检验\t清道夫受体测定" +
                ",临床化学检验\t青霉胺负荷试验" +
                ",临床化学检验\t巯基嘌呤甲基转移酶测定" +
                ",临床化学检验\t醛固酮-18-葡萄糖醛酸苷测定" +
                ",临床化学检验\t醛固酮测定" +
                ",临床化学检验\t醛固酮合成酶测定" +
                ",临床化学检验\t醛固酮／肾素比值" +
                ",临床化学检验\t去氨加压素试验" +
                ",临床化学检验\t醛缩酶同工酶测定" +
                ",临床化学检验\t全血葡萄糖-6-磷酸酶测定" +
                ",临床化学检验\t缺血性修饰白蛋白测定" +
                ",临床化学检验\t去甲丙米嗪血药浓度测定" +
                ",临床化学检验\t去甲肾上腺素测定" +
                ",临床化学检验\t去甲替林血药浓度测定" +
                ",临床化学检验\t人绒毛膜促甲状腺激素测定" +
                ",临床化学检验\t人绒毛膜促性腺激素测定" +
                ",临床化学检验\t人绒毛膜促性腺激素刺激试验" +
                ",临床化学检验\t妊娠带蛋白测定" +
                ",临床化学检验\t妊娠期糖尿病筛查试验" +
                ",临床化学检验\t妊娠期糖尿病筛查试验" +
                ",临床化学检验\t妊娠特异性β<sub>1</sub>-糖蛋白" +
                ",临床化学检验\t妊娠相关血浆蛋白A测定" +
                ",临床化学检验\t妊娠相关血浆蛋白B测定" +
                ",临床化学检验\t人胎盘催乳素测定" +
                ",临床化学检验\t热休克蛋白" +
                ",临床化学检验\t乳酸检测" +
                ",临床化学检验\t乳酸脱氢酶测定" +
                ",临床化学检验\t乳酸脱氢酶同工酶测定" +
                ",临床化学检验\t色氨酸测定" +
                ",临床化学检验\t上消化道癌人群筛查及早诊早治技术方案" +
                ",临床化学检验\t山梨醇脱氢酶测定" +
                ",临床化学检验\t肾功能常规检查" +
                ",临床化学检验\t生物素酶测定" +
                ",临床化学检验\t生长激素测定" +
                ",临床化学检验\t生长激素和胰岛素样生长因子测定" +
                ",临床化学检验\t生长激素结合蛋白测定" +
                ",临床化学检验\t生长激素抗体测定" +
                ",临床化学检验\t生长激素释放激素测定" +
                ",临床化学检验\t生长激素释放激素兴奋试验" +
                ",临床化学检验\t生长激素释放抑制激素测定" +
                ",临床化学检验\t生长激素释放抑制素测定" +
                ",临床化学检验\t生长激素兴奋试验" +
                ",临床化学检验\t生长激素抑制试验" +
                ",临床化学检验\t神经降压素测定" +
                ",临床化学检验\t神经肽Y测定" +
                ",临床化学检验\t肾上腺素测定" +
                ",临床化学检验\t肾素测定" +
                ",临床化学检验\t肾小管葡萄糖最大重吸收量测定" +
                ",临床化学检验\t肾小管重吸收磷率试验" +
                ",临床化学检验\t肾小管最大对氨基马尿酸排泄量试验" +
                ",临床化学检验\t实际碳酸氢盐测定" +
                ",临床化学检验\t石墨炉原子吸收光谱法全血铅测定" +
                ",临床化学检验\t瘦素测定" +
                ",临床化学检验\t双氢睾酮测定" +
                ",临床化学检验\t丝氨酸测定" +
                ",临床化学检验\t丝氨酸蛋白酶测定" +
                ",临床化学检验\t酸负荷试验" +
                ",临床化学检验\t苏氨酸测定" +
                ",临床化学检验\t酸性磷酸酶测定" +
                ",临床化学检验\t酸性磷酸酶同工酶测定" +
                ",临床化学检验\t髓过氧化物酶测定" +
                ",临床化学检验\t随机尿氨基酸谱测定" +
                ",临床化学检验\t随机尿儿茶酚胺测定" +
                ",临床化学检验\t随机尿高香草酸测定" +
                ",临床化学检验\t随机尿香草扁桃酸测定" +
                ",临床化学检验\t随机尿总甲氧基去甲肾上腺素测定" +
                ",临床化学检验\t随机尿β<sub>2</sub>-微球蛋白清除率" +
                ",临床化学检验\t髓鞘相关糖蛋白测定" +
                ",临床化学检验\t速尿（呋塞米）＋立位试验" +
                ",临床化学检验\t胎儿纤连蛋白测定" +
                ",临床化学检验\t胎盘蛋白5测定" +
                ",临床化学检验\t胎盘碱性磷酸酶测定" +
                ",临床化学检验\t他克莫司血药浓度测定" +
                ",临床化学检验\t糖蛋白电泳" +
                ",临床化学检验\t糖化血红蛋白测定——HPLC法" +
                ",临床化学检验\t糖化血红蛋白测定——免疫比浊法" +
                ",临床化学检验\t糖化血红蛋白测定——亲和层析法" +
                ",临床化学检验\t糖化血红蛋白检测" +
                ",临床化学检验\t糖化血清蛋白测定" +
                ",临床化学检验\t糖原磷酸化酶BB测定" +
                ",临床化学检验\t糖原磷酸化酶同工酶" +
                ",临床化学检验\t碳酸锂血药浓度测定" +
                ",临床化学检验\t天冬氨酸氨基转移酶测定" +
                ",临床化学检验\t天冬氨酸氨基转移酶同工酶测定" +
                ",临床化学检验\t天门冬氨酸氨基转移酶同工酶测定" +
                ",临床化学检验\t透明质酸测定" +
                ",临床化学检验\t妥布霉素血药浓度测定" +
                ",临床化学检验\t脱氧皮质酮测定" +
                ",临床化学检验\t唾液游离皮质醇测定" +
                ",临床化学检验\t万古霉素血药浓度测定" +
                ",临床化学检验\t蛙皮素测定" +
                ",临床化学检验\t胃蛋白酶测定" +
                ",临床化学检验\t胃蛋白酶原Ⅰ、Ⅱ测定" +
                ",临床化学检验\t胃动素测定" +
                ",临床化学检验\t微分电位溶出法全血铅测定" +
                ",临床化学检验\t胃复安（甲氧氯普胺）试验" +
                ",临床化学检验\t微量白蛋白尿测定" +
                ",临床化学检验\t微量元素筛查试验" +
                ",临床化学检验\t微粒体甘油三酯转移蛋白测定" +
                ",临床化学检验\t胃泌素测定" +
                ",临床化学检验\t维生素A测定" +
                ",临床化学检验\t维生素A原测定" +
                ",临床化学检验\t维生素B<sub>2</sub>测定" +
                ",临床化学检验\t维生素B<sub>3</sub>测定" +
                ",临床化学检验\t维生素B<sub>5</sub>测定" +
                ",临床化学检验\t维生素B<sub>6</sub>测定" +
                ",临床化学检验\t维生素B<sub>7</sub>测定" +
                ",临床化学检验\t维生素E测定" +
                ",临床化学检验\t维生素K测定" +
                ",临床化学检验\t钨舟无焰原子吸收光谱全血铅测定" +
                ",临床化学检验\t腺苷脱氨酶测定" +
                ",临床化学检验\t酰基辅酶A胆固醇酰基转移酶-2测定" +
                ",临床化学检验\t线粒体呼吸链复合酶测定" +
                ",临床化学检验\t先天性甲状腺功能低下筛查试验" +
                ",临床化学检验\t小而密低密度脂蛋白测定" +
                ",临床化学检验\t硝基苯肼试验" +
                ",临床化学检验\t小剂量地塞米松抑制试验" +
                ",临床化学检验\t细胞间黏附分子-1测定" +
                ",临床化学检验\t缬氨酸测定" +
                ",临床化学检验\t心房利钠肽测定" +
                ",临床化学检验\t性激素结合蛋白测定" +
                ",临床化学检验\t性激素结合球蛋白测定" +
                ",临床化学检验\t性激素六项" +
                ",临床化学检验\t心肌肌钙蛋白T测定" +
                ",临床化学检验\t心肌肌钙蛋白Ⅰ测定" +
                ",临床化学检验\t心肌酶谱四项" +
                ",临床化学检验\t心肌酶谱五项" +
                ",临床化学检验\t心肌纤维蛋白原活性测定" +
                ",临床化学检验\t心钠素测定" +
                ",临床化学检验\t新生儿半乳糖测定" +
                ",临床化学检验\t新生儿胆红素测定" +
                ",临床化学检验\t新生儿先天性甲状腺功能减退症筛查" +
                ",临床化学检验\t雄激素受体测定" +
                ",临床化学检验\t雄烯二酮测定" +
                ",临床化学检验\t修饰型低密度脂蛋白测定" +
                ",临床化学检验\t溴隐亭兴奋试验" +
                ",临床化学检验\t溴隐亭抑制试验" +
                ",临床化学检验\t血氨测定" +
                ",临床化学检验\t血二羟苯甘醇测定" +
                ",临床化学检验\t血管活性肠肽测定" +
                ",临床化学检验\t血管紧张素转化酶测定" +
                ",临床化学检验\t血管紧张素Ⅰ、Ⅱ测定" +
                ",临床化学检验\t血红蛋白A1c测定" +
                ",临床化学检验\t血红蛋白A1测定" +
                ",临床化学检验\t血浆(清)碳酸氢根及总二氧化碳测定" +
                ",临床化学检验\t血浆(清)阴离子间隙" +
                ",临床化学检验\t血浆5-羟色胺测定" +
                ",临床化学检验\t血浆氨基酸谱测定" +
                ",临床化学检验\t血浆丙酮酸测定" +
                ",临床化学检验\t血浆蛋白结合碘测定" +
                ",临床化学检验\t血浆多巴胺测定" +
                ",临床化学检验\t血浆儿茶酚胺测定" +
                ",临床化学检验\t血浆高香草酸测定" +
                ",临床化学检验\t血浆谷胱甘肽测定" +
                ",临床化学检验\t血浆环磷酸鸟苷测定" +
                ",临床化学检验\t血浆氯测定" +
                ",临床化学检验\t血浆前列腺素测定" +
                ",临床化学检验\t血浆醛固酮浓度及尿中醛固酮排泄量测定" +
                ",临床化学检验\t血浆乳酸测定" +
                ",临床化学检验\t血浆渗量检测" +
                ",临床化学检验\t血浆肾素活性测定" +
                ",临床化学检验\t血浆同型半胱氨酸测定" +
                ",临床化学检验\t血浆维生素C测定" +
                ",临床化学检验\t血浆乙酰胆碱测定" +
                ",临床化学检验\t血浆一氧化氮测定" +
                ",临床化学检验\t血浆、尿羟脯氨酸测定" +
                ",临床化学检验\t血浆、尿液组胺测定" +
                ",临床化学检验\t血浆、血清、全血、毛细血管全血葡萄糖测定" +
                ",临床化学检验\t血浆、羊水催产素测定" +
                ",临床化学检验\t血浆、羊水胰高血糖素测定" +
                ",临床化学检验\t血钾尿钾酶法测定" +
                ",临床化学检验\t血间羟肾上腺类似物的测定" +
                ",临床化学检验\t血锰测定" +
                ",临床化学检验\t血钠尿钠酶法测定" +
                ",临床化学检验\t血气分析" +
                ",临床化学检验\t血清3，3′，5′-三碘甲状腺原氨酸测定" +
                ",临床化学检验\t血清AST/ALT比值" +
                ",临床化学检验\t血清氨基末端-B型利钠肽前体测定" +
                ",临床化学检验\t血清氨基肽酶测定" +
                ",临床化学检验\t血清白蛋白测定" +
                ",临床化学检验\t血清半胱氨酸蛋白酶抑制蛋白C测定" +
                ",临床化学检验\t血清超长链脂肪酸测定" +
                ",临床化学检验\t血清蛋白醋酸纤维素膜电泳" +
                ",临床化学检验\t血清蛋白电泳" +
                ",临床化学检验\t血清蛋白毛细管区带电泳" +
                ",临床化学检验\t血清蛋白琼脂糖凝胶电泳" +
                ",临床化学检验\t血清胆红素测定——钒酸盐氧化法" +
                ",临床化学检验\t血清反T<sub>3</sub>测定" +
                ",临床化学检验\t血清非蛋白氮测定");
        huayan.append(",临床化学检验\t血清甘氨胆汁酸测定" +
                ",临床化学检验\t血清胱抑素C测定" +
                ",临床化学检验\t血清谷丙转氨酶测定" +
                ",临床化学检验\t血清骨钙素测定" +
                ",临床化学检验\t血清果糖测定" +
                ",临床化学检验\t血清过氧化脂质测定" +
                ",临床化学检验\t血清钾测定" +
                ",临床化学检验\t血清降钙素测定" +
                ",临床化学检验\t血清碱性磷酸酶测定" +
                ",临床化学检验\t血清甲状腺结合球蛋白测定" +
                ",临床化学检验\t血清甲状腺球蛋白测定" +
                ",临床化学检验\t血清肌钙蛋白测定" +
                ",临床化学检验\t血清肌酐测定<b></b>" +
                ",临床化学检验\t血清肌红蛋白测定" +
                ",临床化学检验\t血清肌酸激酶同工酶MB质量测定" +
                ",临床化学检验\t血清抗酒石酸酸性磷酸酶测定" +
                ",临床化学检验\t血清抗磷脂抗体测定" +
                ",临床化学检验\t血清类黏蛋白测定" +
                ",临床化学检验\t血清磷脂测定" +
                ",临床化学检验\t血清离子钙测定" +
                ",临床化学检验\t血清氯测定" +
                ",临床化学检验\t血清镁测定——Calmagite染料比色法" +
                ",临床化学检验\t血清镁测定——二甲苯胺蓝法" +
                ",临床化学检验\t血清镁测定——甲基麝香草酚蓝比色法" +
                ",临床化学检验\t血清镁测定——原子吸收分光光度法" +
                ",临床化学检验\t血清免疫固定电泳" +
                ",临床化学检验\t血清钠测定" +
                ",临床化学检验\t血清黏蛋白测定" +
                ",临床化学检验\t血清尿酸测定" +
                ",临床化学检验\t血清尿酸／肌酐比值" +
                ",临床化学检验\t血清尿素测定" +
                ",临床化学检验\t血清尿素氮／血清肌酐比值" +
                ",临床化学检验\t血清前白蛋白测定" +
                ",临床化学检验\t血清球蛋白" +
                ",临床化学检验\t血清醛缩酶测定" +
                ",临床化学检验\t血清缺血修饰白蛋白测定" +
                ",临床化学检验\t血清妊娠相关蛋白A测定" +
                ",临床化学检验\t血清溶菌酶测定" +
                ",临床化学检验\t血清三酰甘油测定——变色酸显色法(CDC参考方法)" +
                ",临床化学检验\t血清三酰甘油测定——酶法( GPO-PAP法)" +
                ",临床化学检验\t血清三酰甘油测定——去游离甘油的TG测定(二步酶法)" +
                ",临床化学检验\t血清杀菌活性测定" +
                ",临床化学检验\t血清视黄醇结合蛋白测定" +
                ",临床化学检验\t血清酸性磷酸酶测定" +
                ",临床化学检验\t血清铁比色法测定" +
                ",临床化学检验\t血清铁蛋白测定" +
                ",临床化学检验\t血清铜测定——比色法" +
                ",临床化学检验\t血清铜测定——双环己酮草酰二腙比色法" +
                ",临床化学检验\t血清铜测定——原子吸收分光光度法" +
                ",临床化学检验\t血清铜蓝蛋白测定" +
                ",临床化学检验\t血清同型半胱氨酸测定" +
                ",临床化学检验\t血清无机磷测定" +
                ",临床化学检验\t血清纤维连接蛋白测定" +
                ",临床化学检验\t血清锌测定" +
                ",临床化学检验\t血清心脏型脂肪酸结合蛋白测定" +
                ",临床化学检验\t血清乙酰胆碱酯酶测定" +
                ",临床化学检验\t血清游离T3和游离T4测定" +
                ",临床化学检验\t血清游离甘油测定" +
                ",临床化学检验\t血清游离脂肪酸测定" +
                ",临床化学检验\t血清载脂蛋白测定" +
                ",临床化学检验\t血清脂蛋白-X测定" +
                ",临床化学检验\t血清脂蛋白电泳" +
                ",临床化学检验\t血清脂肪酸结合蛋白4测定" +
                ",临床化学检验\t血清酯化胆固醇测定" +
                ",临床化学检验\t血清转铁蛋白测定" +
                ",临床化学检验\t血清总蛋白测定" +
                ",临床化学检验\t血清总胆固醇测定" +
                ",临床化学检验\t血清总钙测定——邻甲酚酞络合酮比色法" +
                ",临床化学检验\t血清总钙测定——偶氮砷Ⅲ比色法" +
                ",临床化学检验\t血清总铁结合力测定" +
                ",临床化学检验\t血清总脂质测定" +
                ",临床化学检验\t血清α-淀粉样蛋白测定" +
                ",临床化学检验\t血清α-羟丁酸脱氢酶测定" +
                ",临床化学检验\t血清α1-抗胰蛋白酶测定" +
                ",临床化学检验\t血清α<sub>1</sub>-微球蛋白测定" +
                ",临床化学检验\t血清α<sub>2</sub>-巨球蛋白测定" +
                ",临床化学检验\t血清β-羟丁酸" +
                ",临床化学检验\t血清β-羟丁酸测定" +
                ",临床化学检验\t血清、尿液C肽测定" +
                ",临床化学检验\t血清、尿液、羊水雌三醇测定" +
                ",临床化学检验\t血清、血浆、24小时尿、红细胞镁测定" +
                ",临床化学检验\t血清（浆）IGF-Ⅰ及IGFBP-3测定" +
                ",临床化学检验\t血清（浆）甲状腺素测定" +
                ",临床化学检验\t血清（浆）甲状腺素结合球蛋白测定" +
                ",临床化学检验\t血清（浆）三碘甲状腺原氨酸测定" +
                ",临床化学检验\t血清（浆）游离甲状腺素测定" +
                ",临床化学检验\t血糖测定" +
                ",临床化学检验\t血氧饱和度测定" +
                ",临床化学检验\t血氧分压测定" +
                ",临床化学检验\t血氧含量测定" +
                ",临床化学检验\t血液半饱和氧分压" +
                ",临床化学检验\t血液半饱和氧分压测定" +
                ",临床化学检验\t血液电解质检查" +
                ",临床化学检验\t血液二氧化碳分压测定" +
                ",临床化学检验\t血液黏蛋白测定" +
                ",临床化学检验\t血液pH测定" +
                ",临床化学检验\t血液葡萄糖测定" +
                ",临床化学检验\t血液生化全项" +
                ",临床化学检验\t血液唾液酸测定" +
                ",临床化学检验\t血一氧化碳测定" +
                ",临床化学检验\t血脂常规检查" +
                ",临床化学检验\t洋地黄毒苷血药浓度测定" +
                ",临床化学检验\t氧化高密度脂蛋白测定" +
                ",临床化学检验\t羊水胆红素测定" +
                ",临床化学检验\t羊水结晶检查" +
                ",临床化学检验\t夜间睡眠皮质醇分泌试验" +
                ",临床化学检验\t叶酸筛查试验" +
                ",临床化学检验\t遗传筛查" +
                ",临床化学检验\t胰蛋白酶测定" +
                ",临床化学检验\t胰岛瘤相关抗原-2自身抗体检测" +
                ",临床化学检验\t胰岛素测定" +
                ",临床化学检验\t胰岛素低血糖刺激试验" +
                ",临床化学检验\t胰岛素释放试验" +
                ",临床化学检验\t胰岛素诱发低血糖试验" +
                ",临床化学检验\t胰岛素原测定" +
                ",临床化学检验\t胰岛素自身抗体测定" +
                ",临床化学检验\t胰岛素、C肽释放试验" +
                ",临床化学检验\t胰岛细胞自身抗体测定" +
                ",临床化学检验\t胰多肽测定" +
                ",临床化学检验\t胰高血糖素刺激试验" +
                ",临床化学检验\t乙琥胺血药浓度测定" +
                ",临床化学检验\t异亮氨酸测定" +
                ",临床化学检验\t阴道脱落细胞涂片检查" +
                ",临床化学检验\t影响氯吡格雷转化为活性代谢产物的基因多态性" +
                ",临床化学检验\t异柠檬酸脱氢酶测定" +
                ",临床化学检验\t阴离子间隙测定" +
                ",临床化学检验\t抑胃肽测定" +
                ",临床化学检验\t胰腺外分泌功能试验" +
                ",临床化学检验\t抑制素测定" +
                ",临床化学检验\t有机磷农药中毒的血胆碱酯酶活性检测" +
                ",临床化学检验\t游离甲状腺素指数" +
                ",临床化学检验\t游离羟脯氨酸测定" +
                ",临床化学检验\t游离三碘甲状腺原氨酸测定" +
                ",临床化学检验\t游离型雌三醇测定" +
                ",临床化学检验\t游离雄激素指数" +
                ",临床化学检验\t右旋木糖吸收试验" +
                ",临床化学检验\t孕妇血清、羊水甲胎蛋白测定" +
                ",临床化学检验\t孕酮测定" +
                ",临床化学检验\t孕酮撤退试验" +
                ",临床化学检验\t孕酮受体测定" +
                ",临床化学检验\t孕烷二醇测定" +
                ",临床化学检验\t载脂蛋白AⅠ测定" +
                ",临床化学检验\t载脂蛋白AⅡ测定" +
                ",临床化学检验\t载脂蛋白B测定" +
                ",临床化学检验\t载脂蛋白B／载脂蛋白AⅠ比值" +
                ",临床化学检验\t载脂蛋白CⅢ测定" +
                ",临床化学检验\t载脂蛋白E测定" +
                ",临床化学检验\t载脂蛋白H测定" +
                ",临床化学检验\t整合素1测定" +
                ",临床化学检验\t整合素2测定" +
                ",临床化学检验\t脂蛋白磷脂酶A<sub>2</sub>测定" +
                ",临床化学检验\t脂蛋白脂肪酶测定" +
                ",临床化学检验\t脂蛋白（a）测定" +
                ",临床化学检验\t脂肪酶测定" +
                ",临床化学检验\t脂肪吸收试验" +
                ",临床化学检验\t直接胆红素与总胆红素比值" +
                ",临床化学检验\t直接胆红素∶间接胆红素" +
                ",临床化学检验\t脂联素测定" +
                ",临床化学检验\t治疗药物监测" +
                ",临床化学检验\t肿瘤标记物检测" +
                ",临床化学检验\t中枢神经特异蛋白(S100β)测定" +
                ",临床化学检验\t中性粒细胞明胶酶相关脂质运载蛋白" +
                ",临床化学检验\t总雌激素测定" +
                ",临床化学检验\t总胆固醇／高密度脂蛋白胆固醇比值" +
                ",临床化学检验\t总胆红素测定" +
                ",临床化学检验\t总胆汁酸测定" +
                ",临床化学检验\t总甲状腺素和总三碘甲状腺原氨酸测定" +
                ",临床化学检验\t总皮质醇测定" +
                ",临床化学检验\t组氨酸测定" +
                ",临床化学检验\t组氨酸酶测定" +
                ",临床化学检验\t左旋多巴兴奋试验" +
                ",临床化学检验\t左旋多巴抑制试验" +
                ",临床化学检验\t组织蛋白酶测定" +
                ",临床免疫检验\t<i>Septin9</i>基因甲基化检测" +
                ",临床免疫检验\tβ-癌胚抗原测定" +
                ",临床免疫检验\tβ<sub>2</sub>-微球蛋白检测" +
                ",临床免疫检验\tγ-干扰素测定" +
                ",临床免疫检验\tⅠ型超敏反应的皮肤试验" +
                ",临床免疫检验\tⅣ型超敏反应的皮肤试验" +
                ",临床免疫检验\tANA谱检测" +
                ",临床免疫检验\tB细胞分化抗原检测" +
                ",临床免疫检验\tB细胞膜表面免疫球蛋白测定" +
                ",临床免疫检验\tB细胞亚群检测" +
                ",临床免疫检验\tDC检测" +
                ",临床免疫检验\tFCM检测NK细胞杀伤活性" +
                ",临床免疫检验\tHER2／neu蛋白测定" +
                ",临床免疫检验\tHLA-B27测定" +
                ",临床免疫检验\tHPV E6/E7" +
                ",临床免疫检验\tIgD检测" +
                ",临床免疫检验\tIgG、IgA和IgM检测" +
                ",临床免疫检验\tKi-67蛋白测定" +
                ",临床免疫检验\tM2型丙酮酸激酶测定" +
                ",临床免疫检验\tM蛋白检测" +
                ",临床免疫检验\tNK细胞的绝对计数及百分率检测" +
                ",临床免疫检验\tNK细胞检测" +
                ",临床免疫检验\tS-100蛋白测定" +
                ",临床免疫检验\tT细胞分化抗原测定" +
                ",临床免疫检验\tT细胞及其亚群（CD3／CD4／CD8）计数" +
                ",临床免疫检验\tT细胞亚群表型检测" +
                ",临床免疫检验\tT细胞转化试验" +
                ",临床免疫检验\t癌胚抗原测定" +
                ",临床免疫检验\t癌胚异铁蛋白测定" +
                ",临床免疫检验\t白三烯B<sub>4</sub>水平测定" +
                ",临床免疫检验\t白细胞介素-10检测" +
                ",临床免疫检验\t白细胞介素-11测定" +
                ",临床免疫检验\t白细胞介素-12测定" +
                ",临床免疫检验\t白细胞介素-13测定" +
                ",临床免疫检验\t白细胞介素-14测定" +
                ",临床免疫检验\t白细胞介素-15测定" +
                ",临床免疫检验\t白细胞介素-16测定" +
                ",临床免疫检验\t白细胞介素-17检测" +
                ",临床免疫检验\t白细胞介素-18测定" +
                ",临床免疫检验\t白细胞介素-19测定" +
                ",临床免疫检验\t白细胞介素-1测定" +
                ",临床免疫检验\t白细胞介素-20测定" +
                ",临床免疫检验\t白细胞介素-21测定" +
                ",临床免疫检验\t白细胞介素-22测定" +
                ",临床免疫检验\t白细胞介素-23测定" +
                ",临床免疫检验\t白细胞介素-24测定" +
                ",临床免疫检验\t白细胞介素-25测定" +
                ",临床免疫检验\t白细胞介素-26测定" +
                ",临床免疫检验\t白细胞介素-27测定" +
                ",临床免疫检验\t白细胞介素-2测定" +
                ",临床免疫检验\t白细胞介素-3测定" +
                ",临床免疫检验\t白细胞介素-4检测" +
                ",临床免疫检验\t白细胞介素-5测定" +
                ",临床免疫检验\t白细胞介素-6检测" +
                ",临床免疫检验\t白细胞介素-7测定" +
                ",临床免疫检验\t白细胞介素-8检测" +
                ",临床免疫检验\t白细胞介素-9测定" +
                ",临床免疫检验\t白细胞移动抑制试验" +
                ",临床免疫检验\t白血病免疫分型" +
                ",临床免疫检验\t膀胱肿瘤抗原测定" +
                ",临床免疫检验\t表皮生长因子测定" +
                ",临床免疫检验\t表皮生长因子受体1测定" +
                ",临床免疫检验\t表皮生长因子受体2测定" +
                ",临床免疫检验\t丙肝病毒核心抗原测定" +
                ",临床免疫检验\t丙肝病毒抗体测定" +
                ",临床免疫检验\t不规则抗体筛选和鉴定" +
                ",临床免疫检验\t补体C1q含量检测" +
                ",临床免疫检验\t补体C1酯酶抑制物测定" +
                ",临床免疫检验\t补体C2测定" +
                ",临床免疫检验\t补体C3a测定" +
                ",临床免疫检验\t补体C3b测定" +
                ",临床免疫检验\t补体C3c测定" +
                ",临床免疫检验\t补体C3d测定" +
                ",临床免疫检验\t补体C3、C4含量检测" +
                ",临床免疫检验\t补体C4a测定" +
                ",临床免疫检验\t补体C4测定" +
                ",临床免疫检验\t补体C5a测定" +
                ",临床免疫检验\t补体C5b～C9测定" +
                ",临床免疫检验\t补体C6测定" +
                ",临床免疫检验\t补体C7测定" +
                ",临床免疫检验\t补体C8测定" +
                ",临床免疫检验\t补体C9测定" +
                ",临床免疫检验\t补体经典途径溶血活性(CH50)检测" +
                ",临床免疫检验\t补体旁路途径溶血活性（AH50）检测" +
                ",临床免疫检验\t补体受体-1测定" +
                ",临床免疫检验\t补体受体-2测定" +
                ",临床免疫检验\t补体受体-3β测定" +
                ",临床免疫检验\t成纤维细胞生长因子测定" +
                ",临床免疫检验\t雌激素受体测定" +
                ",临床免疫检验\t促红细胞生成素测定" +
                ",临床免疫检验\t单纯疱疹病毒抗体测定" +
                ",临床免疫检验\t单个补体成分的测定" +
                ",临床免疫检验\t单克隆免疫球蛋白测定" +
                ",临床免疫检验\t丁型肝炎病毒抗体测定" +
                ",临床免疫检验\t端粒酶活性测定" +
                ",临床免疫检验\t多胺测定" +
                ",临床免疫检验\t肥大细胞脱颗粒试验" +
                ",临床免疫检验\t非梅毒螺旋体血清学试验" +
                ",临床免疫检验\t封闭抗体测定" +
                ",临床免疫检验\t钙卫蛋白测定" +
                ",临床免疫检验\t干扰素-γ检测" +
                ",临床免疫检验\t肝细胞生长因子测定" +
                ",临床免疫检验\t高敏C-反应蛋白测定" +
                ",临床免疫检验\t固相酶免疫测定" +
                ",临床免疫检验\t核基质蛋白22测定" +
                ",临床免疫检验\t核糖核酸酶测定" +
                ",临床免疫检验\t红斑狼疮细胞测定" +
                ",临床免疫检验\t呼吸道合胞病毒抗体" +
                ",临床免疫检验\t降钙素原（PCT）检测（金标法PCT‐Q）" +
                ",临床免疫检验\t降钙素原（PCT）检测（仪器法）" +
                ",临床免疫检验\t甲胎蛋白L3测定" +
                ",临床免疫检验\t甲胎蛋白检测" +
                ",临床免疫检验\t甲胎蛋白异质体检测" +
                ",临床免疫检验\t甲型肝炎抗原测定" +
                ",临床免疫检验\t结核病血清学诊断" +
                ",临床免疫检验\t结核菌涂片检查" +
                ",临床免疫检验\t结合前列腺特异性抗原检测" +
                ",临床免疫检验\t集落刺激因子测定" +
                ",临床免疫检验\t基质金属蛋白酶9测定" +
                ",临床免疫检验\t巨噬细胞移动抑制试验" +
                ",临床免疫检验\t巨细胞病毒pp65抗原测定" +
                ",临床免疫检验\t抗C1q抗体检测" +
                ",临床免疫检验\t抗GP210抗体检测" +
                ",临床免疫检验\t抗Jo-1抗体测定" +
                ",临床免疫检验\t抗Ku抗体测定" +
                ",临床免疫检验\t抗M3毒蕈乙酰胆碱受体抗体测定" +
                ",临床免疫检验\t抗PM-Scl抗体测定" +
                ",临床免疫检验\t抗SS-A／Ro抗体测定" +
                ",临床免疫检验\t抗SS-B抗体测定" +
                ",临床免疫检验\t抗Sa抗体测定" +
                ",临床免疫检验\t抗Scl-70抗体测定" +
                ",临床免疫检验\t抗Sm抗体测定" +
                ",临床免疫检验\t抗U1RNP抗体测定" +
                ",临床免疫检验\t抗促甲状腺素受体抗体检测" +
                ",临床免疫检验\t抗单链DNA抗体" +
                ",临床免疫检验\t抗肝肾微粒体抗体测定" +
                ",临床免疫检验\t抗肝细胞膜抗体测定" +
                ",临床免疫检验\t抗肝细胞溶质抗原Ⅰ型抗体检测" +
                ",临床免疫检验\t抗谷氨酸脱羧酶抗体测定" +
                ",临床免疫检验\t抗骨骼肌抗体测定" +
                ",临床免疫检验\t抗核RNP抗体测定" +
                ",临床免疫检验\t抗核抗体检测" +
                ",临床免疫检验\t抗核膜抗体测定" +
                ",临床免疫检验\t抗核仁抗体" +
                ",临床免疫检验\t抗核糖体P蛋白抗体测定" +
                ",临床免疫检验\t抗核小体抗体检测" +
                ",临床免疫检验\t抗环瓜氨酸肽抗体检测" +
                ",临床免疫检验\t抗角蛋白抗体测定" +
                ",临床免疫检验\t抗甲状腺球蛋白与抗甲状腺过氧化物酶抗体检测" +
                ",临床免疫检验\t抗精子抗体检测" +
                ",临床免疫检验\t抗肌炎-硬皮病抗体测定" +
                ",临床免疫检验\t抗可溶性肝抗原/肝胰抗原-抗体检测" +
                ",临床免疫检验\t抗可溶性肝抗原抗体测定" +
                ",临床免疫检验\t抗可溶性酸性磷酸化核蛋白SP100抗体测定" +
                ",临床免疫检验\t抗可提取性核抗原测定" +
                ",临床免疫检验\t抗链球菌溶血素“O”测定" +
                ",临床免疫检验\t抗淋巴细胞抗体" +
                ",临床免疫检验\t抗卵巢抗体检测" +
                ",临床免疫检验\t抗内皮细胞抗体测定" +
                ",临床免疫检验\t抗内因子抗体检测" +
                ",临床免疫检验\t抗酿酒酵母抗体检测" +
                ",临床免疫检验\t抗平滑肌抗体检测" +
                ",临床免疫检验\t抗去唾液酸糖蛋白受体抗体检测" +
                ",临床免疫检验\t抗肾上腺皮质抗体测定" +
                ",临床免疫检验\t抗肾小球基底膜抗体检测" +
                ",临床免疫检验\t抗双链DNA抗体检测" +
                ",临床免疫检验\t抗体依赖性细胞介导的细胞毒测定" +
                ",临床免疫检验\t抗透明带抗体检测" +
                ",临床免疫检验\t抗胃壁细胞抗体检测" +
                ",临床免疫检验\t抗线粒体2型抗体测定" +
                ",临床免疫检验\t抗线粒体抗体检测" +
                ",临床免疫检验\t抗小肠杯状细胞抗体检测" +
                ",临床免疫检验\t抗胰岛素抗体检测" +
                ",临床免疫检验\t抗胰岛细胞抗体检测" +
                ",临床免疫检验\t抗乙酰胆碱受体抗体检测" +
                ",临床免疫检验\t抗胰腺腺泡抗体检测" +
                ",临床免疫检验\t抗增殖细胞核抗原抗体测定" +
                ",临床免疫检验\t抗着丝点抗体测定" +
                ",临床免疫检验\t抗中性粒细胞胞浆抗体测定" +
                ",临床免疫检验\t抗中性粒细胞胞质抗体检测——特异性ANCA" +
                ",临床免疫检验\t抗中性粒细胞胞质抗体检测——总ANCA" +
                ",临床免疫检验\t抗子宫内膜抗体测定" +
                ",临床免疫检验\t抗组蛋白抗体检测" +
                ",临床免疫检验\t抗α-胞衬蛋白抗体检测" +
                ",临床免疫检验\t抗β2-GP1抗体检测" +
                ",临床免疫检验\t抗“O”检测" +
                ",临床免疫检验\t可溶性白细胞介素-2受体检测" +
                ",临床免疫检验\t可溶性间皮素相关肽测定" +
                ",临床免疫检验\t可溶性转铁蛋白受体测定" +
                ",临床免疫检验\t快速红斑狼疮因子试验" +
                ",临床免疫检验\t快速尿素酶试验" +
                ",临床免疫检验\t类风湿因子检测" +
                ",临床免疫检验\t类黏蛋白肿瘤相关抗原测定" +
                ",临床免疫检验\t冷球蛋白检测" +
                ",临床免疫检验\t淋巴细胞毒试验" +
                ",临床免疫检验\t淋巴细胞功能亚群检测" +
                ",临床免疫检验\t淋巴细胞免疫表型检测" +
                ",临床免疫检验\t鳞状上皮细胞癌抗原检测" +
                ",临床免疫检验\t鳞状细胞癌抗原测定" +
                ",临床免疫检验\t螺内酯（安体舒通）试验" +
                ",临床免疫检验\t梅毒螺旋体血清学试验" +
                ",临床免疫检验\t酶联免疫吸附试验" +
                ",临床免疫检验\t免疫球蛋白A亚类测定" +
                ",临床免疫检验\t免疫球蛋白G4测定" +
                ",临床免疫检验\t免疫球蛋白G合成率" +
                ",临床免疫检验\t免疫球蛋白G亚类测定" +
                ",临床免疫检验\t免疫球蛋白测定" +
                ",临床免疫检验\t免疫球蛋白电泳" +
                ",临床免疫检验\t免疫球蛋白检测" +
                ",临床免疫检验\t末端脱氧核苷酸转移酶测定" +
                ",临床免疫检验\t脑脊液IgG鞘内合成率/24小时检测" +
                ",临床免疫检验\t内毒素检测" +
                ",临床免疫检验\t尿液κ/λ比" +
                ",临床免疫检验\t旁路途径的溶血活性测定" +
                ",临床免疫检验\t葡萄糖-6-磷酸异构酶测定" +
                ",临床免疫检验\t前列腺癌的血清肿瘤标记物" +
                ",临床免疫检验\t前列腺酸性磷酸酶测定" +
                ",临床免疫检验\t前列腺特异性膜抗原测定" +
                ",临床免疫检验\t群体反应性抗体测定" +
                ",临床免疫检验\t人表皮生长因子受体-2蛋白胞外区检测" +
                ",临床免疫检验\t人附睾蛋白4测定" +
                ",临床免疫检验\t人类白细胞抗原系统检测" +
                ",临床免疫检验\t人型支原体核酸检测" +
                ",临床免疫检验\t伤寒血清学检测" +
                ",临床免疫检验\t上皮钙黏附蛋白测定" +
                ",临床免疫检验\t生殖系统肿瘤标志物" +
                ",临床免疫检验\t生殖支原体核酸检测" +
                ",临床免疫检验\t神经生长因子测定" +
                ",临床免疫检验\t神经元特异烯醇化酶检测" +
                ",临床免疫检验\t嗜铬粒蛋白A测定" +
                ",临床免疫检验\t食物特异性IgG测定" +
                ",临床免疫检验\t糖蛋白类肿瘤标志物检测" +
                ",临床免疫检验\t糖类抗原125检测" +
                ",临床免疫检验\t糖类抗原19-5测定" +
                ",临床免疫检验\t糖类抗原19-9检测" +
                ",临床免疫检验\t糖类抗原242检测" +
                ",临床免疫检验\t糖类抗原27-29测定" +
                ",临床免疫检验\t糖类抗原50测定" +
                ",临床免疫检验\t糖类抗原549测定" +
                ",临床免疫检验\t糖类抗原72-4检测" +
                ",临床免疫检验\t糖链抗原15-3测定" +
                ",临床免疫检验\t特异性IgE检测" +
                ",临床免疫检验\t天疱疮抗体测定" +
                ",临床免疫检验\t体液免疫检测法" +
                ",临床免疫检验\t脱-γ-羧基凝血酶原测定" +
                ",临床免疫检验\t唾液酰基转移酶测定" +
                ",临床免疫检验\t外周血T淋巴细胞及其亚类监测" +
                ",临床免疫检验\t胃蛋白酶原Ⅱ测定" +
                ",临床免疫检验\t胃泌素释放肽前体检测" +
                ",临床免疫检验\t硝基四唑氮蓝试验" +
                ",临床免疫检验\t细胞角蛋白19片段21-1测定" +
                ",临床免疫检验\t细胞角蛋白19片段测定" +
                ",临床免疫检验\t细胞角蛋白19片段检测" +
                ",临床免疫检验\t细胞因子与细胞黏附因子的测定" +
                ",临床免疫检验\t性病研究实验室试验" +
                ",临床免疫检验\t性传播疾病免疫检测" +
                ",临床免疫检验\t胸苷激酶测定" +
                ",临床免疫检验\t雄激素受体测定" +
                ",临床免疫检验\t选择素族黏附分子测定" +
                ",临床免疫检验\t血管内皮生长因子测定" +
                ",临床免疫检验\t血清C反应蛋白测定" +
                ",临床免疫检验\t血清IgG亚类检测" +
                ",临床免疫检验\t血清淀粉样蛋白A检测" +
                ",临床免疫检验\t血清免疫球蛋白A测定" +
                ",临床免疫检验\t血清免疫球蛋白D测定" +
                ",临床免疫检验\t血清免疫球蛋白E测定" +
                ",临床免疫检验\t血清免疫球蛋白G测定" +
                ",临床免疫检验\t血清κ/λ比" +
                ",临床免疫检验\t血清、脑脊液免疫球蛋白A测定" +
                ",临床免疫检验\t血清、脑脊液免疫球蛋白M测定" +
                ",临床免疫检验\t血清、随机尿、脑脊液新蝶呤测定" +
                ",临床免疫检验\t循环核酸测定" +
                ",临床免疫检验\t循环免疫复合物检测" +
                ",临床免疫检验\t循环肿瘤细胞测定" +
                ",临床免疫检验\t液基薄层细胞制作技术" +
                ",临床免疫检验\t胰癌胚抗原测定" +
                ",临床免疫检验\t胰岛素样生长因子测定" +
                ",临床免疫检验\t胰岛素样生长因子结合蛋白-3测定" +
                ",临床免疫检验\t乙肝五项" +
                ",临床免疫检验\t乙醛脱氢酶2多态性的检测" +
                ",临床免疫检验\t胰腺癌相关抗原-2测定" +
                ",临床免疫检验\t乙酰胆碱酯酶抗体测定" +
                ",临床免疫检验\t胰腺肿瘤标记物" +
                ",临床免疫检验\t乙型肝炎病毒前S<sub>1</sub>抗原及前S<sub>1</sub>抗体" +
                ",临床免疫检验\t游离前列腺特异性抗原检测" +
                ",临床免疫检验\t游离轻链检测" +
                ",临床免疫检验\t幽门螺旋杆菌抗体检查" +
                ",临床免疫检验\t孕激素受体测定" +
                ",临床免疫检验\t孕酮受体测定" +
                ",临床免疫检验\t脂质相关唾液酸测定" +
                ",临床免疫检验\t肿瘤坏死因子-α检测" +
                ",临床免疫检验\t肿瘤特异性生长因子" +
                ",临床免疫检验\t肿瘤相关抗原TA90测定" +
                ",临床免疫检验\t中性粒细胞黏附功能检测" +
                ",临床免疫检验\t中性粒细胞趋化功能检测" +
                ",临床免疫检验\t中性粒细胞吞噬与杀菌功能检测" +
                ",临床免疫检验\t转化生长因子-β检测" +
                ",临床免疫检验\t主要组织相容性抗原" +
                ",临床免疫检验\t自然杀伤细胞活性检测" +
                ",临床免疫检验\t自身抗体检测" +
                ",临床免疫检验\t总IgE检测" +
                ",临床免疫检验\t总补体溶血活性测定" +
                ",临床免疫检验\t总前列腺特异性抗原检测" +
                ",临床免疫检验\t总溶血补体活性测定" +
                ",临床免疫检验\t组织多肽抗原测定" +
                ",临床微生物与寄生虫检验\t2-酮基葡萄糖酸盐试验" +
                ",临床微生物与寄生虫检验\t3-酮基乳糖酸盐试验" +
                ",临床微生物与寄生虫检验\tβ-D-葡聚糖试验" +
                ",临床微生物与寄生虫检验\tβ-半乳糖苷酶试验" +
                ",临床微生物与寄生虫检验\t（1→3）‐β‐D‐葡聚糖检测（G－试验）" +
                ",临床微生物与寄生虫检验\tATB FUNGUS 3酵母菌药敏试验" +
                ",临床微生物与寄生虫检验\tBD Phoenix‐100细菌鉴定／药敏" +
                ",临床微生物与寄生虫检验\tCAMP试验" +
                ",临床微生物与寄生虫检验\tCAMP抑制试验" +
                ",临床微生物与寄生虫检验\tDNA酶试验" +
                ",临床微生物与寄生虫检验\tEB病毒抗体测定" +
                ",临床微生物与寄生虫检验\tEpstein-Barr病毒检验" +
                ",临床微生物与寄生虫检验\tHIV-1核酸定量检测" +
                ",临床微生物与寄生虫检验\tHIV-1核酸定性检测" +
                ",临床微生物与寄生虫检验\tHIV抗体确证试验" +
                ",临床微生物与寄生虫检验\tHIV抗体筛查试验" +
                ",临床微生物与寄生虫检验\tHPV病理学检查" +
                ",临床微生物与寄生虫检验\tHPV核酸检测" +
                ",临床微生物与寄生虫检验\tKoga 培养法" +
                ",临床微生物与寄生虫检验\tL型沙眼衣原体核酸检测" +
                ",临床微生物与寄生虫检验\tL型沙眼衣原体基因测序分型法" +
                ",临床微生物与寄生虫检验\tO/129抑菌试验" +
                ",临床微生物与寄生虫检验\tOptochin敏感试验" +
                ",临床微生物与寄生虫检验\tSARS冠状病毒检验" +
                ",临床微生物与寄生虫检验\tTORCH检测" +
                ",临床微生物与寄生虫检验\tVP试验" +
                ",临床微生物与寄生虫检验\tXpert MTB/RIF标准操作规程" +
                ",临床微生物与寄生虫检验\t埃可病毒抗体测定" +
                ",临床微生物与寄生虫检验\t埃立克体属检验" +
                ",临床微生物与寄生虫检验\t埃希菌属检验" +
                ",临床微生物与寄生虫检验\t氨基酸脱氨酶试验" +
                ",临床微生物与寄生虫检验\t氨基酸脱羧酶试验" +
                ",临床微生物与寄生虫检验\t氨基糖苷类高水平耐药（HL AR）肠球菌检测" +
                ",临床微生物与寄生虫检验\t暗色真菌检验" +
                ",临床微生物与寄生虫检验\t斑点酶联免疫吸附试验" +
                ",临床微生物与寄生虫检验\t棒杆菌属检验" +
                ",临床微生物与寄生虫检验\t半乳甘露聚糖抗原试验" +
                ",临床微生物与寄生虫检验\t饱和盐水浮聚法" +
                ",临床微生物与寄生虫检验\t鲍特菌属检验" +
                ",临床微生物与寄生虫检验\t巴斯德菌属检验" +
                ",临床微生物与寄生虫检验\t巴通体属检验" +
                ",临床微生物与寄生虫检验\t病毒常规检测" +
                ",临床微生物与寄生虫检验\t病毒蛋白抗原检测" +
                ",临床微生物与寄生虫检验\t病毒感染的实验室检查" +
                ",临床微生物与寄生虫检验\t病毒培养与鉴定" +
                ",临床微生物与寄生虫检验\t丙二酸盐利用试验" +
                ",临床微生物与寄生虫检验\t丙型肝炎病毒RNA定性检测" +
                ",临床微生物与寄生虫检验\t丙型肝炎病毒抗体检测" +
                ",临床微生物与寄生虫检验\t并殖吸虫病检验" +
                ",临床微生物与寄生虫检验\t伯克霍尔德菌属检验" +
                ",临床微生物与寄生虫检验\t伯氏疏螺旋体抗体测定" +
                ",临床微生物与寄生虫检验\t不动杆菌属检验" +
                ",临床微生物与寄生虫检验\t布鲁菌属检验" +
                ",临床微生物与寄生虫检验\t布氏杆菌抗体凝集试验" +
                ",临床微生物与寄生虫检验\t肠道病毒培养和鉴定" +
                ",临床微生物与寄生虫检验\t肠道病毒通用型（EV）核酸检测" +
                ",临床微生物与寄生虫检验\t肠杆菌科菌碳青霉烯酶检测" +
                ",临床微生物与寄生虫检验\t肠杆菌属、枸橼酸杆菌属、沙雷菌属、变形杆菌属检验" +
                ",临床微生物与寄生虫检验\t肠球菌属检验" +
                ",临床微生物与寄生虫检验\t肠绦虫相关检验" +
                ",临床微生物与寄生虫检验\t常用的抗菌药物敏感试验" +
                ",临床微生物与寄生虫检验\t产碱杆菌属检验" +
                ",临床微生物与寄生虫检验\t产青霉素酶的淋球菌的检测" +
                ",临床微生物与寄生虫检验\t穿刺液标本的微生物检验" +
                ",临床微生物与寄生虫检验\t传染病四项" +
                ",临床微生物与寄生虫检验\t触酶试验" +
                ",临床微生物与寄生虫检验\t醋酸盐利用试验" +
                ",临床微生物与寄生虫检验\t大环内酯‐林可霉素‐链阳霉素B（MLS<sub>B</sub>）耐药检测" +
                ",临床微生物与寄生虫检验\t单纯疱疹病毒1型和2型抗原检测" +
                ",临床微生物与寄生虫检验\t单纯疱疹病毒检验" +
                ",临床微生物与寄生虫检验\t单纯疱疹病毒Ⅱ型核酸检测" +
                ",临床微生物与寄生虫检验\t丹毒丝菌属检验" +
                ",临床微生物与寄生虫检验\t胆汁溶菌试验" +
                ",临床微生物与寄生虫检验\t导管的微生物检验" +
                ",临床微生物与寄生虫检验\t淀粉水解试验" +
                ",临床微生物与寄生虫检验\t定量透明法检验寄生虫" +
                ",临床微生物与寄生虫检验\t丁型肝炎抗原" +
                ",临床微生物与寄生虫检验\t动力试验" +
                ",临床微生物与寄生虫检验\t对流免疫电泳试验" +
                ",临床微生物与寄生虫检验\t杜克雷嗜血杆菌的分离培养" +
                ",临床微生物与寄生虫检验\t杜克雷嗜血杆菌的鉴定" +
                ",临床微生物与寄生虫检验\t杜克雷嗜血杆菌显微镜检查" +
                ",临床微生物与寄生虫检验\t放线菌属检验" +
                ",临床微生物与寄生虫检验\t肺孢子虫检验" +
                ",临床微生物与寄生虫检验\t肥达反应" +
                ",临床微生物与寄生虫检验\t非结核分枝杆菌检验" +
                ",临床微生物与寄生虫检验\t肺螨虫检验" +
                ",临床微生物与寄生虫检验\t肺炎链球菌检验" +
                ",临床微生物与寄生虫检验\t肺炎支原体分离培养及鉴定" +
                ",临床微生物与寄生虫检验\t肺炎支原体抗体测定" +
                ",临床微生物与寄生虫检验\t肺炎支原体商品化检测系统" +
                ",临床微生物与寄生虫检验\t肺炎支原体血清学试验" +
                ",临床微生物与寄生虫检验\t粪便标本的微生物检验" +
                ",临床微生物与寄生虫检验\t粪便标本细菌检验" +
                ",临床微生物与寄生虫检验\t粪便难辨梭状芽胞杆菌检验" +
                ",临床微生物与寄生虫检验\t粪便直接涂片法" +
                ",临床微生物与寄生虫检验\t粪类圆线虫检验" +
                ",临床微生物与寄生虫检验\t分枝杆菌金胺O染色" +
                ",临床微生物与寄生虫检验\t分枝杆菌罗氏培养" +
                ",临床微生物与寄生虫检验\t分枝杆菌萋尼染色" +
                ",临床微生物与寄生虫检验\t分枝杆菌液体培养" +
                ",临床微生物与寄生虫检验\t弗朗西斯菌属检验" +
                ",临床微生物与寄生虫检验\t副流感病毒抗体测定" +
                ",临床微生物与寄生虫检验\t改良加藤厚涂片法" +
                ",临床微生物与寄生虫检验\t肛门拭子法与肛周蛲虫检查法" +
                ",临床微生物与寄生虫检验\t杆菌肽敏感试验" +
                ",临床微生物与寄生虫检验\t肝炎病毒检验" +
                ",临床微生物与寄生虫检验\t甘油复红试验" +
                ",临床微生物与寄生虫检验\t高危型人乳头瘤病毒（HPV）基因分型定量检测" +
                ",临床微生物与寄生虫检验\t革兰染色" +
                ",临床微生物与寄生虫检验\t革兰阳性无芽胞厌氧杆菌和放线菌属检验" +
                ",临床微生物与寄生虫检验\t革兰阴性杆菌葡萄糖代谢类型鉴别试验" +
                ",临床微生物与寄生虫检验\t汞碘醛离心沉淀法" +
                ",临床微生物与寄生虫检验\t弓形虫IgG检测" +
                ",临床微生物与寄生虫检验\t弓形虫检验" +
                ",临床微生物与寄生虫检验\t弓形虫筛查" +
                ",临床微生物与寄生虫检验\t弓形体抗体测定" +
                ",临床微生物与寄生虫检验\t钩虫相关检验" +
                ",临床微生物与寄生虫检验\t钩端螺旋体乳胶凝集试验" +
                ",临床微生物与寄生虫检验\t钩蚴培养法" +
                ",临床微生物与寄生虫检验\t枸橼酸盐利用试验" +
                ",临床微生物与寄生虫检验\t广州管圆线虫检验" +
                ",临床微生物与寄生虫检验\t关节液培养" +
                ",临床微生物与寄生虫检验\t灌洗液培养" +
                ",临床微生物与寄生虫检验\t汉坦病毒检验" +
                ",临床微生物与寄生虫检验\t红酵母属检验" +
                ",临床微生物与寄生虫检验\t环蚴沉淀试验" +
                ",临床微生物与寄生虫检验\t华支睾吸虫检验" +
                ",临床微生物与寄生虫检验\t蛔虫检验" +
                ",临床微生物与寄生虫检验\t弧菌属检验" +
                ",临床微生物与寄生虫检验\t霍乱红试验" +
                ",临床微生物与寄生虫检验\t霍乱弧菌检验" +
                ",临床微生物与寄生虫检验\t呼吸道标本细菌检验" +
                ",临床微生物与寄生虫检验\t呼吸道病毒培养和鉴定" +
                ",临床微生物与寄生虫检验\t呼吸道合胞病毒（RSV）核酸检测" +
                ",临床微生物与寄生虫检验\t假单胞菌属检验" +
                ",临床微生物与寄生虫检验\t贾第虫检验" +
                ",临床微生物与寄生虫检验\t甲基红试验" +
                ",临床微生物与寄生虫检验\t荚膜肿胀试验" +
                ",临床微生物与寄生虫检验\t浆膜腔积液寄生虫检查" +
                ",临床微生物与寄生虫检验\t姜片虫相关检验" +
                ",临床微生物与寄生虫检验\t间接法ELISA检测TORCH特异性IgG抗体" +
                ",临床微生物与寄生虫检验\t间接血凝试验" +
                ",临床微生物与寄生虫检验\t甲型H1N1流感病毒核酸检测" +
                ",临床微生物与寄生虫检验\t甲型肝炎病毒IgM抗体（HAV IgM）测定" +
                ",临床微生物与寄生虫检验\t甲型流感病毒抗原检测（胶体金法）" +
                ",临床微生物与寄生虫检验\t甲氧西林耐药葡萄球菌（MRS）检测" +
                ",临床微生物与寄生虫检验\t结核分枝杆菌RNA检测" +
                ",临床微生物与寄生虫检验\t结核分枝杆菌检验" +
                ",临床微生物与寄生虫检验\t结核分枝杆菌抗体检测" +
                ",临床微生物与寄生虫检验\t接合菌检验" +
                ",临床微生物与寄生虫检验\t疥螨检查" +
                ",临床微生物与寄生虫检验\t解脲脲原体RNA检测" +
                ",临床微生物与寄生虫检验\t解脲脲原体分离培养及鉴定" +
                ",临床微生物与寄生虫检验\t解脲支原体核酸检测" +
                ",临床微生物与寄生虫检验\t精氨酸双水解酶（ADH）试验" +
                ",临床微生物与寄生虫检验\t棘球蚴检验" +
                ",临床微生物与寄生虫检验\t寄生虫标本的采集、保存与鉴定" +
                ",临床微生物与寄生虫检验\t寄生虫肠黏膜活检" +
                ",临床微生物与寄生虫检验\t寄生虫单向琼脂扩散试验" +
                ",临床微生物与寄生虫检验\t寄生虫碘液染色法" +
                ",临床微生物与寄生虫检验\t寄生虫肺组织活检" +
                ",临床微生物与寄生虫检验\t寄生虫骨髓穿刺涂片活检" +
                ",临床微生物与寄生虫检验\t寄生虫间接血凝试验" +
                ",临床微生物与寄生虫检验\t寄生虫淋巴结穿刺涂片活检" +
                ",临床微生物与寄生虫检验\t寄生虫皮下及肌肉活组织检查" +
                ",临床微生物与寄生虫检验\t寄生虫乳胶凝集试验" +
                ",临床微生物与寄生虫检验\t寄生虫双向琼脂扩散试验" +
                ",临床微生物与寄生虫检验\t寄生虫痰液检查法" +
                ",临床微生物与寄生虫检验\t寄生虫血膜染色法" +
                ",临床微生物与寄生虫检验\t军团菌属检验" +
                ",临床微生物与寄生虫检验\t军团菌血清特异性抗体检测" +
                ",临床微生物与寄生虫检验\t巨细胞病毒检验" +
                ",临床微生物与寄生虫检验\t巨细胞病毒抗原血症（CMVpp65）检测" +
                ",临床微生物与寄生虫检验\t巨细胞病毒离心小瓶培养法" +
                ",临床微生物与寄生虫检验\t巨细胞病毒免疫检测" +
                ",临床微生物与寄生虫检验\t抗EB病毒衣壳抗原IgA测定" +
                ",临床微生物与寄生虫检验\t抗EB病毒衣壳抗原IgG抗体测定" +
                ",临床微生物与寄生虫检验\t抗EB病毒衣壳抗原IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗EB病毒早期抗原IgA抗体测定" +
                ",临床微生物与寄生虫检验\t抗HSV1型‐IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗HSV2型‐IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗单纯疱疹病毒1型IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗单纯疱疹病毒2型IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗风疹病毒IgG抗体测定" +
                ",临床微生物与寄生虫检验\t抗风疹病毒IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗弓形虫IgG抗体测定" +
                ",临床微生物与寄生虫检验\t抗弓形虫IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗巨细胞病毒IgG抗体测定" +
                ",临床微生物与寄生虫检验\t抗巨细胞病毒IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗麻疹病毒IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗沙眼衣原体、肺炎衣原体和鹦鹉热衣原体IgG和IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗嗜肺军团菌抗体IgG检测" +
                ",临床微生物与寄生虫检验\t抗嗜肺军团菌抗体IgM检测" +
                ",临床微生物与寄生虫检验\t抗体捕获法ELISA检测TORCH特异性IgM抗体" +
                ",临床微生物与寄生虫检验\t卡他莫拉菌检验" +
                ",临床微生物与寄生虫检验\t克雷伯菌属和拉乌尔菌属检验" +
                ",临床微生物与寄生虫检验\t柯萨奇病毒A16型（CA16）核酸检测" +
                ",临床微生物与寄生虫检验\t柯萨奇病毒中和试验" +
                ",临床微生物与寄生虫检验\t克氏双糖铁琼脂试验" +
                ",临床微生物与寄生虫检验\t快速血浆反应素环状卡片试验" +
                ",临床微生物与寄生虫检验\t快速β内酰胺酶试验" +
                ",临床微生物与寄生虫检验\t蓝氏贾第鞭毛虫培养" +
                ",临床微生物与寄生虫检验\t镰刀菌检验" +
                ",临床微生物与寄生虫检验\t链球菌属检验" +
                ",临床微生物与寄生虫检验\t立克次体属检验" +
                ",临床微生物与寄生虫检验\t淋病奈瑟菌检验" +
                ",临床微生物与寄生虫检验\t淋病奈瑟球菌显微镜检查" +
                ",临床微生物与寄生虫检验\t淋病双球菌镜检" +
                ",临床微生物与寄生虫检验\t淋病双球菌培养" +
                ",临床微生物与寄生虫检验\t临床常用的细菌抗原检测试验" +
                ",临床微生物与寄生虫检验\t淋球菌RNA检测" +
                ",临床微生物与寄生虫检验\t淋球菌的初步鉴定：氧化酶试验" +
                ",临床微生物与寄生虫检验\t淋球菌的分离培养" +
                ",临床微生物与寄生虫检验\t淋球菌的确认鉴定：糖发酵试验" +
                ",临床微生物与寄生虫检验\t淋球菌定性药敏检测（纸片扩散法）" +
                ",临床微生物与寄生虫检验\t淋球菌免疫学检查" +
                ",临床微生物与寄生虫检验\t利什曼原虫检验" +
                ",临床微生物与寄生虫检验\t利什曼原虫培养" +
                ",临床微生物与寄生虫检验\t李斯特菌属检验" +
                ",临床微生物与寄生虫检验\t六胺银染色" +
                ",临床微生物与寄生虫检验\t流感嗜血杆菌检验" +
                ",临床微生物与寄生虫检验\t硫化氢（H<sub>2</sub>S）试验" +
                ",临床微生物与寄生虫检验\t硫酸锌离心浮聚法" +
                ",临床微生物与寄生虫检验\t流行性出血热病毒血清学检查" +
                ",临床微生物与寄生虫检验\t流行性感冒病毒检验" +
                ",临床微生物与寄生虫检验\t留置性导管标本细菌检验" +
                ",临床微生物与寄生虫检验\t卵磷脂酶试验和聂格尔试验" +
                ",临床微生物与寄生虫检验\t轮状病毒离心小瓶培养法" +
                ",临床微生物与寄生虫检验\t轮状病毒试管培养法" +
                ",临床微生物与寄生虫检验\t轮状病毒实时定量PCR检测" +
                ",临床微生物与寄生虫检验\t螺杆菌属检验" +
                ",临床微生物与寄生虫检验\t螺旋体检验" +
                ",临床微生物与寄生虫检验\t螺旋体形态学检查" +
                ",临床微生物与寄生虫检验\t麻风杆菌皮肤涂片检查" +
                ",临床微生物与寄生虫检验\t马尿酸钠水解试验" +
                ",临床微生物与寄生虫检验\t毛孢子菌属检验" +
                ",临床微生物与寄生虫检验\t毛蚴孵化法" +
                ",临床微生物与寄生虫检验\t梅毒病原学（梅毒螺旋体）检测" +
                ",临床微生物与寄生虫检验\t梅毒螺旋体抗体检测" +
                ",临床微生物与寄生虫检验\t梅毒螺旋体明胶颗粒凝集试验" +
                ",临床微生物与寄生虫检验\t酶联免疫吸附试验" +
                ",临床微生物与寄生虫检验\t免疫荧光标记技术" +
                ",临床微生物与寄生虫检验\t明胶液化试验" +
                ",临床微生物与寄生虫检验\t泌尿生殖道分泌物培养" +
                ",临床微生物与寄生虫检验\t莫拉菌属检验" +
                ",临床微生物与寄生虫检验\t墨汁染色" +
                ",临床微生物与寄生虫检验\t难辨梭菌毒素A＋B检测" +
                ",临床微生物与寄生虫检验\t囊尾蚴检验" +
                ",临床微生物与寄生虫检验\t蛲虫检验" +
                ",临床微生物与寄生虫检验\t脑脊液标本细菌检验" +
                ",临床微生物与寄生虫检验\t脑脊液的微生物检验" +
                ",临床微生物与寄生虫检验\t脑脊液中寄生虫检查" +
                ",临床微生物与寄生虫检验\t脑膜炎奈瑟菌检验" +
                ",临床微生物与寄生虫检验\t黏丝试验" +
                ",临床微生物与寄生虫检验\t念珠菌属检验" +
                ",临床微生物与寄生虫检验\t念珠菌药物敏感性测定" +
                ",临床微生物与寄生虫检验\t尿素酶试验" +
                ",临床微生物与寄生虫检验\t尿液标本的微生物检验" +
                ",临床微生物与寄生虫检验\t尿液标本细菌检验" +
                ",临床微生物与寄生虫检验\t尿液真菌培养" +
                ",临床微生物与寄生虫检验\t尿液中寄生虫检查" +
                ",临床微生物与寄生虫检验\t拟杆菌属检验" +
                ",临床微生物与寄生虫检验\t尼龙袋集卵法" +
                ",临床微生物与寄生虫检验\t凝固酶试验" +
                ",临床微生物与寄生虫检验\t脓液及创伤感染分泌物细菌学检查" +
                ",临床微生物与寄生虫检验\t疟原虫检验" +
                ",临床微生物与寄生虫检验\t疟原虫培养" +
                ",临床微生物与寄生虫检验\t诺卡菌检验" +
                ",临床微生物与寄生虫检验\t诺卡菌属检验" +
                ",临床微生物与寄生虫检验\t疱疹病毒培养和鉴定" +
                ",临床微生物与寄生虫检验\t皮肤癣菌检验" +
                ",临床微生物与寄生虫检验\t葡萄球菌属检验" +
                ",临床微生物与寄生虫检验\t葡萄糖代谢类型鉴别试验" +
                ",临床微生物与寄生虫检验\t鞘膜积液寄生虫检查" +
                ",临床微生物与寄生虫检验\t气单胞菌科检验" +
                ",临床微生物与寄生虫检验\t氰化钾抑制试验" +
                ",临床微生物与寄生虫检验\t青霉素耐药肺炎链球菌（PRSP）检测" +
                ",临床微生物与寄生虫检验\t氢氧化钾拉丝试验" +
                ",临床微生物与寄生虫检验\t琼脂稀释法淋球菌药敏检测" +
                ",临床微生物与寄生虫检验\t其他肠杆菌科细菌检验" +
                ",临床微生物与寄生虫检验\t七叶苷水解试验" +
                ",临床微生物与寄生虫检验\t醛醚沉淀法" +
                ",临床微生物与寄生虫检验\t曲霉菌属检验" +
                ",临床微生物与寄生虫检验\t染料抑制试验" +
                ",临床微生物与寄生虫检验\t人类轮状病毒检验" +
                ",临床微生物与寄生虫检验\t人类免疫缺陷病毒抗体（HIV1＋2）及抗原（HIV1p24）联合检测" +
                ",临床微生物与寄生虫检验\t人类免疫缺陷性病毒检验" +
                ",临床微生物与寄生虫检验\t人类乳头状瘤病毒检验" +
                ",临床微生物与寄生虫检验\t人芽囊原虫培养" +
                ",临床微生物与寄生虫检验\t溶组织内阿米巴检验" +
                ",临床微生物与寄生虫检验\t溶组织内阿米巴培养" +
                ",临床微生物与寄生虫检验\t朊病毒检验" +
                ",临床微生物与寄生虫检验\t弱抗酸染色" +
                ",临床微生物与寄生虫检验\t蠕形螨检查" +
                ",临床微生物与寄生虫检验\t蠕形螨检验" +
                ",临床微生物与寄生虫检验\t赛多孢霉检验" +
                ",临床微生物与寄生虫检验\t沙门菌属检验" +
                ",临床微生物与寄生虫检验\t伤口、脓液及组织标本细菌检验" +
                ",临床微生物与寄生虫检验\t沙眼衣原体RNA检测" +
                ",临床微生物与寄生虫检验\t沙眼衣原体抗原检测" +
                ",临床微生物与寄生虫检验\t生殖道标本的微生物检验" +
                ",临床微生物与寄生虫检验\t生殖道标本细菌检验" +
                ",临床微生物与寄生虫检验\t生殖道标本真菌镜检" +
                ",临床微生物与寄生虫检验\t生殖道念珠菌培养鉴定法" +
                ",临床微生物与寄生虫检验\t生殖道念珠菌显微镜检法" +
                ",临床微生物与寄生虫检验\t生殖道沙眼衣原体细胞培养" +
                ",临床微生物与寄生虫检验\t生殖道支原体分离培养" +
                ",临床微生物与寄生虫检验\t生殖道支原体核酸检测" +
                ",临床微生物与寄生虫检验\t舌形虫检验" +
                ",临床微生物与寄生虫检验\t十二指肠液和胆汁检验寄生虫" +
                ",临床微生物与寄生虫检验\t试管凝集反应" +
                ",临床微生物与寄生虫检验\t噬菌体鉴定细菌" +
                ",临床微生物与寄生虫检验\t石蕊牛乳试验" +
                ",临床微生物与寄生虫检验\t实时定量PCR法检测TORCH特异性核酸" +
                ",临床微生物与寄生虫检验\t嗜血杆菌属检验" +
                ",临床微生物与寄生虫检验\t实验动物中卫氏并殖吸虫循环抗原（CAg）动力学观察" +
                ",临床微生物与寄生虫检验\t双相真菌检验" +
                ",临床微生物与寄生虫检验\t水洗沉淀法" +
                ",临床微生物与寄生虫检验\t术前八项" +
                ",临床微生物与寄生虫检验\t丝虫检验" +
                ",临床微生物与寄生虫检验\t梭菌属检验" +
                ",临床微生物与寄生虫检验\t糖（醇）类发酵试验" +
                ",临床微生物与寄生虫检验\t痰细菌培养的前处理" +
                ",临床微生物与寄生虫检验\t痰液病原体检查" +
                ",临床微生物与寄生虫检验\t梯度扩散法淋球菌药敏检测" +
                ",临床微生物与寄生虫检验\t铁苏木素染色法" +
                ",临床微生物与寄生虫检验\t体外抗生素敏感试验方法" +
                ",临床微生物与寄生虫检验\t体液标本细菌检验" +
                ",临床微生物与寄生虫检验\t透明胶带法丝状真菌镜检" +
                ",临床微生物与寄生虫检验\t透明质酸酶试验" +
                ",临床微生物与寄生虫检验\t土拉伦斯杆菌凝集试验" +
                ",临床微生物与寄生虫检验\t外斐反应" +
                ",临床微生物与寄生虫检验\t万古霉素耐药肠球菌（VRE）检测" +
                ",临床微生物与寄生虫检验\t弯曲菌属检验" +
                ",临床微生物与寄生虫检验\t微量肉汤稀释法淋球菌药敏检测" +
                ",临床微生物与寄生虫检验\t微球菌属检验" +
                ",临床微生物与寄生虫检验\t微丝蚴血检法" +
                ",临床微生物与寄生虫检验\t尾蚴膜反应" +
                ",临床微生物与寄生虫检验\t温度生长试验" +
                ",临床微生物与寄生虫检验\t戊型肝炎病毒IgG抗体测定" +
                ",临床微生物与寄生虫检验\t戊型肝炎病毒IgM抗体（HEV IgM）测定" +
                ",临床微生物与寄生虫检验\t无形体属检验" +
                ",临床微生物与寄生虫检验\t下呼吸道标本的处理和微生物检验" +
                ",临床微生物与寄生虫检验\t腺病毒检验" +
                ",临床微生物与寄生虫检验\t消化链球菌属检验" +
                ",临床微生物与寄生虫检验\t消化球菌属检验" +
                ",临床微生物与寄生虫检验\t硝酸盐还原试验" +
                ",临床微生物与寄生虫检验\t细菌感染性疾病的免疫检测" +
                ",临床微生物与寄生虫检验\t细菌内毒素检查法" +
                ",临床微生物与寄生虫检验\t细菌培养及鉴定" +
                ",临床微生物与寄生虫检验\t细菌性痢疾检验" +
                ",临床微生物与寄生虫检验\t性病性淋巴肉芽肿血清学检测" +
                ",临床微生物与寄生虫检验\t新生霉素敏感试验" +
                ",临床微生物与寄生虫检验\t新型冠状病毒核酸检测" +
                ",临床微生物与寄生虫检验\t旋毛虫检验" +
                ",临床微生物与寄生虫检验\t血培养" +
                ",临床微生物与寄生虫检验\t血吸虫环卵沉淀试验" +
                ",临床微生物与寄生虫检验\t血吸虫间接血凝试验" +
                ",临床微生物与寄生虫检验\t血吸虫检验" +
                ",临床微生物与寄生虫检验\t血吸虫毛蚴孵化法" +
                ",临床微生物与寄生虫检验\t血吸虫尼龙绢筛集卵法" +
                ",临床微生物与寄生虫检验\t血液及骨髓标本细菌检验" +
                ",临床微生物与寄生虫检验\t血液及骨髓细菌培养" +
                ",临床微生物与寄生虫检验\t血液寄生虫检查" +
                ",临床微生物与寄生虫检验\t芽胞杆菌属检验" +
                ",临床微生物与寄生虫检验\t氧化酶试验" +
                ",临床微生物与寄生虫检验\t氧化酶试验（改良法）" +
                ",临床微生物与寄生虫检验\t厌氧菌鉴定流程" +
                ",临床微生物与寄生虫检验\t厌氧菌检验" +
                ",临床微生物与寄生虫检验\t眼、耳标本细菌检验" +
                ",临床微生物与寄生虫检验\t耶尔森菌属检验" +
                ",临床微生物与寄生虫检验\t隐孢子虫检验" +
                ",临床微生物与寄生虫检验\t隐孢子虫卵囊染色检查" +
                ",临床微生物与寄生虫检验\t阴道加德纳菌检验" +
                ",临床微生物与寄生虫检验\t阴道毛滴虫培养" +
                ",临床微生物与寄生虫检验\t阴道、前列腺液分泌物寄生虫检查" +
                ",临床微生物与寄生虫检验\t吲哚试验" +
                ",临床微生物与寄生虫检验\t荧光梅毒螺旋体抗体吸收试验FTA‐ABS IgG" +
                ",临床微生物与寄生虫检验\t荧光梅毒螺旋体抗体吸收试验FTA‐ABS IgM" +
                ",临床微生物与寄生虫检验\t隐秘杆菌属检验" +
                ",临床微生物与寄生虫检验\t隐球菌抗原乳胶凝集试验" +
                ",临床微生物与寄生虫检验\t隐球菌属检验" +
                ",临床微生物与寄生虫检验\t阴虱检查" +
                ",临床微生物与寄生虫检验\t阴虱镜检" +
                ",临床微生物与寄生虫检验\t乙酰胺利用试验" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒表面抗体定量检测" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒表面抗原检测" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒e抗体测定" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒e抗原测定" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒核心抗体测定" +
                ",临床微生物与寄生虫检验\t医院环境卫生监测标本细菌检验" +
                ",临床微生物与寄生虫检验\t衣原体Giemsa染色" +
                ",临床微生物与寄生虫检验\t衣原体抗原测定" +
                ",临床微生物与寄生虫检验\t衣原体抗原快速金标检测" +
                ",临床微生物与寄生虫检验\t衣原体酶免疫检测" +
                ",临床微生物与寄生虫检验\t衣原体属检验" +
                ",临床微生物与寄生虫检验\t衣原体微量免疫荧光试验" +
                ",临床微生物与寄生虫检验\t衣原体直接免疫荧光染色" +
                ",临床微生物与寄生虫检验\t幽门螺旋杆菌（Hp）核酸检测" +
                ",临床微生物与寄生虫检验\t窄食单胞菌属检验" +
                ",临床微生物与寄生虫检验\t真菌Etest药敏试验" +
                ",临床微生物与寄生虫检验\t真菌KOH压片" +
                ",临床微生物与寄生虫检验\t真菌分离培养" +
                ",临床微生物与寄生虫检验\t真菌感染的实验室检查" +
                ",临床微生物与寄生虫检验\t真菌鉴定流程" +
                ",临床微生物与寄生虫检验\t真菌检验" +
                ",临床微生物与寄生虫检验\t真菌染色标本镜检" +
                ",临床微生物与寄生虫检验\t真菌小培养" +
                ",临床微生物与寄生虫检验\t真菌直接标本镜检" +
                ",临床微生物与寄生虫检验\t制动试验" +
                ",临床微生物与寄生虫检验\t志贺菌属检验" +
                ",临床微生物与寄生虫检验\t志贺菌属（痢疾杆菌属）的系统鉴定方法" +
                ",临床微生物与寄生虫检验\t脂酶试验" +
                ",临床微生物与寄生虫检验\t纸片扩散法药敏试验" +
                ",临床微生物与寄生虫检验\t支原体培养及药敏" +
                ",临床微生物与寄生虫检验\t支原体属和脲原体属检验" +
                ",临床微生物与寄生虫检验\t锥虫检验" +
                ",临床微生物与寄生虫检验\t猪囊虫抗体测定" +
                ",临床微生物与寄生虫检验\t组织标本的微生物检验" +
                ",其他\t角膜荧光素染色" +
                ",其他\t基础代谢功能检测组合" +
                ",其他\t快速石蜡切片" +
                ",其他\t颞动脉活检" +
                ",其他\t全套代谢功能检测组合" +
                ",其他\t唾液流量测定" +
                ",其他\t肿瘤筛查");


        huayanSplit = huayan.toString().split(",");
        for(String temp: huayanSplit) {
            String[] ele = temp.split("\t");
            huayanMap.put(ele[1],ele[0]);
        }

        StringBuilder jianyan = new StringBuilder();
        jianyan.append("临床血液检验\t11-去氢-血栓烷B<sub>2</sub>测定" +
                ",临床血液检验\t6-酮-前列腺素F1α测定" +
                ",临床血液检验\tα<sub>2</sub>-抗纤溶酶活性测定" +
                ",临床血液检验\tα<sub>2</sub>-抗纤溶酶抗原测定" +
                ",临床血液检验\tα<sub>2</sub>-纤溶酶抑制物活性测定" +
                ",临床血液检验\tα<sub>2</sub>-纤溶酶抑制物抗原测定" +
                ",临床血液检验\tδ-氨基乙酰丙酸" +
                ",临床血液检验\tδ-氨基乙酰丙酸脱氢酶测定" +
                ",临床血液检验\t常用细胞化学染色" +
                ",临床血液检验\t细胞免疫化学染色技术——碱性磷酸酶-抗碱性磷酸酶法" +
                ",临床血液检验\t细胞免疫化学染色技术——链霉素抗生物素-生物素法" +
                ",临床血液检验\t血浆纤维蛋白原片段<sub>1+2</sub>测定——ELISA法" +
                ",临床血液检验\tABO基因分型技术" +
                ",临床血液检验\tABO新生儿溶血病血型血清学检查——患儿血样检查" +
                ",临床血液检验\tABO血型鉴定" +
                ",临床血液检验\tABO亚型检测方法" +
                ",临床血液检验\tDonath-Landsteiner试验" +
                ",临床血液检验\tELISA法定量测定D-二聚体" +
                ",临床血液检验\tHbF酸洗脱试验" +
                ",临床血液检验\tIgG血型同种抗体筛查试验" +
                ",临床血液检验\tIgM血型同种抗体筛查试验" +
                ",临床血液检验\tMN血型定型" +
                ",临床血液检验\tP1Pk血型定型" +
                ",临床血液检验\tProCGlobal试验" +
                ",临床血液检验\tRh新生儿溶血病血型血清学检查——产妇血清学检查" +
                ",临床血液检验\tRh新生儿溶血病血型血清学检查——婴儿血清学检查" +
                ",临床血液检验\tRh血型定型" +
                ",临床血液检验\tSchumm试验" +
                ",临床血液检验\tTEG血小板图实验（ADP途径）" +
                ",临床血液检验\tTEG血小板图实验（花生四烯酸途径）" +
                ",临床血液检验\t阿司匹林耐量试验" +
                ",临床血液检验\t白细胞分类计数" +
                ",临床血液检验\t白细胞计数" +
                ",临床血液检验\t白细胞免疫表型分析" +
                ",临床血液检验\t白细胞形态学检查" +
                ",临床血液检验\t靶形红细胞" +
                ",临床血液检验\t变性珠蛋白小体试验" +
                ",临床血液检验\t丙酮酸激酶活性测定" +
                ",临床血液检验\t丙酮酸激酶荧光斑点试验" +
                ",临床血液检验\t丙型肝炎病毒抗体免疫球蛋白G" +
                ",临床血液检验\t不稳定血红蛋白试验" +
                ",临床血液检验\t产前母体内IgG类抗-A（抗-B）效价检测" +
                ",临床血液检验\t产前筛查试验" +
                ",临床血液检验\t产前筛查血标本采集" +
                ",临床血液检验\t沉淀反应" +
                ",临床血液检验\t出血时间测定" +
                ",临床血液检验\t促红细胞生成素受体测定" +
                ",临床血液检验\t醋酸萘酯酶染色和氟化钠抑制试验" +
                ",临床血液检验\t蛋白C活性测定" +
                ",临床血液检验\t蛋白C抗原测定" +
                ",临床血液检验\t蛋白S抗原测定" +
                ",临床血液检验\t单核细胞计数" +
                ",临床血液检验\t胆色素原脱氨基酶测定" +
                ",临床血液检验\t单特异性抗球蛋白试验" +
                ",临床血液检验\t丁酸萘酯酶染色" +
                ",临床血液检验\t非特异性酯酶染色" +
                ",临床血液检验\t复钙交叉时间测定" +
                ",临床血液检验\t复钙交叉试验" +
                ",临床血液检验\t肝素定量测定" +
                ",临床血液检验\t高铁血红蛋白还原试验" +
                ",临床血液检验\t骨髓病理学检查" +
                ",临床血液检验\t骨髓细胞形态学检验" +
                ",临床血液检验\t骨髓有核细胞计数" +
                ",临床血液检验\t亨氏小体测定" +
                ",临床血液检验\t红细胞变形性测定" +
                ",临床血液检验\t红细胞表面电荷测定" +
                ",临床血液检验\t红细胞比积" +
                ",临床血液检验\t红细胞丙酮酸激酶活性测定" +
                ",临床血液检验\t红细胞卟啉测定" +
                ",临床血液检验\t红细胞沉降率测定" +
                ",临床血液检验\t红细胞电泳时间测定" +
                ",临床血液检验\t红细胞孵育渗透脆性试验" +
                ",临床血液检验\t红细胞交叉配血试验" +
                ",临床血液检验\t红细胞计数" +
                ",临床血液检验\t红细胞聚集性测定" +
                ",临床血液检验\t红细胞聚集指数" +
                ",临床血液检验\t红细胞镰变试验" +
                ",临床血液检验\t红细胞膜蛋白电泳" +
                ",临床血液检验\t红细胞膜蛋白电泳分析" +
                ",临床血液检验\t红细胞膜缺陷的检验——红细胞自身溶血试验" +
                ",临床血液检验\t红细胞膜缺陷的基因检测" +
                ",临床血液检验\t红细胞内游离原卟啉测定" +
                ",临床血液检验\t红细胞平均指数" +
                ",临床血液检验\t红细胞破坏部位测定" +
                ",临床血液检验\t红细胞葡萄糖-6-磷酸脱氢酶活性测定" +
                ",临床血液检验\t红细胞渗透脆性试验" +
                ",临床血液检验\t红细胞寿命检验" +
                ",临床血液检验\t红细胞体积分布宽度" +
                ",临床血液检验\t红细胞形态学检查" +
                ",临床血液检验\t红细胞血型抗体鉴定" +
                ",临床血液检验\t红细胞血型配型测定" +
                ",临床血液检验\t红细胞游离原卟啉测定" +
                ",临床血液检验\t红细胞直径" +
                ",临床血液检验\t红细胞自身溶血试验及其纠正试验" +
                ",临床血液检验\t活化部分凝血活酶时间测定" +
                ",临床血液检验\t活化蛋白C抵抗试验" +
                ",临床血液检验\t活化凝血时间测定" +
                ",临床血液检验\t浆细胞形态学" +
                ",临床血液检验\t间接抗人球蛋白试验" +
                ",临床血液检验\t简易凝血活酶生成试验及纠正试验" +
                ",临床血液检验\t巨大型血小板" +
                ",临床血液检验\t巨核细胞计数" +
                ",临床血液检验\t聚凝胺试验" +
                ",临床血液检验\t抗红细胞自身抗体测定" +
                ",临床血液检验\t抗坏血酸-氰化物试验" +
                ",临床血液检验\t抗碱血红蛋白测定" +
                ",临床血液检验\t抗凝血酶-Ⅲ" +
                ",临床血液检验\t抗凝血酶活性测定" +
                ",临床血液检验\t抗凝血酶抗原测定" +
                ",临床血液检验\t抗球蛋白交叉配血试验" +
                ",临床血液检验\t抗心磷脂抗体测定" +
                ",临床血液检验\t可溶性纤维蛋白单体复合物测定" +
                ",临床血液检验\t可溶性血管内皮细胞蛋白C受体测定" +
                ",临床血液检验\t狼疮抗凝物的检测" +
                ",临床血液检验\t狼疮抗凝物质Lupo试验、Lucor试验" +
                ",临床血液检验\t泪液分泌型免疫球蛋白A" +
                ",临床血液检验\t冷凝集试验" +
                ",临床血液检验\t冷凝集素试验" +
                ",临床血液检验\t冷热溶血试验" +
                ",临床血液检验\t冷热双相溶血试验" +
                ",临床血液检验\t冷溶血试验" +
                ",临床血液检验\t氯乙酸ASD萘酚酯酶(CE)染色" +
                ",临床血液检验\t轮状病毒抗原的检测" +
                ",临床血液检验\t毛细血管脆性试验" +
                ",临床血液检验\t梅毒血清试验" +
                ",临床血液检验\t玫瑰花环试验" +
                ",临床血液检验\t内皮素-1测定" +
                ",临床血液检验\t尿卟啉原脱羧基酶测定" +
                ",临床血液检验\t尿含铁血黄素试验" +
                ",临床血液检验\t尿激酶型纤溶酶原激活剂活性测定" +
                ",临床血液检验\t尿激酶型纤溶酶原激活剂抗原测定" +
                ",临床血液检验\t尿激酶型纤溶酶原激活物活性测定" +
                ",临床血液检验\t尿液1-脱氨基-8-右旋精氨酸加压素测定" +
                ",临床血液检验\t尿液纤维蛋白（原）降解产物" +
                ",临床血液检验\t尿液游离血红蛋白检测" +
                ",临床血液检验\t凝血常规检查" +
                ",临床血液检验\t凝血酶-抗凝血酶复合物测定" +
                ",临床血液检验\t凝血酶测定" +
                ",临床血液检验\t凝血酶调节蛋白活性测定（TM:A）" +
                ",临床血液检验\t凝血酶调节蛋白抗原测定（TM:Ag）" +
                ",临床血液检验\t凝血酶激活的纤溶抑制剂活性测定" +
                ",临床血液检验\t凝血酶时间" +
                ",临床血液检验\t凝血酶时间延长的纠正试验" +
                ",临床血液检验\t凝血酶原片段1＋2测定" +
                ",临床血液检验\t凝血因子Ⅱ、Ⅴ、Ⅶ和Ⅹ的活性测定(一期法)" +
                ",临床血液检验\t凝血因子ⅧA、ⅧB亚基抗原测定" +
                ",临床血液检验\t凝血因子Ⅷ定性试验" +
                ",临床血液检验\t凝血因子Ⅷ抑制物测定" +
                ",临床血液检验\t凝血因子Ⅷ、Ⅸ、Ⅺ、Ⅻ的活性测定(一期法)" +
                ",临床血液检验\t凝血因子Ⅹa抑制试验" +
                ",临床血液检验\t凝血因子Ⅹ促凝活性测定" +
                ",临床血液检验\t凝血因子ⅩⅢ定性试验" +
                ",临床血液检验\t平均红细胞体积" +
                ",临床血液检验\t平均红细胞血红蛋白含量" +
                ",临床血液检验\t平均红细胞血红蛋白浓度" +
                ",临床血液检验\t贫血原因检测" +
                ",临床血液检验\t葡萄糖-6-磷酸脱氢酶活性测定" +
                ",临床血液检验\t葡萄糖-6-磷酸脱氢酶荧光斑点试验" +
                ",临床血液检验\t全血/血浆黏度测定" +
                ",临床血液检验\t去二甲基-血栓素B<sub>2</sub>、11-去氢-血栓素B<sub>2</sub>测定" +
                ",临床血液检验\t热变性试验" +
                ",临床血液检验\t热放散试验" +
                ",临床血液检验\t热溶血试验" +
                ",临床血液检验\t瑞斯托霉素诱导的血小板凝集试验" +
                ",临床血液检验\t乳胶凝集法定性测定D-二聚体" +
                ",临床血液检验\t弱D型鉴定" +
                ",临床血液检验\t伤寒、副伤寒血清凝集试验" +
                ",临床血液检验\t蛇毒因子溶血试验" +
                ",临床血液检验\t释放试验" +
                ",临床血液检验\t嗜碱性点彩红细胞计数" +
                ",临床血液检验\t嗜碱性粒细胞计数" +
                ",临床血液检验\t嗜酸细胞阳离子蛋白的测定" +
                ",临床血液检验\t嗜酸性粒细胞计数" +
                ",临床血液检验\t嗜异性凝集试验" +
                ",临床血液检验\t嗜中性分叶核粒细胞数" +
                ",临床血液检验\t酸化甘油溶血试验" +
                ",临床血液检验\t酸化溶血试验（Ham试验）" +
                ",临床血液检验\t酸性磷酸酶染色" +
                ",临床血液检验\t苏丹黑B染色" +
                ",临床血液检验\t髓过氧化物酶染色" +
                ",临床血液检验\t肽链裂解试验" +
                ",临床血液检验\t碳氧血红蛋白检测" +
                ",临床血液检验\t特殊情况下ABO血型的检测" +
                ",临床血液检验\t体外血栓形成监测" +
                ",临床血液检验\t同种血小板抗体检测" +
                ",临床血液检验\t唾液A，B，H，Le<sup>a</sup>，Le<sup>b</sup>血型物质检测方法" +
                ",临床血液检验\t唾液中ABH血型物质测定" +
                ",临床血液检验\tvWF瑞斯托霉素辅因子测定" +
                ",临床血液检验\t外周血T细胞计数" +
                ",临床血液检验\t外周血细胞涂片形态学检查" +
                ",临床血液检验\t网织红细胞计数" +
                ",临床血液检验\t网织红细胞生成指数" +
                ",临床血液检验\t微量补体溶血敏感试验" +
                ",临床血液检验\t维生素B<sub>12</sub>吸收试验" +
                ",临床血液检验\t温氏法血细胞比容测定" +
                ",临床血液检验\t无创产前检测" +
                ",临床血液检验\t纤溶酶-抗纤溶酶复合物测定" +
                ",临床血液检验\t纤溶酶原活性测定" +
                ",临床血液检验\t纤溶酶原激活抑制剂-1活性测定" +
                ",临床血液检验\t纤溶酶原激活抑制剂-1抗原测定" +
                ",临床血液检验\t纤溶系统检测" +
                ",临床血液检验\t纤维蛋白肽A测定" +
                ",临床血液检验\t纤维蛋白原测定" +
                ",临床血液检验\t纤维蛋白（原）降解产物检测" +
                ",临床血液检验\t献血者健康检查方法" +
                ",临床血液检验\t细胞分子生物学检验" +
                ",临床血液检验\t细胞过碘酸Schiff (糖原)染色" +
                ",临床血液检验\t细胞甲苯胺蓝染色" +
                ",临床血液检验\t细胞铁染色法" +
                ",临床血液检验\t细胞遗传学检验" +
                ",临床血液检验\t新生儿疾病筛查样本采集" +
                ",临床血液检验\t新生儿溶血症筛查" +
                ",临床血液检验\t新生儿血型血清学检查" +
                ",临床血液检验\t新生儿周围血像和骨髓像特点" +
                ",临床血液检验\t锌原卟啉测定" +
                ",临床血液检验\t血沉方程K值" +
                ",临床血液检验\t血管内皮细胞功能试验" +
                ",临床血液检验\t血管性血友病因子抗原测定" +
                ",临床血液检验\t血管性血友病因子瑞斯托霉素辅因子测定" +
                ",临床血液检验\t血红蛋白A2测定" +
                ",临床血液检验\t血红蛋白C检测" +
                ",临床血液检验\t血红蛋白F测定" +
                ",临床血液检验\t血红蛋白F碱变性试验" +
                ",临床血液检验\t血红蛋白F酸洗脱试验" +
                ",临床血液检验\t血红蛋白H包涵体检查" +
                ",临床血液检验\t血红蛋白S测定" +
                ",临床血液检验\t血红蛋白S溶解度试验" +
                ",临床血液检验\t血红蛋白测定" +
                ",临床血液检验\t血红蛋白醋酸纤维膜电泳" +
                ",临床血液检验\t血红蛋白电泳" +
                ",临床血液检验\t血红蛋白尿测定" +
                ",临床血液检验\t血红蛋白区带电泳分析" +
                ",临床血液检验\t血红蛋白热不稳定试验" +
                ",临床血液检验\t血红蛋白溶解度试验" +
                ",临床血液检验\t血红蛋白组分色谱分析" +
                ",临床血液检验\t血红素检验" +
                ",临床血液检验\t血红素结合蛋白测定" +
                ",临床血液检验\t血浆D-二聚体测定" +
                ",临床血液检验\t血浆P-选择素测定" +
                ",临床血液检验\t血浆卟啉测定" +
                ",临床血液检验\t血浆蛋白C和蛋白S检测" +
                ",临床血液检验\t血浆单个凝血因子促凝血活性测定" +
                ",临床血液检验\t血浆肝素浓度测定" +
                ",临床血液检验\t血浆激肽释放酶原测定" +
                ",临床血液检验\t血浆抗凝血酶检测" +
                ",临床血液检验\t血浆硫酸鱼精蛋白副凝固试验" +
                ",临床血液检验\t血浆黏度测定" +
                ",临床血液检验\t血浆凝血活化分子标志物检测" +
                ",临床血液检验\t血浆凝血酶-抗凝血酶复合物测定" +
                ",临床血液检验\t血浆凝血酶原时间测定" +
                ",临床血液检验\t血浆凝血因子抑制物检测" +
                ",临床血液检验\t血浆凝血因子Ⅶ活性测定" +
                ",临床血液检验\t血浆糖蛋白特异性自身抗体测定（改良MAIPA法）" +
                ",临床血液检验\t血浆纤溶酶原检测" +
                ",临床血液检验\t血浆纤溶抑制物检测" +
                ",临床血液检验\t血浆纤维蛋白原测定——凝血酶法" +
                ",临床血液检验\t血浆纤维蛋白原定量" +
                ",临床血液检验\t血浆因子ⅫⅠ定性试验" +
                ",临床血液检验\t血浆游离血红蛋白测定" +
                ",临床血液检验\t血浆组织型纤溶酶原激活物及其抑制物检测" +
                ",临床血液检验\t血浆、尿液去甲基-6-酮-前列腺素F1α测定" +
                ",临床血液检验\t血块或血浆凝块溶解时间测定" +
                ",临床血液检验\t血块收缩试验" +
                ",临床血液检验\t血清补体C3裂解产物" +
                ",临床血液检验\t血清和红细胞叶酸测定" +
                ",临床血液检验\t血清结合珠蛋白测定" +
                ",临床血液检验\t血清内因子阻断抗体测定" +
                ",临床血液检验\t血清黏度测定" +
                ",临床血液检验\t血清凝血酶原时间测定" +
                ",临床血液检验\t血清铁测定" +
                ",临床血液检验\t血清铁四项" +
                ",临床血液检验\t血清维生素B<sub>12</sub>测定" +
                ",临床血液检验\t血清纤维蛋白(原)降解产物定性试验" +
                ",临床血液检验\t血清转铁蛋白受体测定" +
                ",临床血液检验\t血清、尿液2-甲基柠檬酸测定" +
                ",临床血液检验\t血清、尿液胱硫醚测定" +
                ",临床血液检验\t血清、尿液甲基丙二酸测定" +
                ",临床血液检验\t血容量测定" +
                ",临床血液检验\t血栓前体蛋白测定" +
                ",临床血液检验\t血栓三项" +
                ",临床血液检验\t血栓素B<sub>2</sub>检测" +
                ",临床血液检验\t血小板ATP释放试验" +
                ",临床血液检验\t血小板P-选择素测定" +
                ",临床血液检验\t血小板促凝活性测定" +
                ",临床血液检验\t血小板第3因子有效性测定" +
                ",临床血液检验\t血小板第四因子测定" +
                ",临床血液检验\t血小板钙流的测定" +
                ",临床血液检验\t血小板功能试验" +
                ",临床血液检验\t血小板交叉配血试验" +
                ",临床血液检验\t血小板计数" +
                ",临床血液检验\t血小板聚集试验" +
                ",临床血液检验\t血小板颗粒膜糖蛋白CD62P测定" +
                ",临床血液检验\t血小板颗粒膜糖蛋白CD63测定" +
                ",临床血液检验\t血小板膜糖蛋白检测" +
                ",临床血液检验\t血小板黏附试验" +
                ",临床血液检验\t血小板平均容积" +
                ",临床血液检验\t血小板平均体积" +
                ",临床血液检验\t血小板寿命测定" +
                ",临床血液检验\t血小板糖蛋白测定" +
                ",临床血液检验\t血小板糖蛋白Ⅱb、Ⅲa自身抗体测定" +
                ",临床血液检验\t血小板体积分布宽度" +
                ",临床血液检验\t血小板微颗粒测定" +
                ",临床血液检验\t血小板相关补体C4测定" +
                ",临床血液检验\t血小板相关免疫球蛋白测定" +
                ",临床血液检验\t血小板形态学检查" +
                ",临床血液检验\t血小板血型检测" +
                ",临床血液检验\t血小板压积" +
                ",临床血液检验\t血小板自身抗体测定" +
                ",临床血液检验\t血细胞比容测定" +
                ",临床血液检验\t血细胞膜CD55/CD59表型分析" +
                ",临床血液检验\t血细胞组织化学染色" +
                ",临床血液检验\t血型的基因分型" +
                ",临床血液检验\t血型鉴定" +
                ",临床血液检验\t血型抗体效价测定" +
                ",临床血液检验\t血型血清学——抗球蛋白试验" +
                ",临床血液检验\t血型血清——吸收试验" +
                ",临床血液检验\t血液常规检查" +
                ",临床血液检验\t血液分析仪检验技术" +
                ",临床血液检验\t血液流变学检查" +
                ",临床血液检验\t血液酸碱度测定" +
                ",临床血液检验\t血液粘度测定" +
                ",临床血液检验\t血液肿瘤分子遗传学检测" +
                ",临床血液检验\t盐水介质交叉配血试验" +
                ",临床血液检验\t叶酸/维生素B<sub>12</sub>治疗性试验" +
                ",临床血液检验\t叶酸吸收试验" +
                ",临床血液检验\t异丙醇沉淀试验" +
                ",临床血液检验\t异丙醇试验" +
                ",临床血液检验\t异常红细胞形态检查" +
                ",临床血液检验\t乙醇凝胶试验" +
                ",临床血液检验\t乙醚放散法" +
                ",临床血液检验\t婴ABO血型不合新生儿溶血病——孕妇IgG抗A（B）检测" +
                ",临床血液检验\t意外抗体筛查与鉴定" +
                ",临床血液检验\t乙酰胆碱酯酶测定" +
                ",临床血液检验\t乙型肝炎核心抗原" +
                ",临床血液检验\t异型淋巴细胞测定" +
                ",临床血液检验\t有核红细胞测定" +
                ",临床血液检验\t游离肝素时间测定" +
                ",临床血液检验\t游离试验" +
                ",临床血液检验\t优球蛋白溶解时间" +
                ",临床血液检验\t造血干细胞计数" +
                ",临床血液检验\t造血祖细胞培养" +
                ",临床血液检验\t早孕期母外周血筛查试验" +
                ",临床血液检验\t正常血浆APTT纠正试验" +
                ",临床血液检验\t正常血浆PT纠正试验" +
                ",临床血液检验\t蔗糖溶血试验" +
                ",临床血液检验\t蔗糖水溶血试验" +
                ",临床血液检验\t直接抗人球蛋白试验" +
                ",临床血液检验\t中性杆状核粒细胞" +
                ",临床血液检验\t中性粒细胞测定" +
                ",临床血液检验\t中性粒细胞的分离与功能检测" +
                ",临床血液检验\t中性粒细胞碱性磷酸酶染色" +
                ",临床血液检验\t中性粒细胞计数" +
                ",临床血液检验\t中孕期母外周血筛查试验" +
                ",临床血液检验\t转铁蛋白饱和度测定" +
                ",临床血液检验\t珠蛋白基因突变检测" +
                ",临床血液检验\t珠蛋白肽链分析" +
                ",临床血液检验\t珠蛋白肽链聚丙烯酰胺凝胶电泳分析" +
                ",临床血液检验\t柱凝集法交叉配血试验" +
                ",临床血液检验\t自身红细胞抗体" +
                ",临床血液检验\t总血红蛋白测定" +
                ",临床血液检验\t组织型纤溶酶原激活剂活性测定" +
                ",临床血液检验\t组织型纤溶酶原激活剂抗原测定" +
                ",临床血液检验\t组织因子促凝活性测定" +
                ",临床血液检验\t组织因子途径抑制物活性测定" +
                ",临床体液检验\t1小时尿沉渣计数" +
                ",临床体液检验\t24小时尿卟啉测定" +
                ",临床体液检验\t24小时尿蛋白定量" +
                ",临床体液检验\t24小时尿量测定" +
                ",临床体液检验\t24小时尿磷测定" +
                ",临床体液检验\t24小时尿锰测定" +
                ",临床体液检验\t24小时尿钼测定" +
                ",临床体液检验\t24小时尿铜测定" +
                ",临床体液检验\t24小时尿硒测定" +
                ",临床体液检验\t24小时尿液儿茶酚胺测定" +
                ",临床体液检验\t24小时尿液环磷酸腺苷" +
                ",临床体液检验\t24小时尿液环磷酸腺苷测定" +
                ",临床体液检验\t24小时尿液肌酸测定" +
                ",临床体液检验\t24小时尿液生长激素测定" +
                ",临床体液检验\t浆膜腔积液一般检验" +
                ",临床体液检验\t（神经）梅毒脑脊液检测" +
                ",临床体液检验\t艾迪斯计数法" +
                ",临床体液检验\t白细胞酯酶检验" +
                ",临床体液检验\t板层小体计数" +
                ",临床体液检验\t鼻腔分泌物的生化检查" +
                ",临床体液检验\t不孕不育的免疫学检查" +
                ",临床体液检验\t仓鼠卵-精子穿透试验" +
                ",临床体液检验\t虫卵计数法" +
                ",临床体液检验\t穿刺液常规检查" +
                ",临床体液检验\t穿刺液培养" +
                ",临床体液检验\t促胰酶素-促胰液素试验" +
                ",临床体液检验\t蛋白质定量测定" +
                ",临床体液检验\t胆色素原测定" +
                ",临床体液检验\t定量法尿液有形成分检查" +
                ",临床体液检验\t定性法尿液有形成分检查" +
                ",临床体液检验\t粪便白细胞" +
                ",临床体液检验\t粪便标本采集" +
                ",临床体液检验\t粪便卟啉测定" +
                ",临床体液检验\t粪便常规检查" +
                ",临床体液检验\t粪便胆红素测定" +
                ",临床体液检验\t粪便淀粉颗粒" +
                ",临床体液检验\t粪便红细胞" +
                ",临床体液检验\t粪便酵母菌" +
                ",临床体液检验\t粪便肌肉纤维的定性试验" +
                ",临床体液检验\t粪便寄生虫检查" +
                ",临床体液检验\t粪便理学检查" +
                ",临床体液检验\t粪便黏液" +
                ",临床体液检验\t粪便脓液" +
                ",临床体液检验\t粪便pH测定" +
                ",临床体液检验\t粪便吞噬细胞" +
                ",临床体液检验\t粪便显微镜检查" +
                ",临床体液检验\t粪便细菌培养" +
                ",临床体液检验\t粪便颜色" +
                ",临床体液检验\t粪便胰弹性蛋白酶检测" +
                ",临床体液检验\t粪便隐血试验" +
                ",临床体液检验\t粪便有形成分显微镜检查" +
                ",临床体液检验\t粪便脂肪检查" +
                ",临床体液检验\t粪便脂肪平衡试验" +
                ",临床体液检验\t粪便中性脂肪测定" +
                ",临床体液检验\t粪胆素测定" +
                ",临床体液检验\t粪胆素原测定" +
                ",临床体液检验\t粪食物残渣" +
                ",临床体液检验\t粪脂肪苏丹染色" +
                ",临床体液检验\t附睾功能检验" +
                ",临床体液检验\t腹水常规检查" +
                ",临床体液检验\t甘氨酰脯氨酸二肽氨基肽酶测定" +
                ",临床体液检验\t宫颈5%醋酸染色肉眼观察" +
                ",临床体液检验\t宫颈5%复方碘染色肉眼观察" +
                ",临床体液检验\t宫颈分泌物淋病奈瑟菌培养" +
                ",临床体液检验\t宫颈分泌物沙眼衣原体快速抗原检测" +
                ",临床体液检验\t宫颈分泌物涂片革兰染色显微镜检查" +
                ",临床体液检验\t宫颈黏液检验" +
                ",临床体液检验\t宫腔细菌培养" +
                ",临床体液检验\t关节腔积液常规检查" +
                ",临床体液检验\t汗液电解质检测" +
                ",临床体液检验\t汗液氯化物测定" +
                ",临床体液检验\t滑膜液标本采集与一般性状检验" +
                ",临床体液检验\t滑膜液补体检测" +
                ",临床体液检验\t滑膜液蛋白质检测" +
                ",临床体液检验\t滑膜液结晶检查" +
                ",临床体液检验\t滑膜液抗核抗体检测" +
                ",临床体液检验\t滑膜液类风湿因子检测" +
                ",临床体液检验\t滑膜液黏蛋白凝块形成试验" +
                ",临床体液检验\t滑膜液葡萄糖检测" +
                ",临床体液检验\t滑膜液乳酸检测" +
                ",临床体液检验\t混合抗球蛋白反应试验" +
                ",临床体液检验\t碱负荷试验" +
                ",临床体液检验\t浆膜腔积液C-反应蛋白测定" +
                ",临床体液检验\t浆膜腔积液癌胚抗原检测" +
                ",临床体液检验\t浆膜腔积液病原生物学检查" +
                ",临床体液检验\t浆膜腔积液采集与理学检验" +
                ",临床体液检验\t浆膜腔积液淀粉酶检测" +
                ",临床体液检验\t浆膜腔积液流式细胞分析技术" +
                ",临床体液检验\t浆膜腔积液黏蛋白定性试验" +
                ",临床体液检验\t浆膜腔积液葡萄糖检测" +
                ",临床体液检验\t浆膜腔积液溶菌酶检测" +
                ",临床体液检验\t浆膜腔积液乳酸脱氢酶检测" +
                ",临床体液检验\t浆膜腔积液酸碱度测定" +
                ",临床体液检验\t浆膜腔积液腺苷脱氨酶检测" +
                ",临床体液检验\t浆膜腔积液纤维蛋白（原）降解产物测定" +
                ",临床体液检验\t浆膜腔积液显微镜检查" +
                ",临床体液检验\t浆膜腔积液纤维连接蛋白测定" +
                ",临床体液检验\t浆膜腔积液细胞学检查" +
                ",临床体液检验\t浆膜腔积液有形成分检查" +
                ",临床体液检验\t浆膜腔积液脂蛋白电泳" +
                ",临床体液检验\t浆膜腔积液总蛋白定量检测" +
                ",临床体液检验\t精浆的生物化学检查" +
                ",临床体液检验\t精浆果糖测定" +
                ",临床体液检验\t精浆镁测定" +
                ",临床体液检验\t精浆免疫球蛋白测定" +
                ",临床体液检验\t精浆免疫抑制物质测定" +
                ",临床体液检验\t精浆柠檬酸测定" +
                ",临床体液检验\t精浆酸性磷酸酶测定" +
                ",临床体液检验\t精浆锌测定" +
                ",临床体液检验\t精浆中性α-葡萄糖苷酶测定" +
                ",临床体液检验\t精浆γ-GT测定" +
                ",临床体液检验\t精浆γ-谷氨酰基转肽酶测定" +
                ",临床体液检验\t精囊腺功能检验" +
                ",临床体液检验\t精液白细胞检验" +
                ",临床体液检验\t精液标本收集与精液理学检验" +
                ",临床体液检验\t精液常规检查" +
                ",临床体液检验\t精液弹性蛋白酶检验" +
                ",临床体液检验\t精液凋亡细胞检验" +
                ",临床体液检验\t精液pH测定" +
                ",临床体液检验\t精液乳酸脱氢酶X测定" +
                ",临床体液检验\t精液液化时间" +
                ",临床体液检验\t精子-宫颈黏液相互作用试验" +
                ",临床体液检验\t精子穿卵试验" +
                ",临床体液检验\t精子顶体反应及顶体酶检验" +
                ",临床体液检验\t精子顶体酶活性测定" +
                ",临床体液检验\t精子顶体完整率检验" +
                ",临床体液检验\t精子核检验" +
                ",临床体液检验\t精子活动力测定" +
                ",临床体液检验\t精子活动率测定" +
                ",临床体液检验\t精子计数" +
                ",临床体液检验\t精子膜功能检验" +
                ",临床体液检验\t精子凝集现象检查和有形成分检查" +
                ",临床体液检验\t精子爬高试验" +
                ",临床体液检验\t精子尾部低渗肿胀试验" +
                ",临床体液检验\t精子形态观察" +
                ",临床体液检验\t精子运动速度" +
                ",临床体液检验\t禁水加压素试验" +
                ",临床体液检验\t禁水试验" +
                ",临床体液检验\t抗精子抗体免疫珠试验" +
                ",临床体液检验\t泪液pH检测" +
                ",临床体液检验\t泪液溶菌酶检测" +
                ",临床体液检验\t泪液乳铁蛋白检测" +
                ",临床体液检验\t流式尿沉渣分析" +
                ",临床体液检验\t李文生试验" +
                ",临床体液检验\t氯化铵负荷试验" +
                ",临床体液检验\t卵磷脂小体测定" +
                ",临床体液检验\t卵磷脂／鞘磷脂比值测定" +
                ",临床体液检验\t脑脊液5-羟色胺测定" +
                ",临床体液检验\t脑脊液5-羟吲哚乙酸测定" +
                ",临床体液检验\t脑脊液C-反应蛋白测定" +
                ",临床体液检验\t脑脊液IgA测定" +
                ",临床体液检验\t脑脊液IgG测定" +
                ",临床体液检验\t脑脊液IgM测定" +
                ",临床体液检验\t脑脊液Tau蛋白测定" +
                ",临床体液检验\t脑脊液氨基酸总量测定" +
                ",临床体液检验\t脑脊液氨基酸组分" +
                ",临床体液检验\t脑脊液丙氨酸氨基转移酶测定" +
                ",临床体液检验\t脑脊液病原体形态学检查" +
                ",临床体液检验\t脑脊液常规检查" +
                ",临床体液检验\t脑脊液蛋白质定性试验" +
                ",临床体液检验\t脑脊液的寡克隆电泳检测" +
                ",临床体液检验\t脑脊液电泳" +
                ",临床体液检验\t脑脊液多巴胺测定" +
                ",临床体液检验\t脑脊液甘氨酸测定" +
                ",临床体液检验\t脑脊液高香草酸测定" +
                ",临床体液检验\t脑脊液假胆碱酯酶测定" +
                ",临床体液检验\t脑脊液肌酸激酶测定" +
                ",临床体液检验\t脑脊液理学检查" +
                ",临床体液检验\t脑脊液氯化物测定" +
                ",临床体液检验\t脑脊液免疫球蛋白 G和白蛋白比率测定" +
                ",临床体液检验\t脑脊液葡萄糖测定" +
                ",临床体液检验\t脑脊液醛缩酶测定" +
                ",临床体液检验\t脑脊液乳酸检测" +
                ",临床体液检验\t脑脊液乳酸脱氢酶及同工酶测定" +
                ",临床体液检验\t脑脊液色氨酸试验" +
                ",临床体液检验\t脑脊液酸性磷酸酶测定" +
                ",临床体液检验\t脑脊液髓鞘碱性蛋白测定" +
                ",临床体液检验\t脑脊液特异性梅毒螺旋体抗体" +
                ",临床体液检验\t脑脊液天冬氨酸氨基转移酶测定" +
                ",临床体液检验\t脑脊液腺苷酸激酶测定" +
                ",临床体液检验\t脑脊液腺苷脱氨酶测定" +
                ",临床体液检验\t脑脊液显微镜检查" +
                ",临床体液检验\t脑脊液细胞计数" +
                ",临床体液检验\t脑脊液细菌培养标本采集与处理" +
                ",临床体液检验\t脑脊液隐血试验" +
                ",临床体液检验\t脑脊液乙酰胆碱测定" +
                ",临床体液检验\t脑脊液乙酰胆碱酯酶测定" +
                ",临床体液检验\t脑脊液有形成分检查——细胞学检查" +
                ",临床体液检验\t脑脊液总蛋白" +
                ",临床体液检验\t脑脊液β-葡萄糖苷酶测定" +
                ",临床体液检验\t尿17-酮类固醇测定" +
                ",临床体液检验\t尿T-H糖蛋白测定" +
                ",临床体液检验\t尿苯丙酮定性试验" +
                ",临床体液检验\t尿本周蛋白定性试验" +
                ",临床体液检验\t尿沉渣白细胞测定" +
                ",临床体液检验\t尿沉渣定量检查" +
                ",临床体液检验\t尿沉渣红细胞测定" +
                ",临床体液检验\t尿沉渣上皮细胞和异形细胞" +
                ",临床体液检验\t尿沉渣显微镜检查" +
                ",临床体液检验\t尿蛋白定性试验" +
                ",临床体液检验\t尿胆红素定性试验" +
                ",临床体液检验\t尿胆原定性试验" +
                ",临床体液检验\t尿胱氨酸定性试验" +
                ",临床体液检验\t尿含铁血黄素定性试验" +
                ",临床体液检验\t尿红细胞位相留取" +
                ",临床体液检验\t尿红细胞形态分析" +
                ",临床体液检验\t尿结石成份分析" +
                ",临床体液检验\t尿肌红蛋白定性试验" +
                ",临床体液检验\t尿酪氨酸定性试验" +
                ",临床体液检验\t尿量测定" +
                ",临床体液检验\t尿流率测定" +
                ",临床体液检验\t尿氯测定" +
                ",临床体液检验\t尿浓缩试验" +
                ",临床体液检验\t尿培养" +
                ",临床体液检验\t尿葡萄糖定性试验" +
                ",临床体液检验\t尿妊娠试验" +
                ",临床体液检验\t尿乳糜定性试验" +
                ",临床体液检验\t尿三杯试验" +
                ",临床体液检验\t尿三氯化铁试验" +
                ",临床体液检验\t尿渗量测定" +
                ",临床体液检验\t尿渗量和自由水清除率测定" +
                ",临床体液检验\t尿酸化功能测定" +
                ",临床体液检验\t尿酮体定性试验" +
                ",临床体液检验\t尿同型半胱氨酸测定" +
                ",临床体液检验\t尿微量白蛋白检测" +
                ",临床体液检验\t尿液5-羟吲哚乙酸测定" +
                ",临床体液检验\t尿液N-乙酰-β-D-氨基葡萄糖苷酶测定" +
                ",临床体液检验\t尿液N’-甲基尼克酰胺测定" +
                ",临床体液检验\t尿液SDS-PAGE" +
                ",临床体液检验\t尿液Tamm-Horsfall蛋白测定" +
                ",临床体液检验\t尿液氨基酸测定" +
                ",临床体液检验\t尿液白蛋白测定" +
                ",临床体液检验\t尿液白细胞" +
                ",临床体液检验\t尿液白细胞酯酶测定" +
                ",临床体液检验\t尿液半乳糖苷酶" +
                ",临床体液检验\t尿液比密测定" +
                ",临床体液检验\t尿液丙氨酸氨基肽酶测定" +
                ",临床体液检验\t尿液比重测定" +
                ",临床体液检验\t尿液草酸测定" +
                ",临床体液检验\t尿液常规检查" +
                ",临床体液检验\t尿液成体干细胞" +
                ",临床体液检验\t尿液蛋白选择性指数" +
                ",临床体液检验\t尿液蛋白质测定" +
                ",临床体液检验\t尿液碘测定" +
                ",临床体液检验\t尿液淀粉酶测定" +
                ",临床体液检验\t尿液分泌型IgA检测" +
                ",临床体液检验\t尿液氟测定" +
                ",临床体液检验\t尿液钙测定" +
                ",临床体液检验\t尿液干化学分析" +
                ",临床体液检验\t尿液铬测定" +
                ",临床体液检验\t尿液胱氨酸测定" +
                ",临床体液检验\t尿液胱抑素C检测" +
                ",临床体液检验\t尿液钴测定" +
                ",临床体液检验\t尿液还原糖试验" +
                ",临床体液检验\t尿液红细胞" +
                ",临床体液检验\t尿液钾测定" +
                ",临床体液检验\t尿液碱性磷酸酶测定" +
                ",临床体液检验\t尿液结晶测定" +
                ",临床体液检验\t尿液来源的诱导多潜能干细胞" +
                ",临床体液检验\t尿液亮氨酸氨基肽酶测定" +
                ",临床体液检验\t尿液铝测定" +
                ",临床体液检验\t尿液氯化物测定" +
                ",临床体液检验\t尿液镁测定" +
                ",临床体液检验\t尿液锰测定" +
                ",临床体液检验\t尿液免疫球蛋白测定" +
                ",临床体液检验\t尿液钼测定" +
                ",临床体液检验\t尿液钠测定" +
                ",临床体液检验\t尿液尿胆原测定" +
                ",临床体液检验\t尿液镍测定" +
                ",临床体液检验\t尿液柠檬酸测定" +
                ",临床体液检验\t尿液浓缩稀释试验" +
                ",临床体液检验\t尿液葡萄糖测定" +
                ",临床体液检验\t尿液溶菌酶测定" +
                ",临床体液检验\t尿液乳清酸测定" +
                ",临床体液检验\t尿液乳酸脱氢酶测定" +
                ",临床体液检验\t尿液上皮细胞" +
                ",临床体液检验\t尿液酸碱度测定" +
                ",临床体液检验\t尿液铜测定" +
                ",临床体液检验\t尿液维生素B<sub>1</sub>测定" +
                ",临床体液检验\t尿液维生素C测定" +
                ",临床体液检验\t尿液无机磷测定" +
                ",临床体液检验\t尿液酰基肉碱测定" +
                ",临床体液检验\t尿液细胞学检查" +
                ",临床体液检验\t尿液硒测定" +
                ",临床体液检验\t尿液细菌培养" +
                ",临床体液检验\t尿液细菌学检查" +
                ",临床体液检验\t尿液锌测定" +
                ",临床体液检验\t尿液颜色和透明度测定" +
                ",临床体液检验\t尿液亚硝酸盐测定" +
                ",临床体液检验\t尿液游离免疫球蛋白轻链测定" +
                ",临床体液检验\t尿液有形成分分析仪检验" +
                ",临床体液检验\t尿液有形成分检查的推荐参考方法" +
                ",临床体液检验\t尿液有形成分显微镜检查方法" +
                ",临床体液检验\t尿液真菌培养" +
                ",临床体液检验\t尿液中病毒感染细胞及其包涵体" +
                ",临床体液检验\t尿液转铁蛋白测定" +
                ",临床体液检验\t尿液总蛋白测定" +
                ",临床体液检验\t尿液α<sub>2</sub>-巨球蛋白测定" +
                ",临床体液检验\t尿液β-葡萄糖醛酸苷酶测定" +
                ",临床体液检验\t尿液β<sub>2</sub>-微球蛋白测定" +
                ",临床体液检验\t尿液γ-谷氨酰基转肽酶测定" +
                ",临床体液检验\t尿隐血（血红蛋白）定性测定" +
                ",临床体液检验\t尿有机酸测定" +
                ",临床体液检验\t尿游离皮质醇" +
                ",临床体液检验\t尿β<sub>2</sub>-微球蛋白" +
                ",临床体液检验\t尿γ-谷氨酰基转移酶测定" +
                ",临床体液检验\t尿Ⅰ型胶原交联氨基末端肽测定" +
                ",临床体液检验\t泡沫稳定性指数" +
                ",临床体液检验\t前列腺功能检验" +
                ",临床体液检验\t前列腺液LDH5／LDH1检验" +
                ",临床体液检验\t前列腺液标本收集及理学检验" +
                ",临床体液检验\t前列腺液高敏C反应蛋白" +
                ",临床体液检验\t前列腺液检查" +
                ",临床体液检验\t前列腺液柠檬酸检测" +
                ",临床体液检验\t前列腺液酸碱度测定" +
                ",临床体液检验\t前列腺液酸性磷酸酶测定" +
                ",临床体液检验\t前列腺液锌检测" +
                ",临床体液检验\t前列腺液有形成分分析" +
                ",临床体液检验\t全自动尿10项干化学常规分析（URISYS 2400型）" +
                ",临床体液检验\t染色法尿液有形成分检查" +
                ",临床体液检验\t绒毛采集" +
                ",临床体液检验\t乳糜尿检查" +
                ",临床体液检验\t乳头溢液的细胞学检查" +
                ",临床体液检验\t生精细胞检验" +
                ",临床体液检验\t肾功能检查" +
                ",临床体液检验\t生长抑素测定" +
                ",临床体液检验\t生殖内分泌激素检验" +
                ",临床体液检验\t肾血流量测定" +
                ",临床体液检验\t十二指肠液胆红素测定" +
                ",临床体液检验\t十二指肠液胆汁酸测定" +
                ",临床体液检验\t十二指肠液胰蛋白酶检测" +
                ",临床体液检验\t十二指肠液γ-谷氨酰基转肽酶测定" +
                ",临床体液检验\t十二指肠引流液常规检查" +
                ",临床体液检验\t酸负荷试验" +
                ",临床体液检验\t随机尿儿茶酚胺测定" +
                ",临床体液检验\t随机尿肌酐测定" +
                ",临床体液检验\t痰培养" +
                ",临床体液检验\t碳酸氢离子重吸收排泌试验" +
                ",临床体液检验\t痰液标本的采集与及理学检查" +
                ",临床体液检验\t痰液分泌型IgA检测" +
                ",临床体液检验\t痰液检查" +
                ",临床体液检验\t痰液乳酸脱氢酶测定" +
                ",临床体液检验\t痰液细胞涂片制作" +
                ",临床体液检验\t痰液细胞学检查" +
                ",临床体液检验\t痰液有形成分分析" +
                ",临床体液检验\t唾液蛋白质测定" +
                ",临床体液检验\t唾液淀粉酶测定" +
                ",临床体液检验\t唾液电解质测定" +
                ",临床体液检验\t唾液毒物分析" +
                ",临床体液检验\t唾液非蛋白氮类物质测定" +
                ",临床体液检验\t唾液激素测定" +
                ",临床体液检验\t唾液黏蛋白测定" +
                ",临床体液检验\t唾液溶菌酶测定" +
                ",临床体液检验\t唾液有机成分" +
                ",临床体液检验\t唾液游离皮质醇测定" +
                ",临床体液检验\t唾液治疗药物监测" +
                ",临床体液检验\t外阴阴道假丝酵母菌病检验" +
                ",临床体液检验\t微量白蛋白尿测定" +
                ",临床体液检验\t胃酸分泌量检测" +
                ",临床体液检验\t胃液常规检查" +
                ",临床体液检验\t胃液尿素检测" +
                ",临床体液检验\t胃液气味" +
                ",临床体液检验\t胃液乳酸检测" +
                ",临床体液检验\t胃液锌检测" +
                ",临床体液检验\t胃液隐血试验" +
                ",临床体液检验\t无精子因子检验" +
                ",临床体液检验\t线索细胞检查（湿片法）" +
                ",临床体液检验\t线索细胞检查（阴道菌群革兰氏染色法）" +
                ",临床体液检验\t细菌性阴道病检验" +
                ",临床体液检验\t胸腔积液微生物学检查" +
                ",临床体液检验\t胸腔积液β<sub>2</sub>-微球蛋白" +
                ",临床体液检验\t胸水常规检查" +
                ",临床体液检验\t胸（腹）水及心包积液常规检查" +
                ",临床体液检验\t血浆、尿液渗透压测定" +
                ",临床体液检验\t血清、24小时尿液尿素氮测定" +
                ",临床体液检验\t羊水氨基酸测定" +
                ",临床体液检验\t羊水采集" +
                ",临床体液检验\t羊水促黄体生成素测定" +
                ",临床体液检验\t羊水催乳素测定" +
                ",临床体液检验\t羊水促肾上腺皮质激素测定" +
                ",临床体液检验\t羊水胆红素测定" +
                ",临床体液检验\t羊水钙测定" +
                ",临床体液检验\t羊水睾酮测定" +
                ",临床体液检验\t羊水钾测定" +
                ",临床体液检验\t羊水碱性磷酸酶测定" +
                ",临床体液检验\t羊水甲状腺素测定" +
                ",临床体液检验\t羊水肌酐测定" +
                ",临床体液检验\t羊水磷脂酰甘油测定" +
                ",临床体液检验\t羊水氯测定" +
                ",临床体液检验\t羊水卵磷脂测定" +
                ",临床体液检验\t羊水卵泡刺激素测定" +
                ",临床体液检验\t羊水镁测定" +
                ",临床体液检验\t羊水钠测定" +
                ",临床体液检验\t羊水皮质醇测定" +
                ",临床体液检验\t羊水前列腺素E测定" +
                ",临床体液检验\t羊水前列腺素F测定" +
                ",临床体液检验\t羊水人绒毛膜促性腺激素测定" +
                ",临床体液检验\t羊水人胎盘生乳素测定" +
                ",临床体液检验\t羊水乳铁蛋白测定" +
                ",临床体液检验\t羊水生长激素测定" +
                ",临床体液检验\t羊水铁测定" +
                ",临床体液检验\t羊水铁蛋白测定" +
                ",临床体液检验\t羊水维生素A测定" +
                ",临床体液检验\t羊水维生素B<sub>12</sub>测定" +
                ",临床体液检验\t羊水维生素D测定" +
                ",临床体液检验\t羊水维生素E测定" +
                ",临床体液检验\t羊水血型物质测定" +
                ",临床体液检验\t羊水胰岛素测定<b></b>" +
                ",临床体液检验\t羊水乙酰胆碱酯酶测定" +
                ",临床体液检验\t羊水孕酮测定" +
                ",临床体液检验\t羊水转铁蛋白测定" +
                ",临床体液检验\t羊水总蛋白测定" +
                ",临床体液检验\t羊水γ-谷氨酰基转移酶测定" +
                ",临床体液检验\t咽拭子细菌培养及药敏试验" +
                ",临床体液检验\t阴道分泌物胺试验" +
                ",临床体液检验\t阴道分泌物的采集" +
                ",临床体液检验\t阴道分泌物干化学酶学检验" +
                ",临床体液检验\t阴道分泌物检查" +
                ",临床体液检验\t阴道分泌物pH测定" +
                ",临床体液检验\t阴道分泌物湿片显微镜检查" +
                ",临床体液检验\t阴道分泌物涂片革兰染色显微镜检查" +
                ",临床体液检验\t阴道过氧化氢试验" +
                ",临床体液检验\t阴道活检" +
                ",临床体液检验\t阴道毛滴虫病检验" +
                ",临床体液检验\t阴道清洁度评估" +
                ",临床体液检验\t阴道脱落细胞检验" +
                ",临床体液检验\t阴道唾液酸酐酶检验" +
                ",临床体液检验\t阴道真菌检验" +
                ",临床体液检验\t阴道支原体感染检验" +
                ",临床体液检验\t荧光偏振试验" +
                ",临床体液检验\t支气管肺泡灌洗液检验" +
                ",临床体液检验\t昼夜尿比密试验和3h尿比密试验" +
                ",临床化学检验\t11-脱氧皮质醇测定" +
                ",临床化学检验\t11β-羟化酶测定" +
                ",临床化学检验\t17-羟孕酮测定" +
                ",临床化学检验\t17α-羟化酶测定" +
                ",临床化学检验\t1，25-二羟维生素D<sub>3</sub>测定" +
                ",临床化学检验\t21-羟化酶测定" +
                ",临床化学检验\t21-羟化酶抗体测定" +
                ",临床化学检验\t24h尿液电解质测定" +
                ",临床化学检验\t24小时尿18-羟皮质酮测定" +
                ",临床化学检验\t24小时尿促肾上腺皮质激素测定" +
                ",临床化学检验\t24小时尿碘测定" +
                ",临床化学检验\t24小时尿多巴胺测定" +
                ",临床化学检验\t24小时尿氟测定" +
                ",临床化学检验\t24小时尿钾测定" +
                ",临床化学检验\t24小时尿锰测定" +
                ",临床化学检验\t24小时尿钼测定" +
                ",临床化学检验\t24小时尿钠测定" +
                ",临床化学检验\t24小时尿镍测定" +
                ",临床化学检验\t24小时尿铜测定" +
                ",临床化学检验\t24小时尿维生素B<sub>2</sub>测定" +
                ",临床化学检验\t24小时尿维生素B<sub>5</sub>测定" +
                ",临床化学检验\t24小时尿维生素C测定" +
                ",临床化学检验\t24小时尿硒测定" +
                ",临床化学检验\t24小时尿液儿茶酚胺测定" +
                ",临床化学检验\t24小时尿液生长激素测定" +
                ",临床化学检验\t24小时尿总雌三醇测定" +
                ",临床化学检验\t24小时尿（血）18-羟皮质醇测定" +
                ",临床化学检验\t3-甲氧基去甲肾上腺素测定" +
                ",临床化学检验\t3β-羟类固醇脱氢酶测定" +
                ",临床化学检验\t5'-核苷酸酶测定" +
                ",临床化学检验\t5-羟色胺释放试验" +
                ",临床化学检验\t5′-核苷酸磷酸二酯酶" +
                ",临床化学检验\t9α-氟氢可的松试验" +
                ",临床化学检验\t<sup>14</sup>C-氨基比林呼气试验" +
                ",临床化学检验\tα-L-岩藻糖苷酶测定" +
                ",临床化学检验\tα-半乳糖苷酶测定" +
                ",临床化学检验\tα-羟丁酸脱氢酶测定" +
                ",临床化学检验\tα<sub>1</sub>-酸性糖蛋白测定" +
                ",临床化学检验\tβ-N-乙酰氨基葡萄糖苷酶测定" +
                ",临床化学检验\tβ-氨基己糖苷酶测定" +
                ",临床化学检验\tβ-淀粉样蛋白测定" +
                ",临床化学检验\tβ-内啡肽测定" +
                ",临床化学检验\tγ-谷氨酰基转移酶同工酶测定" +
                ",临床化学检验\tδ-胆红素测定" +
                ",临床化学检验\tⅠ型胶原吡啶交联终肽测定" +
                ",临床化学检验\tⅠ型胶原交联C末端肽、N末端肽测定" +
                ",临床化学检验\tⅠ型胶原交联羧基末端肽测定" +
                ",临床化学检验\tⅠ型前胶原前肽测定" +
                ",临床化学检验\tⅠ型前胶原羧基、氨基前肽测定" +
                ",临床化学检验\tⅠ型前胶原肽测定" +
                ",临床化学检验\tⅣ型胶原7S片段测定" +
                ",临床化学检验\tⅣ型胶原NC1片段测定" +
                ",临床化学检验\tⅣ型胶原测定" +
                ",临床化学检验\t阿扑吗啡试验" +
                ",临床化学检验\t噻嗪类利尿药兴奋试验" +
                ",临床化学检验\t糖化血红蛋白测定——酶法" +
                ",临床化学检验\t血清氯化物测定——电量分析法" +
                ",临床化学检验\t血清氯化物测定——硫氰酸汞比色法" +
                ",临床化学检验\t血清氯化物测定——酶法测定" +
                ",临床化学检验\t血清总钙测定——甲基麝香草酚蓝比色法" +
                ",临床化学检验\tACTH兴奋试验" +
                ",临床化学检验\tCRH-LVP兴奋试验" +
                ",临床化学检验\tC肽检测" +
                ",临床化学检验\tGuthrie试验" +
                ",临床化学检验\tICA512自身抗体测定" +
                ",临床化学检验\tL-γ-谷氨酰基转移酶测定" +
                ",临床化学检验\tLDL／HDL比值" +
                ",临床化学检验\tLHRH垂体兴奋试验" +
                ",临床化学检验\tN-乙酰-β-D-氨基葡萄糖苷酶同工酶测定" +
                ",临床化学检验\tN<sup>5</sup>-甲基四氢叶酸-同型半胱氨酸甲基转移酶测定" +
                ",临床化学检验\tN<sup>5</sup>，N<sup>10</sup>-甲烯四氢叶酸还原酶测定" +
                ",临床化学检验\tPaigen试验" +
                ",临床化学检验\tP物质测定" +
                ",临床化学检验\tTRH兴奋试验" +
                ",临床化学检验\tTRH／LHRH兴奋试验" +
                ",临床化学检验\tTT<sub>4</sub>／TBG比值" +
                ",临床化学检验\t阿米卡星血药浓度测定" +
                ",临床化学检验\t阿米替林血药浓度测定" +
                ",临床化学检验\t白蛋白／球蛋白比值" +
                ",临床化学检验\t半胱氨酸测定" +
                ",临床化学检验\t半乳糖-1-磷酸尿苷酰转移酶测定" +
                ",临床化学检验\t半乳糖耐量试验" +
                ",临床化学检验\t半乳糖凝集素-3测定" +
                ",临床化学检验\t半乳糖血症个体化检测" +
                ",临床化学检验\t备解素因子B测定" +
                ",临床化学检验\t苯巴比妥血药浓度测定" +
                ",临床化学检验\t苯丙氨酸测定" +
                ",临床化学检验\t苯妥英血药浓度测定" +
                ",临床化学检验\t便携式血糖仪床旁血糖监测" +
                ",临床化学检验\t标准碳酸氢盐测定" +
                ",临床化学检验\t吡啶酚、脱氧吡啶酚测定" +
                ",临床化学检验\t吡啶斯的明（PD）与左旋多巴（L-dopa）联合兴奋试验" +
                ",临床化学检验\t丙氨酸氨基转移酶测定" +
                ",临床化学检验\t丙氨酸氨基转移酶同工酶测定" +
                ",临床化学检验\t丙氨酸测定" +
                ",临床化学检验\t丙吡胺血药浓度测定" +
                ",临床化学检验\t丙米嗪血药浓度测定" +
                ",临床化学检验\t丙咪嗪血药浓度测定" +
                ",临床化学检验\t丙酮酸检测" +
                ",临床化学检验\t丙戊酸血药浓度测定" +
                ",临床化学检验\t层粘连蛋白测定" +
                ",临床化学检验\t茶碱血药浓度测定" +
                ",临床化学检验\t超敏雌二醇测定" +
                ",临床化学检验\t超氧化物歧化酶测定" +
                ",临床化学检验\t垂体促性腺激素直接测定" +
                ",临床化学检验\t垂体后叶素注射试验" +
                ",临床化学检验\t垂体激素检验" +
                ",临床化学检验\t垂体腺苷酸环化酶激活肽测定" +
                ",临床化学检验\t触珠蛋白测定" +
                ",临床化学检验\t雌二醇测定" +
                ",临床化学检验\t雌激素撤退试验" +
                ",临床化学检验\t雌激素受体测定" +
                ",临床化学检验\t雌四醇测定" +
                ",临床化学检验\t雌酮测定" +
                ",临床化学检验\t雌、孕激素联合试验" +
                ",临床化学检验\t促黄体素测定" +
                ",临床化学检验\t催产素酶测定" +
                ",临床化学检验\t催乳素测定" +
                ",临床化学检验\t促甲状腺激素测定" +
                ",临床化学检验\t促甲状腺激素释放激素兴奋试验" +
                ",临床化学检验\t促甲状腺激素受体抗体测定" +
                ",临床化学检验\t促甲状腺素释放激素兴奋试验" +
                ",临床化学检验\t促肾上腺皮质激素和相关肽测定" +
                ",临床化学检验\t促肾上腺皮质激素释放激素测定" +
                ",临床化学检验\t促肾上腺皮质激素释放激素兴奋试验" +
                ",临床化学检验\t促胃液素测定" +
                ",临床化学检验\t粗纤维调节素测定" +
                ",临床化学检验\t促性腺激素释放激素测定" +
                ",临床化学检验\t促性腺激素释放激素兴奋试验" +
                ",临床化学检验\t促胰液素测定" +
                ",临床化学检验\t大剂量地塞米松抑制试验" +
                ",临床化学检验\t蛋氨酸测定" +
                ",临床化学检验\t单胺氧化酶测定" +
                ",临床化学检验\t蛋白结合己糖测定" +
                ",临床化学检验\t胆固醇调节元件结合蛋白测定" +
                ",临床化学检验\t胆固醇裂解酶测定" +
                ",临床化学检验\t胆固醇酯转移蛋白测定" +
                ",临床化学检验\t胆红素代谢功能检测" +
                ",临床化学检验\t胆碱酯酶测定" +
                ",临床化学检验\t胆碱酯酶同工酶测定" +
                ",临床化学检验\t胆囊收缩素测定" +
                ",临床化学检验\t胆石胆固醇测定" +
                ",临床化学检验\t胆石胆红素测定" +
                ",临床化学检验\t胆石胆绿素测定" +
                ",临床化学检验\t胆石钙测定" +
                ",临床化学检验\t胆石磷脂测定" +
                ",临床化学检验\t胆汁胆酸测定" +
                ",临床化学检验\t胆汁钙测定" +
                ",临床化学检验\t胆汁磷脂测定" +
                ",临床化学检验\t胆汁酸测定" +
                ",临床化学检验\t胆汁酸代谢功能检测" +
                ",临床化学检验\t胆汁无机磷测定" +
                ",临床化学检验\t胆汁直接胆红素测定" +
                ",临床化学检验\t胆汁总胆汁酸测定" +
                ",临床化学检验\t胆汁β-葡萄糖醛酸苷酶测定" +
                ",临床化学检验\t淀粉酶测定" +
                ",临床化学检验\t淀粉酶同工酶测定" +
                ",临床化学检验\t靛氰绿试验" +
                ",临床化学检验\t地布卡因指数" +
                ",临床化学检验\t低分子量肝素监测" +
                ",临床化学检验\t低钙试验" +
                ",临床化学检验\t地高辛血药浓度测定" +
                ",临床化学检验\t抵抗素测定" +
                ",临床化学检验\t低磷试验" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定——Friedewald公式计算法" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定——PVS沉淀法" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定——匀相测定法" +
                ",临床化学检验\t低密度脂蛋白胆固醇测定——β-定量法" +
                ",临床化学检验\t低密度脂蛋白受体功能试验" +
                ",临床化学检验\t低密度脂蛋白受体相关蛋白测定" +
                ",临床化学检验\t低密度脂蛋白亚型测定" +
                ",临床化学检验\t地塞米松抑制试验" +
                ",临床化学检验\t动脉-肺泡氧分压比测定" +
                ",临床化学检验\t对乙酰氨基酚血药浓度测定" +
                ",临床化学检验\t多虑平血药浓度测定" +
                ",临床化学检验\t二氢生物喋啶还原酶测定" +
                ",临床化学检验\t二氧化碳结合力测定" +
                ",临床化学检验\t二氧化碳总量测定" +
                ",临床化学检验\t放射免疫分析" +
                ",临床化学检验\t反三碘甲状腺原氨酸测定" +
                ",临床化学检验\t肺表面活性蛋白质D测定" +
                ",临床化学检验\t非高密度脂蛋白胆固醇测定" +
                ",临床化学检验\t非结合胆红素测定" +
                ",临床化学检验\t肺泡-动脉氧分压差测定" +
                ",临床化学检验\t肺泡氧分压测定" +
                ",临床化学检验\t酚红排泄试验" +
                ",临床化学检验\t酚妥拉明试验" +
                ",临床化学检验\t脯氨酸测定" +
                ",临床化学检验\t脯氨酰羟化酶测定" +
                ",临床化学检验\t氟胞嘧啶血药浓度测定" +
                ",临床化学检验\t呋塞米兴奋试验" +
                ",临床化学检验\t改良J-G法血清胆红素测定" +
                ",临床化学检验\t钙耐量试验" +
                ",临床化学检验\t甘氨胆酸测定" +
                ",临床化学检验\t甘氨鹅脱氧胆酸测定" +
                ",临床化学检验\t甘氨酸测定" +
                ",临床化学检验\t肝促凝血酶原酶测定" +
                ",临床化学检验\t刚果红试验" +
                ",临床化学检验\t肝功能常规检查" +
                ",临床化学检验\t肝素辅因子Ⅱ活性测定" +
                ",临床化学检验\t肝纤维化四项" +
                ",临床化学检验\t甘油三酯测定" +
                ",临床化学检验\t肝脂酶测定" +
                ",临床化学检验\t肝组织铜测定" +
                ",临床化学检验\t肝、胆、胰疾病常用实验检测项目" +
                ",临床化学检验\t高尔基体蛋白73测定" +
                ",临床化学检验\t高密度脂蛋白胆固醇测定" +
                ",临床化学检验\t高密度脂蛋白胆固醇测定——超速离心结合选择性沉淀法" +
                ",临床化学检验\t高密度脂蛋白胆固醇测定——硫酸葡聚糖-Mg沉淀法和磷钨酸-镁沉淀法" +
                ",临床化学检验\t高密度脂蛋白胆固醇测定——匀相测定法" +
                ",临床化学检验\t高密度脂蛋白亚型测定" +
                ",临床化学检验\t高敏促甲状腺激素测定" +
                ",临床化学检验\t高渗盐水试验" +
                ",临床化学检验\t高糖基化人绒毛膜促性腺激素测定" +
                ",临床化学检验\t睾酮测定" +
                ",临床化学检验\t骨I型胶原降解产物测定" +
                ",临床化学检验\t瓜氨酸测定" +
                ",临床化学检验\t胱硫醚β合成酶测定" +
                ",临床化学检验\t谷氨酸测定" +
                ",临床化学检验\t谷氨酸脱氢酶测定" +
                ",临床化学检验\t谷氨酸脱羧酶65自身抗体测定" +
                ",临床化学检验\t谷氨酰胺测定" +
                ",临床化学检验\t谷胱甘肽还原酶检测" +
                ",临床化学检验\t谷胱甘肽转移酶测定" +
                ",临床化学检验\t谷胱甘肽转移酶同工酶" +
                ",临床化学检验\t骨碱性磷酸酶测定" +
                ",临床化学检验\t过氯酸钾排泄试验" +
                ",临床化学检验\t过氯酸钾释放试验" +
                ",临床化学检验\t果糖胺检测" +
                ",临床化学检验\t估算的肾小球滤过率" +
                ",临床化学检验\t汗液氯化物测定" +
                ",临床化学检验\t黑色细胞刺激素" +
                ",临床化学检验\t红细胞铬测定" +
                ",临床化学检验\t红细胞钴测定" +
                ",临床化学检验\t红细胞还原型谷胱甘肽测定" +
                ",临床化学检验\t红细胞磷酸甘油酸激酶" +
                ",临床化学检验\t红细胞维生素B<sub>1</sub>测定" +
                ",临床化学检验\t红细胞锌测定" +
                ",临床化学检验\t环孢霉素A测定" +
                ",临床化学检验\t环孢素血药浓度测定" +
                ",临床化学检验\t缓冲碱测定" +
                ",临床化学检验\t环鸟苷酸测定" +
                ",临床化学检验\t火焰光度法血清钠钾测定" +
                ",临床化学检验\t甲氨蝶呤血药浓度测定" +
                ",临床化学检验\t钾负荷试验" +
                ",临床化学检验\t甲功五项" +
                ",临床化学检验\t甲亢平（卡比马唑）试验" +
                ",临床化学检验\t碱负荷试验" +
                ",临床化学检验\t降钙素基因相关肽测定" +
                ",临床化学检验\t渐进性糖基化终末产物测定" +
                ",临床化学检验\t碱剩余测定" +
                ",临床化学检验\t肌氨酸测定" +
                ",临床化学检验\t碱性磷酸酶测定" +
                ",临床化学检验\t碱性磷酸酶同工酶测定" +
                ",临床化学检验\t简易高渗盐水试验" +
                ",临床化学检验\t甲氧基肾上腺素和甲氧基去甲肾上腺素测定" +
                ",临床化学检验\t甲氧氯普胺兴奋试验" +
                ",临床化学检验\t甲状旁腺素测定");
        jianyan.append(",临床化学检验\t甲状旁腺素相关蛋白测定" +
                ",临床化学检验\t甲状腺刺激免疫球蛋白测定" +
                ",临床化学检验\t甲状腺功能异常筛查试验" +
                ",临床化学检验\t甲状腺激素抗体测定" +
                ",临床化学检验\t甲状腺激素抑制试验" +
                ",临床化学检验\t甲状腺激素自身抗体测定" +
                ",临床化学检验\t甲状腺片抑制甲状腺吸<sup>131</sup>I试验" +
                ",临床化学检验\t甲状腺素结合力测定" +
                ",临床化学检验\t甲状腺微粒体抗体测定" +
                ",临床化学检验\t甲状腺吸<sup>131</sup>I率试验" +
                ",临床化学检验\t基础代谢率测定" +
                ",临床化学检验\t基础体温测定" +
                ",临床化学检验\t极低密度脂蛋白受体测定" +
                ",临床化学检验\t结合胆红素测定" +
                ",临床化学检验\t饥饿试验" +
                ",临床化学检验\t精氨酸测定" +
                ",临床化学检验\t精氨酸酶测定" +
                ",临床化学检验\t精氨酸兴奋试验" +
                ",临床化学检验\t精氨酰琥珀酸裂解酶测定" +
                ",临床化学检验\t经皮脐血采集" +
                ",临床化学检验\t禁水与垂体后叶素联合试验" +
                ",临床化学检验\t肌酸激酶测定" +
                ",临床化学检验\t巨催乳素测定" +
                ",临床化学检验\t菊粉清除率试验" +
                ",临床化学检验\t咖啡因血药浓度测定" +
                ",临床化学检验\t开博通抑制试验" +
                ",临床化学检验\t卡马西平血药浓度测定" +
                ",临床化学检验\t抗蛋白酶3抗体测定" +
                ",临床化学检验\t抗甲状腺球蛋白抗体测定" +
                ",临床化学检验\t抗甲状腺微粒体抗体测定" +
                ",临床化学检验\t抗利尿激素测定" +
                ",临床化学检验\t抗苗勒管激素测定" +
                ",临床化学检验\t抗人绒毛膜促性腺激素抗体测定" +
                ",临床化学检验\t抗生素最低抑／杀菌浓度测定" +
                ",临床化学检验\t抗髓过氧化物酶抗体测定" +
                ",临床化学检验\t抗心肌抗体测定" +
                ",临床化学检验\t卡托普利抑制试验" +
                ",临床化学检验\t可乐定兴奋试验" +
                ",临床化学检验\t可溶性CD40配体测定" +
                ",临床化学检验\t口服葡萄糖耐量试验" +
                ",临床化学检验\t快速滴注钙抑制试验" +
                ",临床化学检验\t奎尼丁血药浓度测定" +
                ",临床化学检验\t赖氨酸测定" +
                ",临床化学检验\t拉莫三嗪血药浓度测定" +
                ",临床化学检验\t酪胺试验" +
                ",临床化学检验\t酪氨酸测定" +
                ",临床化学检验\t酪酪肽测定" +
                ",临床化学检验\t类胆固醇颗粒样残留物测定" +
                ",临床化学检验\t类甘油三酯颗粒样残留物测定" +
                ",临床化学检验\t类胰蛋白酶测定" +
                ",临床化学检验\t冷加压试验" +
                ",临床化学检验\t亮氨酸测定" +
                ",临床化学检验\t利多卡因试验" +
                ",临床化学检验\t利多卡因血药浓度测定" +
                ",临床化学检验\t利伐沙班监测" +
                ",临床化学检验\t磷清除率试验" +
                ",临床化学检验\t磷清除试验" +
                ",临床化学检验\t磷酸己糖异构酶测定" +
                ",临床化学检验\t磷酸盐排泄指数" +
                ",临床化学检验\t磷酸组织胺试验" +
                ",临床化学检验\t磷重吸收试验" +
                ",临床化学检验\t硫酸雌酮测定" +
                ",临床化学检验\t硫酸脱氢表雄酮测定" +
                ",临床化学检验\t硫酸脱氢表雄酮负荷试验" +
                ",临床化学检验\t锂血药浓度测定" +
                ",临床化学检验\t氯丙嗪兴奋试验" +
                ",临床化学检验\t滤过钠排泄分数" +
                ",临床化学检验\t氯米芬负荷试验" +
                ",临床化学检验\t氯压定（可乐定）试验" +
                ",临床化学检验\t卵磷脂胆固醇酯酰转移酶测定" +
                ",临床化学检验\t卵磷脂胆固醇脂酰转移酶测定" +
                ",临床化学检验\t卵泡刺激素测定" +
                ",临床化学检验\t螺内酯（安体舒通）醛固酮比率测定" +
                ",临床化学检验\t麦考酚酸血药浓度测定" +
                ",临床化学检验\t毛细管血总二氧化碳测定" +
                ",临床化学检验\t霉酚酸酯测定" +
                ",临床化学检验\t美替拉酮兴奋试验" +
                ",临床化学检验\t免疫透射比浊法测定ApoAⅠ和ApoB" +
                ",临床化学检验\t免疫透射比浊法测定ApoE" +
                ",临床化学检验\t免疫透射比浊法测定脂蛋白（a）" +
                ",临床化学检验\t泌乳素测定" +
                ",临床化学检验\t泌乳素分泌兴奋试验" +
                ",临床化学检验\t泌乳素分泌抑制试验" +
                ",临床化学检验\t木糖吸收试验" +
                ",临床化学检验\t钠负荷试验" +
                ",临床化学检验\t奈替米星血药浓度测定" +
                ",临床化学检验\t脑啡肽测定" +
                ",临床化学检验\t脑脊液氨基酸谱测定" +
                ",临床化学检验\t脑脊液白蛋白测定" +
                ",临床化学检验\t脑脊液免疫球蛋白G测定" +
                ",临床化学检验\t脑脊液总蛋白测定" +
                ",临床化学检验\t脑钠肽测定" +
                ",临床化学检验\t内生肌酐清除率测定" +
                ",临床化学检验\t内因子测定" +
                ",临床化学检验\t内因子阻滞抗体测定" +
                ",临床化学检验\t黏多糖测定" +
                ",临床化学检验\t尿17-羟类固醇测定" +
                ",临床化学检验\t尿17-生酮皮质类固醇测定" +
                ",临床化学检验\t尿17-酮类固醇测定" +
                ",临床化学检验\t尿3-甲氧基-4-羟苦杏仁酸测定" +
                ",临床化学检验\t鸟氨酸氨基甲酰转移酶测定" +
                ",临床化学检验\t鸟氨酸测定" +
                ",临床化学检验\t尿雌激素／肌酐比值" +
                ",临床化学检验\t尿促性腺激素刺激试验" +
                ",临床化学检验\t尿蛋白电泳" +
                ",临床化学检验\t尿蛋白选择性指数测定" +
                ",临床化学检验\t尿道分泌物检查" +
                ",临床化学检验\t尿结石铵盐测定" +
                ",临床化学检验\t尿结石胞苷测定" +
                ",临床化学检验\t尿结石草酸钙测定" +
                ",临床化学检验\t尿结石胆固醇测定" +
                ",临床化学检验\t尿结石胆红素测定" +
                ",临床化学检验\t尿结石胆绿素测定" +
                ",临床化学检验\t尿结石靛蓝测定" +
                ",临床化学检验\t尿结石钙测定" +
                ",临床化学检验\t尿结石胱氨酸测定" +
                ",临床化学检验\t尿结石磺胺类药物测定" +
                ",临床化学检验\t尿结石黄嘌呤测定" +
                ",临床化学检验\t尿结石磷酸盐测定" +
                ",临床化学检验\t尿结石镁测定" +
                ",临床化学检验\t尿结石尿酸测定" +
                ",临床化学检验\t尿结石尿脂石测定" +
                ",临床化学检验\t尿结石碳酸盐测定" +
                ",临床化学检验\t尿结石铁测定" +
                ",临床化学检验\t尿结石纤维蛋白测定" +
                ",临床化学检验\t尿滤过钠排泄分数测定" +
                ",临床化学检验\t尿氯化物测定" +
                ",临床化学检验\t尿免疫球蛋白G测定" +
                ",临床化学检验\t尿三氯化铁试验" +
                ",临床化学检验\t尿视黄醇结合蛋白测定" +
                ",临床化学检验\t尿香草基苦杏仁酸测定" +
                ",临床化学检验\t尿液17-羟皮质类固醇测定" +
                ",临床化学检验\t尿液17-酮类固醇测定" +
                ",临床化学检验\t尿液17-酮皮质类固醇测定" +
                ",临床化学检验\t尿液5-羟吲哚乙酸测定" +
                ",临床化学检验\t尿液还原糖试验" +
                ",临床化学检验\t尿液浓缩稀释试验" +
                ",临床化学检验\t尿液香草扁桃酸测定" +
                ",临床化学检验\t尿中4-吡哆酸含量测定" +
                ",临床化学检验\t尿转铁蛋白测定" +
                ",临床化学检验\t尿总蛋白测定" +
                ",临床化学检验\t牛磺酸测定" +
                ",临床化学检验\tp24蛋白测定" +
                ",临床化学检验\t嘌呤/嘧啶测定" +
                ",临床化学检验\t苹果酸脱氢酶测定" +
                ",临床化学检验\t皮质醇测定" +
                ",临床化学检验\t皮质醇昼夜节律试验" +
                ",临床化学检验\t皮质类固醇结合蛋白测定" +
                ",临床化学检验\t皮质酮测定" +
                ",临床化学检验\t普乐可复（FK506）测定" +
                ",临床化学检验\t普鲁卡因血药浓度测定" +
                ",临床化学检验\t葡萄糖负荷试验" +
                ",临床化学检验\t葡萄糖脑苷酶测定" +
                ",临床化学检验\t普通血栓弹力图实验" +
                ",临床化学检验\t强啡肽测定" +
                ",临床化学检验\t羟甲基戊二酰辅酶A还原酶测定" +
                ",临床化学检验\t鞘磷脂酶测定" +
                ",临床化学检验\t壳三糖苷酶测定" +
                ",临床化学检验\t庆大霉素血药浓度测定" +
                ",临床化学检验\t清道夫受体测定" +
                ",临床化学检验\t青霉胺负荷试验" +
                ",临床化学检验\t巯基嘌呤甲基转移酶测定" +
                ",临床化学检验\t醛固酮-18-葡萄糖醛酸苷测定" +
                ",临床化学检验\t醛固酮测定" +
                ",临床化学检验\t醛固酮合成酶测定" +
                ",临床化学检验\t醛固酮／肾素比值" +
                ",临床化学检验\t去氨加压素试验" +
                ",临床化学检验\t醛缩酶同工酶测定" +
                ",临床化学检验\t全血葡萄糖-6-磷酸酶测定" +
                ",临床化学检验\t缺血性修饰白蛋白测定" +
                ",临床化学检验\t去甲丙米嗪血药浓度测定" +
                ",临床化学检验\t去甲肾上腺素测定" +
                ",临床化学检验\t去甲替林血药浓度测定" +
                ",临床化学检验\t人绒毛膜促甲状腺激素测定" +
                ",临床化学检验\t人绒毛膜促性腺激素测定" +
                ",临床化学检验\t人绒毛膜促性腺激素刺激试验" +
                ",临床化学检验\t妊娠带蛋白测定" +
                ",临床化学检验\t妊娠期糖尿病筛查试验" +
                ",临床化学检验\t妊娠期糖尿病筛查试验" +
                ",临床化学检验\t妊娠特异性β<sub>1</sub>-糖蛋白" +
                ",临床化学检验\t妊娠相关血浆蛋白A测定" +
                ",临床化学检验\t妊娠相关血浆蛋白B测定" +
                ",临床化学检验\t人胎盘催乳素测定" +
                ",临床化学检验\t热休克蛋白" +
                ",临床化学检验\t乳酸检测" +
                ",临床化学检验\t乳酸脱氢酶测定" +
                ",临床化学检验\t乳酸脱氢酶同工酶测定" +
                ",临床化学检验\t色氨酸测定" +
                ",临床化学检验\t上消化道癌人群筛查及早诊早治技术方案" +
                ",临床化学检验\t山梨醇脱氢酶测定" +
                ",临床化学检验\t肾功能常规检查" +
                ",临床化学检验\t生物素酶测定" +
                ",临床化学检验\t生长激素测定" +
                ",临床化学检验\t生长激素和胰岛素样生长因子测定" +
                ",临床化学检验\t生长激素结合蛋白测定" +
                ",临床化学检验\t生长激素抗体测定" +
                ",临床化学检验\t生长激素释放激素测定" +
                ",临床化学检验\t生长激素释放激素兴奋试验" +
                ",临床化学检验\t生长激素释放抑制激素测定" +
                ",临床化学检验\t生长激素释放抑制素测定" +
                ",临床化学检验\t生长激素兴奋试验" +
                ",临床化学检验\t生长激素抑制试验" +
                ",临床化学检验\t神经降压素测定" +
                ",临床化学检验\t神经肽Y测定" +
                ",临床化学检验\t肾上腺素测定" +
                ",临床化学检验\t肾素测定" +
                ",临床化学检验\t肾小管葡萄糖最大重吸收量测定" +
                ",临床化学检验\t肾小管重吸收磷率试验" +
                ",临床化学检验\t肾小管最大对氨基马尿酸排泄量试验" +
                ",临床化学检验\t实际碳酸氢盐测定" +
                ",临床化学检验\t石墨炉原子吸收光谱法全血铅测定" +
                ",临床化学检验\t瘦素测定" +
                ",临床化学检验\t双氢睾酮测定" +
                ",临床化学检验\t丝氨酸测定" +
                ",临床化学检验\t丝氨酸蛋白酶测定" +
                ",临床化学检验\t酸负荷试验" +
                ",临床化学检验\t苏氨酸测定" +
                ",临床化学检验\t酸性磷酸酶测定" +
                ",临床化学检验\t酸性磷酸酶同工酶测定" +
                ",临床化学检验\t髓过氧化物酶测定" +
                ",临床化学检验\t随机尿氨基酸谱测定" +
                ",临床化学检验\t随机尿儿茶酚胺测定" +
                ",临床化学检验\t随机尿高香草酸测定" +
                ",临床化学检验\t随机尿香草扁桃酸测定" +
                ",临床化学检验\t随机尿总甲氧基去甲肾上腺素测定" +
                ",临床化学检验\t随机尿β<sub>2</sub>-微球蛋白清除率" +
                ",临床化学检验\t髓鞘相关糖蛋白测定" +
                ",临床化学检验\t速尿（呋塞米）＋立位试验" +
                ",临床化学检验\t胎儿纤连蛋白测定" +
                ",临床化学检验\t胎盘蛋白5测定" +
                ",临床化学检验\t胎盘碱性磷酸酶测定" +
                ",临床化学检验\t他克莫司血药浓度测定" +
                ",临床化学检验\t糖蛋白电泳" +
                ",临床化学检验\t糖化血红蛋白测定——HPLC法" +
                ",临床化学检验\t糖化血红蛋白测定——免疫比浊法" +
                ",临床化学检验\t糖化血红蛋白测定——亲和层析法" +
                ",临床化学检验\t糖化血红蛋白检测" +
                ",临床化学检验\t糖化血清蛋白测定" +
                ",临床化学检验\t糖原磷酸化酶BB测定" +
                ",临床化学检验\t糖原磷酸化酶同工酶" +
                ",临床化学检验\t碳酸锂血药浓度测定" +
                ",临床化学检验\t天冬氨酸氨基转移酶测定" +
                ",临床化学检验\t天冬氨酸氨基转移酶同工酶测定" +
                ",临床化学检验\t天门冬氨酸氨基转移酶同工酶测定" +
                ",临床化学检验\t透明质酸测定" +
                ",临床化学检验\t妥布霉素血药浓度测定" +
                ",临床化学检验\t脱氧皮质酮测定" +
                ",临床化学检验\t唾液游离皮质醇测定" +
                ",临床化学检验\t万古霉素血药浓度测定" +
                ",临床化学检验\t蛙皮素测定" +
                ",临床化学检验\t胃蛋白酶测定" +
                ",临床化学检验\t胃蛋白酶原Ⅰ、Ⅱ测定" +
                ",临床化学检验\t胃动素测定" +
                ",临床化学检验\t微分电位溶出法全血铅测定" +
                ",临床化学检验\t胃复安（甲氧氯普胺）试验" +
                ",临床化学检验\t微量白蛋白尿测定" +
                ",临床化学检验\t微量元素筛查试验" +
                ",临床化学检验\t微粒体甘油三酯转移蛋白测定" +
                ",临床化学检验\t胃泌素测定" +
                ",临床化学检验\t维生素A测定" +
                ",临床化学检验\t维生素A原测定" +
                ",临床化学检验\t维生素B<sub>2</sub>测定" +
                ",临床化学检验\t维生素B<sub>3</sub>测定" +
                ",临床化学检验\t维生素B<sub>5</sub>测定" +
                ",临床化学检验\t维生素B<sub>6</sub>测定" +
                ",临床化学检验\t维生素B<sub>7</sub>测定" +
                ",临床化学检验\t维生素E测定" +
                ",临床化学检验\t维生素K测定" +
                ",临床化学检验\t钨舟无焰原子吸收光谱全血铅测定" +
                ",临床化学检验\t腺苷脱氨酶测定" +
                ",临床化学检验\t酰基辅酶A胆固醇酰基转移酶-2测定" +
                ",临床化学检验\t线粒体呼吸链复合酶测定" +
                ",临床化学检验\t先天性甲状腺功能低下筛查试验" +
                ",临床化学检验\t小而密低密度脂蛋白测定" +
                ",临床化学检验\t硝基苯肼试验" +
                ",临床化学检验\t小剂量地塞米松抑制试验" +
                ",临床化学检验\t细胞间黏附分子-1测定" +
                ",临床化学检验\t缬氨酸测定" +
                ",临床化学检验\t心房利钠肽测定" +
                ",临床化学检验\t性激素结合蛋白测定" +
                ",临床化学检验\t性激素结合球蛋白测定" +
                ",临床化学检验\t性激素六项" +
                ",临床化学检验\t心肌肌钙蛋白T测定" +
                ",临床化学检验\t心肌肌钙蛋白Ⅰ测定" +
                ",临床化学检验\t心肌酶谱四项" +
                ",临床化学检验\t心肌酶谱五项" +
                ",临床化学检验\t心肌纤维蛋白原活性测定" +
                ",临床化学检验\t心钠素测定" +
                ",临床化学检验\t新生儿半乳糖测定" +
                ",临床化学检验\t新生儿胆红素测定" +
                ",临床化学检验\t新生儿先天性甲状腺功能减退症筛查" +
                ",临床化学检验\t雄激素受体测定" +
                ",临床化学检验\t雄烯二酮测定" +
                ",临床化学检验\t修饰型低密度脂蛋白测定" +
                ",临床化学检验\t溴隐亭兴奋试验" +
                ",临床化学检验\t溴隐亭抑制试验" +
                ",临床化学检验\t血氨测定" +
                ",临床化学检验\t血二羟苯甘醇测定" +
                ",临床化学检验\t血管活性肠肽测定" +
                ",临床化学检验\t血管紧张素转化酶测定" +
                ",临床化学检验\t血管紧张素Ⅰ、Ⅱ测定" +
                ",临床化学检验\t血红蛋白A1c测定" +
                ",临床化学检验\t血红蛋白A1测定" +
                ",临床化学检验\t血浆(清)碳酸氢根及总二氧化碳测定" +
                ",临床化学检验\t血浆(清)阴离子间隙" +
                ",临床化学检验\t血浆5-羟色胺测定" +
                ",临床化学检验\t血浆氨基酸谱测定" +
                ",临床化学检验\t血浆丙酮酸测定" +
                ",临床化学检验\t血浆蛋白结合碘测定" +
                ",临床化学检验\t血浆多巴胺测定" +
                ",临床化学检验\t血浆儿茶酚胺测定" +
                ",临床化学检验\t血浆高香草酸测定" +
                ",临床化学检验\t血浆谷胱甘肽测定" +
                ",临床化学检验\t血浆环磷酸鸟苷测定" +
                ",临床化学检验\t血浆氯测定" +
                ",临床化学检验\t血浆前列腺素测定" +
                ",临床化学检验\t血浆醛固酮浓度及尿中醛固酮排泄量测定" +
                ",临床化学检验\t血浆乳酸测定" +
                ",临床化学检验\t血浆渗量检测" +
                ",临床化学检验\t血浆肾素活性测定" +
                ",临床化学检验\t血浆同型半胱氨酸测定" +
                ",临床化学检验\t血浆维生素C测定" +
                ",临床化学检验\t血浆乙酰胆碱测定" +
                ",临床化学检验\t血浆一氧化氮测定" +
                ",临床化学检验\t血浆、尿羟脯氨酸测定" +
                ",临床化学检验\t血浆、尿液组胺测定" +
                ",临床化学检验\t血浆、血清、全血、毛细血管全血葡萄糖测定" +
                ",临床化学检验\t血浆、羊水催产素测定" +
                ",临床化学检验\t血浆、羊水胰高血糖素测定" +
                ",临床化学检验\t血钾尿钾酶法测定" +
                ",临床化学检验\t血间羟肾上腺类似物的测定" +
                ",临床化学检验\t血锰测定" +
                ",临床化学检验\t血钠尿钠酶法测定" +
                ",临床化学检验\t血气分析" +
                ",临床化学检验\t血清3，3′，5′-三碘甲状腺原氨酸测定" +
                ",临床化学检验\t血清AST/ALT比值" +
                ",临床化学检验\t血清氨基末端-B型利钠肽前体测定" +
                ",临床化学检验\t血清氨基肽酶测定" +
                ",临床化学检验\t血清白蛋白测定" +
                ",临床化学检验\t血清半胱氨酸蛋白酶抑制蛋白C测定" +
                ",临床化学检验\t血清超长链脂肪酸测定" +
                ",临床化学检验\t血清蛋白醋酸纤维素膜电泳" +
                ",临床化学检验\t血清蛋白电泳" +
                ",临床化学检验\t血清蛋白毛细管区带电泳" +
                ",临床化学检验\t血清蛋白琼脂糖凝胶电泳" +
                ",临床化学检验\t血清胆红素测定——钒酸盐氧化法" +
                ",临床化学检验\t血清反T<sub>3</sub>测定" +
                ",临床化学检验\t血清非蛋白氮测定" +
                ",临床化学检验\t血清甘氨胆汁酸测定" +
                ",临床化学检验\t血清胱抑素C测定" +
                ",临床化学检验\t血清谷丙转氨酶测定" +
                ",临床化学检验\t血清骨钙素测定" +
                ",临床化学检验\t血清果糖测定" +
                ",临床化学检验\t血清过氧化脂质测定" +
                ",临床化学检验\t血清钾测定" +
                ",临床化学检验\t血清降钙素测定" +
                ",临床化学检验\t血清碱性磷酸酶测定" +
                ",临床化学检验\t血清甲状腺结合球蛋白测定" +
                ",临床化学检验\t血清甲状腺球蛋白测定" +
                ",临床化学检验\t血清肌钙蛋白测定" +
                ",临床化学检验\t血清肌酐测定<b></b>" +
                ",临床化学检验\t血清肌红蛋白测定" +
                ",临床化学检验\t血清肌酸激酶同工酶MB质量测定" +
                ",临床化学检验\t血清抗酒石酸酸性磷酸酶测定" +
                ",临床化学检验\t血清抗磷脂抗体测定" +
                ",临床化学检验\t血清类黏蛋白测定" +
                ",临床化学检验\t血清磷脂测定" +
                ",临床化学检验\t血清离子钙测定" +
                ",临床化学检验\t血清氯测定" +
                ",临床化学检验\t血清镁测定——Calmagite染料比色法" +
                ",临床化学检验\t血清镁测定——二甲苯胺蓝法" +
                ",临床化学检验\t血清镁测定——甲基麝香草酚蓝比色法" +
                ",临床化学检验\t血清镁测定——原子吸收分光光度法" +
                ",临床化学检验\t血清免疫固定电泳" +
                ",临床化学检验\t血清钠测定" +
                ",临床化学检验\t血清黏蛋白测定" +
                ",临床化学检验\t血清尿酸测定" +
                ",临床化学检验\t血清尿酸／肌酐比值" +
                ",临床化学检验\t血清尿素测定" +
                ",临床化学检验\t血清尿素氮／血清肌酐比值" +
                ",临床化学检验\t血清前白蛋白测定" +
                ",临床化学检验\t血清球蛋白" +
                ",临床化学检验\t血清醛缩酶测定" +
                ",临床化学检验\t血清缺血修饰白蛋白测定" +
                ",临床化学检验\t血清妊娠相关蛋白A测定" +
                ",临床化学检验\t血清溶菌酶测定" +
                ",临床化学检验\t血清三酰甘油测定——变色酸显色法(CDC参考方法)" +
                ",临床化学检验\t血清三酰甘油测定——酶法( GPO-PAP法)" +
                ",临床化学检验\t血清三酰甘油测定——去游离甘油的TG测定(二步酶法)" +
                ",临床化学检验\t血清杀菌活性测定" +
                ",临床化学检验\t血清视黄醇结合蛋白测定" +
                ",临床化学检验\t血清酸性磷酸酶测定" +
                ",临床化学检验\t血清铁比色法测定" +
                ",临床化学检验\t血清铁蛋白测定" +
                ",临床化学检验\t血清铜测定——比色法" +
                ",临床化学检验\t血清铜测定——双环己酮草酰二腙比色法" +
                ",临床化学检验\t血清铜测定——原子吸收分光光度法" +
                ",临床化学检验\t血清铜蓝蛋白测定" +
                ",临床化学检验\t血清同型半胱氨酸测定" +
                ",临床化学检验\t血清无机磷测定" +
                ",临床化学检验\t血清纤维连接蛋白测定" +
                ",临床化学检验\t血清锌测定" +
                ",临床化学检验\t血清心脏型脂肪酸结合蛋白测定" +
                ",临床化学检验\t血清乙酰胆碱酯酶测定" +
                ",临床化学检验\t血清游离T3和游离T4测定" +
                ",临床化学检验\t血清游离甘油测定" +
                ",临床化学检验\t血清游离脂肪酸测定" +
                ",临床化学检验\t血清载脂蛋白测定" +
                ",临床化学检验\t血清脂蛋白-X测定" +
                ",临床化学检验\t血清脂蛋白电泳" +
                ",临床化学检验\t血清脂肪酸结合蛋白4测定" +
                ",临床化学检验\t血清酯化胆固醇测定" +
                ",临床化学检验\t血清转铁蛋白测定" +
                ",临床化学检验\t血清总蛋白测定" +
                ",临床化学检验\t血清总胆固醇测定" +
                ",临床化学检验\t血清总钙测定——邻甲酚酞络合酮比色法" +
                ",临床化学检验\t血清总钙测定——偶氮砷Ⅲ比色法" +
                ",临床化学检验\t血清总铁结合力测定" +
                ",临床化学检验\t血清总脂质测定" +
                ",临床化学检验\t血清α-淀粉样蛋白测定" +
                ",临床化学检验\t血清α-羟丁酸脱氢酶测定" +
                ",临床化学检验\t血清α1-抗胰蛋白酶测定" +
                ",临床化学检验\t血清α<sub>1</sub>-微球蛋白测定" +
                ",临床化学检验\t血清α<sub>2</sub>-巨球蛋白测定" +
                ",临床化学检验\t血清β-羟丁酸" +
                ",临床化学检验\t血清β-羟丁酸测定" +
                ",临床化学检验\t血清、尿液C肽测定" +
                ",临床化学检验\t血清、尿液、羊水雌三醇测定" +
                ",临床化学检验\t血清、血浆、24小时尿、红细胞镁测定" +
                ",临床化学检验\t血清（浆）IGF-Ⅰ及IGFBP-3测定" +
                ",临床化学检验\t血清（浆）甲状腺素测定" +
                ",临床化学检验\t血清（浆）甲状腺素结合球蛋白测定" +
                ",临床化学检验\t血清（浆）三碘甲状腺原氨酸测定" +
                ",临床化学检验\t血清（浆）游离甲状腺素测定" +
                ",临床化学检验\t血糖测定" +
                ",临床化学检验\t血氧饱和度测定" +
                ",临床化学检验\t血氧分压测定" +
                ",临床化学检验\t血氧含量测定" +
                ",临床化学检验\t血液半饱和氧分压" +
                ",临床化学检验\t血液半饱和氧分压测定" +
                ",临床化学检验\t血液电解质检查" +
                ",临床化学检验\t血液二氧化碳分压测定" +
                ",临床化学检验\t血液黏蛋白测定" +
                ",临床化学检验\t血液pH测定" +
                ",临床化学检验\t血液葡萄糖测定" +
                ",临床化学检验\t血液生化全项" +
                ",临床化学检验\t血液唾液酸测定" +
                ",临床化学检验\t血一氧化碳测定" +
                ",临床化学检验\t血脂常规检查" +
                ",临床化学检验\t洋地黄毒苷血药浓度测定" +
                ",临床化学检验\t氧化高密度脂蛋白测定" +
                ",临床化学检验\t羊水胆红素测定" +
                ",临床化学检验\t羊水结晶检查" +
                ",临床化学检验\t夜间睡眠皮质醇分泌试验" +
                ",临床化学检验\t叶酸筛查试验" +
                ",临床化学检验\t遗传筛查" +
                ",临床化学检验\t胰蛋白酶测定" +
                ",临床化学检验\t胰岛瘤相关抗原-2自身抗体检测" +
                ",临床化学检验\t胰岛素测定" +
                ",临床化学检验\t胰岛素低血糖刺激试验" +
                ",临床化学检验\t胰岛素释放试验" +
                ",临床化学检验\t胰岛素诱发低血糖试验" +
                ",临床化学检验\t胰岛素原测定" +
                ",临床化学检验\t胰岛素自身抗体测定" +
                ",临床化学检验\t胰岛素、C肽释放试验" +
                ",临床化学检验\t胰岛细胞自身抗体测定" +
                ",临床化学检验\t胰多肽测定" +
                ",临床化学检验\t胰高血糖素刺激试验" +
                ",临床化学检验\t乙琥胺血药浓度测定" +
                ",临床化学检验\t异亮氨酸测定" +
                ",临床化学检验\t阴道脱落细胞涂片检查" +
                ",临床化学检验\t影响氯吡格雷转化为活性代谢产物的基因多态性" +
                ",临床化学检验\t异柠檬酸脱氢酶测定" +
                ",临床化学检验\t阴离子间隙测定" +
                ",临床化学检验\t抑胃肽测定" +
                ",临床化学检验\t胰腺外分泌功能试验" +
                ",临床化学检验\t抑制素测定" +
                ",临床化学检验\t有机磷农药中毒的血胆碱酯酶活性检测" +
                ",临床化学检验\t游离甲状腺素指数" +
                ",临床化学检验\t游离羟脯氨酸测定" +
                ",临床化学检验\t游离三碘甲状腺原氨酸测定" +
                ",临床化学检验\t游离型雌三醇测定" +
                ",临床化学检验\t游离雄激素指数" +
                ",临床化学检验\t右旋木糖吸收试验" +
                ",临床化学检验\t孕妇血清、羊水甲胎蛋白测定" +
                ",临床化学检验\t孕酮测定" +
                ",临床化学检验\t孕酮撤退试验" +
                ",临床化学检验\t孕酮受体测定" +
                ",临床化学检验\t孕烷二醇测定" +
                ",临床化学检验\t载脂蛋白AⅠ测定" +
                ",临床化学检验\t载脂蛋白AⅡ测定" +
                ",临床化学检验\t载脂蛋白B测定" +
                ",临床化学检验\t载脂蛋白B／载脂蛋白AⅠ比值" +
                ",临床化学检验\t载脂蛋白CⅢ测定" +
                ",临床化学检验\t载脂蛋白E测定" +
                ",临床化学检验\t载脂蛋白H测定" +
                ",临床化学检验\t整合素1测定" +
                ",临床化学检验\t整合素2测定" +
                ",临床化学检验\t脂蛋白磷脂酶A<sub>2</sub>测定" +
                ",临床化学检验\t脂蛋白脂肪酶测定" +
                ",临床化学检验\t脂蛋白（a）测定" +
                ",临床化学检验\t脂肪酶测定" +
                ",临床化学检验\t脂肪吸收试验" +
                ",临床化学检验\t直接胆红素与总胆红素比值" +
                ",临床化学检验\t直接胆红素∶间接胆红素" +
                ",临床化学检验\t脂联素测定" +
                ",临床化学检验\t治疗药物监测" +
                ",临床化学检验\t肿瘤标记物检测" +
                ",临床化学检验\t中枢神经特异蛋白(S100β)测定" +
                ",临床化学检验\t中性粒细胞明胶酶相关脂质运载蛋白" +
                ",临床化学检验\t总雌激素测定" +
                ",临床化学检验\t总胆固醇／高密度脂蛋白胆固醇比值" +
                ",临床化学检验\t总胆红素测定" +
                ",临床化学检验\t总胆汁酸测定" +
                ",临床化学检验\t总甲状腺素和总三碘甲状腺原氨酸测定" +
                ",临床化学检验\t总皮质醇测定" +
                ",临床化学检验\t组氨酸测定" +
                ",临床化学检验\t组氨酸酶测定" +
                ",临床化学检验\t左旋多巴兴奋试验" +
                ",临床化学检验\t左旋多巴抑制试验" +
                ",临床化学检验\t组织蛋白酶测定" +
                ",临床免疫检验\t<i>Septin9</i>基因甲基化检测" +
                ",临床免疫检验\tβ-癌胚抗原测定" +
                ",临床免疫检验\tβ<sub>2</sub>-微球蛋白检测" +
                ",临床免疫检验\tγ-干扰素测定" +
                ",临床免疫检验\tⅠ型超敏反应的皮肤试验" +
                ",临床免疫检验\tⅣ型超敏反应的皮肤试验" +
                ",临床免疫检验\tANA谱检测" +
                ",临床免疫检验\tB细胞分化抗原检测" +
                ",临床免疫检验\tB细胞膜表面免疫球蛋白测定" +
                ",临床免疫检验\tB细胞亚群检测" +
                ",临床免疫检验\tDC检测" +
                ",临床免疫检验\tFCM检测NK细胞杀伤活性" +
                ",临床免疫检验\tHER2／neu蛋白测定" +
                ",临床免疫检验\tHLA-B27测定" +
                ",临床免疫检验\tHPV E6/E7" +
                ",临床免疫检验\tIgD检测" +
                ",临床免疫检验\tIgG、IgA和IgM检测" +
                ",临床免疫检验\tKi-67蛋白测定" +
                ",临床免疫检验\tM2型丙酮酸激酶测定" +
                ",临床免疫检验\tM蛋白检测" +
                ",临床免疫检验\tNK细胞的绝对计数及百分率检测" +
                ",临床免疫检验\tNK细胞检测" +
                ",临床免疫检验\tS-100蛋白测定" +
                ",临床免疫检验\tT细胞分化抗原测定" +
                ",临床免疫检验\tT细胞及其亚群（CD3／CD4／CD8）计数" +
                ",临床免疫检验\tT细胞亚群表型检测" +
                ",临床免疫检验\tT细胞转化试验" +
                ",临床免疫检验\t癌胚抗原测定" +
                ",临床免疫检验\t癌胚异铁蛋白测定" +
                ",临床免疫检验\t白三烯B<sub>4</sub>水平测定" +
                ",临床免疫检验\t白细胞介素-10检测" +
                ",临床免疫检验\t白细胞介素-11测定" +
                ",临床免疫检验\t白细胞介素-12测定" +
                ",临床免疫检验\t白细胞介素-13测定" +
                ",临床免疫检验\t白细胞介素-14测定" +
                ",临床免疫检验\t白细胞介素-15测定" +
                ",临床免疫检验\t白细胞介素-16测定" +
                ",临床免疫检验\t白细胞介素-17检测" +
                ",临床免疫检验\t白细胞介素-18测定" +
                ",临床免疫检验\t白细胞介素-19测定" +
                ",临床免疫检验\t白细胞介素-1测定" +
                ",临床免疫检验\t白细胞介素-20测定" +
                ",临床免疫检验\t白细胞介素-21测定" +
                ",临床免疫检验\t白细胞介素-22测定" +
                ",临床免疫检验\t白细胞介素-23测定" +
                ",临床免疫检验\t白细胞介素-24测定" +
                ",临床免疫检验\t白细胞介素-25测定" +
                ",临床免疫检验\t白细胞介素-26测定" +
                ",临床免疫检验\t白细胞介素-27测定" +
                ",临床免疫检验\t白细胞介素-2测定" +
                ",临床免疫检验\t白细胞介素-3测定" +
                ",临床免疫检验\t白细胞介素-4检测" +
                ",临床免疫检验\t白细胞介素-5测定" +
                ",临床免疫检验\t白细胞介素-6检测" +
                ",临床免疫检验\t白细胞介素-7测定" +
                ",临床免疫检验\t白细胞介素-8检测" +
                ",临床免疫检验\t白细胞介素-9测定" +
                ",临床免疫检验\t白细胞移动抑制试验" +
                ",临床免疫检验\t白血病免疫分型" +
                ",临床免疫检验\t膀胱肿瘤抗原测定" +
                ",临床免疫检验\t表皮生长因子测定" +
                ",临床免疫检验\t表皮生长因子受体1测定" +
                ",临床免疫检验\t表皮生长因子受体2测定" +
                ",临床免疫检验\t丙肝病毒核心抗原测定" +
                ",临床免疫检验\t丙肝病毒抗体测定" +
                ",临床免疫检验\t不规则抗体筛选和鉴定" +
                ",临床免疫检验\t补体C1q含量检测" +
                ",临床免疫检验\t补体C1酯酶抑制物测定" +
                ",临床免疫检验\t补体C2测定" +
                ",临床免疫检验\t补体C3a测定" +
                ",临床免疫检验\t补体C3b测定" +
                ",临床免疫检验\t补体C3c测定" +
                ",临床免疫检验\t补体C3d测定" +
                ",临床免疫检验\t补体C3、C4含量检测" +
                ",临床免疫检验\t补体C4a测定" +
                ",临床免疫检验\t补体C4测定" +
                ",临床免疫检验\t补体C5a测定" +
                ",临床免疫检验\t补体C5b～C9测定" +
                ",临床免疫检验\t补体C6测定" +
                ",临床免疫检验\t补体C7测定" +
                ",临床免疫检验\t补体C8测定" +
                ",临床免疫检验\t补体C9测定" +
                ",临床免疫检验\t补体经典途径溶血活性(CH50)检测" +
                ",临床免疫检验\t补体旁路途径溶血活性（AH50）检测" +
                ",临床免疫检验\t补体受体-1测定" +
                ",临床免疫检验\t补体受体-2测定" +
                ",临床免疫检验\t补体受体-3β测定" +
                ",临床免疫检验\t成纤维细胞生长因子测定" +
                ",临床免疫检验\t雌激素受体测定" +
                ",临床免疫检验\t促红细胞生成素测定" +
                ",临床免疫检验\t单纯疱疹病毒抗体测定" +
                ",临床免疫检验\t单个补体成分的测定" +
                ",临床免疫检验\t单克隆免疫球蛋白测定" +
                ",临床免疫检验\t丁型肝炎病毒抗体测定" +
                ",临床免疫检验\t端粒酶活性测定" +
                ",临床免疫检验\t多胺测定" +
                ",临床免疫检验\t肥大细胞脱颗粒试验" +
                ",临床免疫检验\t非梅毒螺旋体血清学试验" +
                ",临床免疫检验\t封闭抗体测定" +
                ",临床免疫检验\t钙卫蛋白测定" +
                ",临床免疫检验\t干扰素-γ检测" +
                ",临床免疫检验\t肝细胞生长因子测定" +
                ",临床免疫检验\t高敏C-反应蛋白测定" +
                ",临床免疫检验\t固相酶免疫测定" +
                ",临床免疫检验\t核基质蛋白22测定" +
                ",临床免疫检验\t核糖核酸酶测定" +
                ",临床免疫检验\t红斑狼疮细胞测定" +
                ",临床免疫检验\t呼吸道合胞病毒抗体" +
                ",临床免疫检验\t降钙素原（PCT）检测（金标法PCT‐Q）" +
                ",临床免疫检验\t降钙素原（PCT）检测（仪器法）" +
                ",临床免疫检验\t甲胎蛋白L3测定" +
                ",临床免疫检验\t甲胎蛋白检测" +
                ",临床免疫检验\t甲胎蛋白异质体检测" +
                ",临床免疫检验\t甲型肝炎抗原测定" +
                ",临床免疫检验\t结核病血清学诊断" +
                ",临床免疫检验\t结核菌涂片检查" +
                ",临床免疫检验\t结合前列腺特异性抗原检测" +
                ",临床免疫检验\t集落刺激因子测定" +
                ",临床免疫检验\t基质金属蛋白酶9测定" +
                ",临床免疫检验\t巨噬细胞移动抑制试验" +
                ",临床免疫检验\t巨细胞病毒pp65抗原测定" +
                ",临床免疫检验\t抗C1q抗体检测" +
                ",临床免疫检验\t抗GP210抗体检测" +
                ",临床免疫检验\t抗Jo-1抗体测定" +
                ",临床免疫检验\t抗Ku抗体测定" +
                ",临床免疫检验\t抗M3毒蕈乙酰胆碱受体抗体测定" +
                ",临床免疫检验\t抗PM-Scl抗体测定" +
                ",临床免疫检验\t抗SS-A／Ro抗体测定" +
                ",临床免疫检验\t抗SS-B抗体测定" +
                ",临床免疫检验\t抗Sa抗体测定" +
                ",临床免疫检验\t抗Scl-70抗体测定" +
                ",临床免疫检验\t抗Sm抗体测定" +
                ",临床免疫检验\t抗U1RNP抗体测定" +
                ",临床免疫检验\t抗促甲状腺素受体抗体检测" +
                ",临床免疫检验\t抗单链DNA抗体" +
                ",临床免疫检验\t抗肝肾微粒体抗体测定" +
                ",临床免疫检验\t抗肝细胞膜抗体测定" +
                ",临床免疫检验\t抗肝细胞溶质抗原Ⅰ型抗体检测" +
                ",临床免疫检验\t抗谷氨酸脱羧酶抗体测定" +
                ",临床免疫检验\t抗骨骼肌抗体测定" +
                ",临床免疫检验\t抗核RNP抗体测定" +
                ",临床免疫检验\t抗核抗体检测" +
                ",临床免疫检验\t抗核膜抗体测定" +
                ",临床免疫检验\t抗核仁抗体" +
                ",临床免疫检验\t抗核糖体P蛋白抗体测定" +
                ",临床免疫检验\t抗核小体抗体检测" +
                ",临床免疫检验\t抗环瓜氨酸肽抗体检测" +
                ",临床免疫检验\t抗角蛋白抗体测定" +
                ",临床免疫检验\t抗甲状腺球蛋白与抗甲状腺过氧化物酶抗体检测" +
                ",临床免疫检验\t抗精子抗体检测" +
                ",临床免疫检验\t抗肌炎-硬皮病抗体测定" +
                ",临床免疫检验\t抗可溶性肝抗原/肝胰抗原-抗体检测" +
                ",临床免疫检验\t抗可溶性肝抗原抗体测定" +
                ",临床免疫检验\t抗可溶性酸性磷酸化核蛋白SP100抗体测定" +
                ",临床免疫检验\t抗可提取性核抗原测定" +
                ",临床免疫检验\t抗链球菌溶血素“O”测定" +
                ",临床免疫检验\t抗淋巴细胞抗体" +
                ",临床免疫检验\t抗卵巢抗体检测" +
                ",临床免疫检验\t抗内皮细胞抗体测定" +
                ",临床免疫检验\t抗内因子抗体检测" +
                ",临床免疫检验\t抗酿酒酵母抗体检测" +
                ",临床免疫检验\t抗平滑肌抗体检测" +
                ",临床免疫检验\t抗去唾液酸糖蛋白受体抗体检测" +
                ",临床免疫检验\t抗肾上腺皮质抗体测定" +
                ",临床免疫检验\t抗肾小球基底膜抗体检测" +
                ",临床免疫检验\t抗双链DNA抗体检测" +
                ",临床免疫检验\t抗体依赖性细胞介导的细胞毒测定" +
                ",临床免疫检验\t抗透明带抗体检测" +
                ",临床免疫检验\t抗胃壁细胞抗体检测" +
                ",临床免疫检验\t抗线粒体2型抗体测定" +
                ",临床免疫检验\t抗线粒体抗体检测" +
                ",临床免疫检验\t抗小肠杯状细胞抗体检测" +
                ",临床免疫检验\t抗胰岛素抗体检测" +
                ",临床免疫检验\t抗胰岛细胞抗体检测" +
                ",临床免疫检验\t抗乙酰胆碱受体抗体检测" +
                ",临床免疫检验\t抗胰腺腺泡抗体检测" +
                ",临床免疫检验\t抗增殖细胞核抗原抗体测定" +
                ",临床免疫检验\t抗着丝点抗体测定" +
                ",临床免疫检验\t抗中性粒细胞胞浆抗体测定" +
                ",临床免疫检验\t抗中性粒细胞胞质抗体检测——特异性ANCA" +
                ",临床免疫检验\t抗中性粒细胞胞质抗体检测——总ANCA" +
                ",临床免疫检验\t抗子宫内膜抗体测定" +
                ",临床免疫检验\t抗组蛋白抗体检测" +
                ",临床免疫检验\t抗α-胞衬蛋白抗体检测" +
                ",临床免疫检验\t抗β2-GP1抗体检测" +
                ",临床免疫检验\t抗“O”检测" +
                ",临床免疫检验\t可溶性白细胞介素-2受体检测" +
                ",临床免疫检验\t可溶性间皮素相关肽测定" +
                ",临床免疫检验\t可溶性转铁蛋白受体测定" +
                ",临床免疫检验\t快速红斑狼疮因子试验" +
                ",临床免疫检验\t快速尿素酶试验" +
                ",临床免疫检验\t类风湿因子检测" +
                ",临床免疫检验\t类黏蛋白肿瘤相关抗原测定" +
                ",临床免疫检验\t冷球蛋白检测" +
                ",临床免疫检验\t淋巴细胞毒试验" +
                ",临床免疫检验\t淋巴细胞功能亚群检测" +
                ",临床免疫检验\t淋巴细胞免疫表型检测" +
                ",临床免疫检验\t鳞状上皮细胞癌抗原检测" +
                ",临床免疫检验\t鳞状细胞癌抗原测定" +
                ",临床免疫检验\t螺内酯（安体舒通）试验" +
                ",临床免疫检验\t梅毒螺旋体血清学试验" +
                ",临床免疫检验\t酶联免疫吸附试验" +
                ",临床免疫检验\t免疫球蛋白A亚类测定" +
                ",临床免疫检验\t免疫球蛋白G4测定" +
                ",临床免疫检验\t免疫球蛋白G合成率" +
                ",临床免疫检验\t免疫球蛋白G亚类测定" +
                ",临床免疫检验\t免疫球蛋白测定" +
                ",临床免疫检验\t免疫球蛋白电泳" +
                ",临床免疫检验\t免疫球蛋白检测" +
                ",临床免疫检验\t末端脱氧核苷酸转移酶测定" +
                ",临床免疫检验\t脑脊液IgG鞘内合成率/24小时检测" +
                ",临床免疫检验\t内毒素检测" +
                ",临床免疫检验\t尿液κ/λ比" +
                ",临床免疫检验\t旁路途径的溶血活性测定" +
                ",临床免疫检验\t葡萄糖-6-磷酸异构酶测定" +
                ",临床免疫检验\t前列腺癌的血清肿瘤标记物" +
                ",临床免疫检验\t前列腺酸性磷酸酶测定" +
                ",临床免疫检验\t前列腺特异性膜抗原测定" +
                ",临床免疫检验\t群体反应性抗体测定" +
                ",临床免疫检验\t人表皮生长因子受体-2蛋白胞外区检测" +
                ",临床免疫检验\t人附睾蛋白4测定" +
                ",临床免疫检验\t人类白细胞抗原系统检测" +
                ",临床免疫检验\t人型支原体核酸检测" +
                ",临床免疫检验\t伤寒血清学检测" +
                ",临床免疫检验\t上皮钙黏附蛋白测定" +
                ",临床免疫检验\t生殖系统肿瘤标志物" +
                ",临床免疫检验\t生殖支原体核酸检测" +
                ",临床免疫检验\t神经生长因子测定" +
                ",临床免疫检验\t神经元特异烯醇化酶检测" +
                ",临床免疫检验\t嗜铬粒蛋白A测定" +
                ",临床免疫检验\t食物特异性IgG测定" +
                ",临床免疫检验\t糖蛋白类肿瘤标志物检测" +
                ",临床免疫检验\t糖类抗原125检测" +
                ",临床免疫检验\t糖类抗原19-5测定" +
                ",临床免疫检验\t糖类抗原19-9检测" +
                ",临床免疫检验\t糖类抗原242检测" +
                ",临床免疫检验\t糖类抗原27-29测定" +
                ",临床免疫检验\t糖类抗原50测定" +
                ",临床免疫检验\t糖类抗原549测定" +
                ",临床免疫检验\t糖类抗原72-4检测" +
                ",临床免疫检验\t糖链抗原15-3测定" +
                ",临床免疫检验\t特异性IgE检测" +
                ",临床免疫检验\t天疱疮抗体测定" +
                ",临床免疫检验\t体液免疫检测法" +
                ",临床免疫检验\t脱-γ-羧基凝血酶原测定" +
                ",临床免疫检验\t唾液酰基转移酶测定" +
                ",临床免疫检验\t外周血T淋巴细胞及其亚类监测" +
                ",临床免疫检验\t胃蛋白酶原Ⅱ测定" +
                ",临床免疫检验\t胃泌素释放肽前体检测" +
                ",临床免疫检验\t硝基四唑氮蓝试验" +
                ",临床免疫检验\t细胞角蛋白19片段21-1测定" +
                ",临床免疫检验\t细胞角蛋白19片段测定" +
                ",临床免疫检验\t细胞角蛋白19片段检测" +
                ",临床免疫检验\t细胞因子与细胞黏附因子的测定" +
                ",临床免疫检验\t性病研究实验室试验" +
                ",临床免疫检验\t性传播疾病免疫检测" +
                ",临床免疫检验\t胸苷激酶测定" +
                ",临床免疫检验\t雄激素受体测定" +
                ",临床免疫检验\t选择素族黏附分子测定" +
                ",临床免疫检验\t血管内皮生长因子测定" +
                ",临床免疫检验\t血清C反应蛋白测定" +
                ",临床免疫检验\t血清IgG亚类检测" +
                ",临床免疫检验\t血清淀粉样蛋白A检测" +
                ",临床免疫检验\t血清免疫球蛋白A测定" +
                ",临床免疫检验\t血清免疫球蛋白D测定" +
                ",临床免疫检验\t血清免疫球蛋白E测定" +
                ",临床免疫检验\t血清免疫球蛋白G测定" +
                ",临床免疫检验\t血清κ/λ比" +
                ",临床免疫检验\t血清、脑脊液免疫球蛋白A测定" +
                ",临床免疫检验\t血清、脑脊液免疫球蛋白M测定" +
                ",临床免疫检验\t血清、随机尿、脑脊液新蝶呤测定" +
                ",临床免疫检验\t循环核酸测定" +
                ",临床免疫检验\t循环免疫复合物检测" +
                ",临床免疫检验\t循环肿瘤细胞测定" +
                ",临床免疫检验\t液基薄层细胞制作技术" +
                ",临床免疫检验\t胰癌胚抗原测定" +
                ",临床免疫检验\t胰岛素样生长因子测定" +
                ",临床免疫检验\t胰岛素样生长因子结合蛋白-3测定" +
                ",临床免疫检验\t乙肝五项" +
                ",临床免疫检验\t乙醛脱氢酶2多态性的检测" +
                ",临床免疫检验\t胰腺癌相关抗原-2测定" +
                ",临床免疫检验\t乙酰胆碱酯酶抗体测定" +
                ",临床免疫检验\t胰腺肿瘤标记物" +
                ",临床免疫检验\t乙型肝炎病毒前S<sub>1</sub>抗原及前S<sub>1</sub>抗体" +
                ",临床免疫检验\t游离前列腺特异性抗原检测" +
                ",临床免疫检验\t游离轻链检测" +
                ",临床免疫检验\t幽门螺旋杆菌抗体检查" +
                ",临床免疫检验\t孕激素受体测定" +
                ",临床免疫检验\t孕酮受体测定" +
                ",临床免疫检验\t脂质相关唾液酸测定" +
                ",临床免疫检验\t肿瘤坏死因子-α检测" +
                ",临床免疫检验\t肿瘤特异性生长因子" +
                ",临床免疫检验\t肿瘤相关抗原TA90测定" +
                ",临床免疫检验\t中性粒细胞黏附功能检测" +
                ",临床免疫检验\t中性粒细胞趋化功能检测" +
                ",临床免疫检验\t中性粒细胞吞噬与杀菌功能检测" +
                ",临床免疫检验\t转化生长因子-β检测" +
                ",临床免疫检验\t主要组织相容性抗原" +
                ",临床免疫检验\t自然杀伤细胞活性检测" +
                ",临床免疫检验\t自身抗体检测" +
                ",临床免疫检验\t总IgE检测" +
                ",临床免疫检验\t总补体溶血活性测定" +
                ",临床免疫检验\t总前列腺特异性抗原检测" +
                ",临床免疫检验\t总溶血补体活性测定" +
                ",临床免疫检验\t组织多肽抗原测定" +
                ",临床微生物与寄生虫检验\t2-酮基葡萄糖酸盐试验" +
                ",临床微生物与寄生虫检验\t3-酮基乳糖酸盐试验" +
                ",临床微生物与寄生虫检验\tβ-D-葡聚糖试验" +
                ",临床微生物与寄生虫检验\tβ-半乳糖苷酶试验" +
                ",临床微生物与寄生虫检验\t（1→3）‐β‐D‐葡聚糖检测（G－试验）" +
                ",临床微生物与寄生虫检验\tATB FUNGUS 3酵母菌药敏试验" +
                ",临床微生物与寄生虫检验\tBD Phoenix‐100细菌鉴定／药敏" +
                ",临床微生物与寄生虫检验\tCAMP试验" +
                ",临床微生物与寄生虫检验\tCAMP抑制试验" +
                ",临床微生物与寄生虫检验\tDNA酶试验" +
                ",临床微生物与寄生虫检验\tEB病毒抗体测定" +
                ",临床微生物与寄生虫检验\tEpstein-Barr病毒检验" +
                ",临床微生物与寄生虫检验\tHIV-1核酸定量检测" +
                ",临床微生物与寄生虫检验\tHIV-1核酸定性检测" +
                ",临床微生物与寄生虫检验\tHIV抗体确证试验" +
                ",临床微生物与寄生虫检验\tHIV抗体筛查试验" +
                ",临床微生物与寄生虫检验\tHPV病理学检查" +
                ",临床微生物与寄生虫检验\tHPV核酸检测" +
                ",临床微生物与寄生虫检验\tKoga 培养法" +
                ",临床微生物与寄生虫检验\tL型沙眼衣原体核酸检测" +
                ",临床微生物与寄生虫检验\tL型沙眼衣原体基因测序分型法" +
                ",临床微生物与寄生虫检验\tO/129抑菌试验" +
                ",临床微生物与寄生虫检验\tOptochin敏感试验" +
                ",临床微生物与寄生虫检验\tSARS冠状病毒检验" +
                ",临床微生物与寄生虫检验\tTORCH检测" +
                ",临床微生物与寄生虫检验\tVP试验" +
                ",临床微生物与寄生虫检验\tXpert MTB/RIF标准操作规程" +
                ",临床微生物与寄生虫检验\t埃可病毒抗体测定" +
                ",临床微生物与寄生虫检验\t埃立克体属检验" +
                ",临床微生物与寄生虫检验\t埃希菌属检验" +
                ",临床微生物与寄生虫检验\t氨基酸脱氨酶试验" +
                ",临床微生物与寄生虫检验\t氨基酸脱羧酶试验" +
                ",临床微生物与寄生虫检验\t氨基糖苷类高水平耐药（HL AR）肠球菌检测" +
                ",临床微生物与寄生虫检验\t暗色真菌检验" +
                ",临床微生物与寄生虫检验\t斑点酶联免疫吸附试验" +
                ",临床微生物与寄生虫检验\t棒杆菌属检验" +
                ",临床微生物与寄生虫检验\t半乳甘露聚糖抗原试验" +
                ",临床微生物与寄生虫检验\t饱和盐水浮聚法" +
                ",临床微生物与寄生虫检验\t鲍特菌属检验" +
                ",临床微生物与寄生虫检验\t巴斯德菌属检验" +
                ",临床微生物与寄生虫检验\t巴通体属检验");

        jianyan.append(",临床微生物与寄生虫检验\t病毒常规检测" +
                ",临床微生物与寄生虫检验\t病毒蛋白抗原检测" +
                ",临床微生物与寄生虫检验\t病毒感染的实验室检查" +
                ",临床微生物与寄生虫检验\t病毒培养与鉴定" +
                ",临床微生物与寄生虫检验\t丙二酸盐利用试验" +
                ",临床微生物与寄生虫检验\t丙型肝炎病毒RNA定性检测" +
                ",临床微生物与寄生虫检验\t丙型肝炎病毒抗体检测" +
                ",临床微生物与寄生虫检验\t并殖吸虫病检验" +
                ",临床微生物与寄生虫检验\t伯克霍尔德菌属检验" +
                ",临床微生物与寄生虫检验\t伯氏疏螺旋体抗体测定" +
                ",临床微生物与寄生虫检验\t不动杆菌属检验" +
                ",临床微生物与寄生虫检验\t布鲁菌属检验" +
                ",临床微生物与寄生虫检验\t布氏杆菌抗体凝集试验" +
                ",临床微生物与寄生虫检验\t肠道病毒培养和鉴定" +
                ",临床微生物与寄生虫检验\t肠道病毒通用型（EV）核酸检测" +
                ",临床微生物与寄生虫检验\t肠杆菌科菌碳青霉烯酶检测" +
                ",临床微生物与寄生虫检验\t肠杆菌属、枸橼酸杆菌属、沙雷菌属、变形杆菌属检验" +
                ",临床微生物与寄生虫检验\t肠球菌属检验" +
                ",临床微生物与寄生虫检验\t肠绦虫相关检验" +
                ",临床微生物与寄生虫检验\t常用的抗菌药物敏感试验" +
                ",临床微生物与寄生虫检验\t产碱杆菌属检验" +
                ",临床微生物与寄生虫检验\t产青霉素酶的淋球菌的检测" +
                ",临床微生物与寄生虫检验\t穿刺液标本的微生物检验" +
                ",临床微生物与寄生虫检验\t传染病四项" +
                ",临床微生物与寄生虫检验\t触酶试验" +
                ",临床微生物与寄生虫检验\t醋酸盐利用试验" +
                ",临床微生物与寄生虫检验\t大环内酯‐林可霉素‐链阳霉素B（MLS<sub>B</sub>）耐药检测" +
                ",临床微生物与寄生虫检验\t单纯疱疹病毒1型和2型抗原检测" +
                ",临床微生物与寄生虫检验\t单纯疱疹病毒检验" +
                ",临床微生物与寄生虫检验\t单纯疱疹病毒Ⅱ型核酸检测" +
                ",临床微生物与寄生虫检验\t丹毒丝菌属检验" +
                ",临床微生物与寄生虫检验\t胆汁溶菌试验" +
                ",临床微生物与寄生虫检验\t导管的微生物检验" +
                ",临床微生物与寄生虫检验\t淀粉水解试验" +
                ",临床微生物与寄生虫检验\t定量透明法检验寄生虫" +
                ",临床微生物与寄生虫检验\t丁型肝炎抗原" +
                ",临床微生物与寄生虫检验\t动力试验" +
                ",临床微生物与寄生虫检验\t对流免疫电泳试验" +
                ",临床微生物与寄生虫检验\t杜克雷嗜血杆菌的分离培养" +
                ",临床微生物与寄生虫检验\t杜克雷嗜血杆菌的鉴定" +
                ",临床微生物与寄生虫检验\t杜克雷嗜血杆菌显微镜检查" +
                ",临床微生物与寄生虫检验\t放线菌属检验" +
                ",临床微生物与寄生虫检验\t肺孢子虫检验" +
                ",临床微生物与寄生虫检验\t肥达反应" +
                ",临床微生物与寄生虫检验\t非结核分枝杆菌检验" +
                ",临床微生物与寄生虫检验\t肺螨虫检验" +
                ",临床微生物与寄生虫检验\t肺炎链球菌检验" +
                ",临床微生物与寄生虫检验\t肺炎支原体分离培养及鉴定" +
                ",临床微生物与寄生虫检验\t肺炎支原体抗体测定" +
                ",临床微生物与寄生虫检验\t肺炎支原体商品化检测系统" +
                ",临床微生物与寄生虫检验\t肺炎支原体血清学试验" +
                ",临床微生物与寄生虫检验\t粪便标本的微生物检验" +
                ",临床微生物与寄生虫检验\t粪便标本细菌检验" +
                ",临床微生物与寄生虫检验\t粪便难辨梭状芽胞杆菌检验" +
                ",临床微生物与寄生虫检验\t粪便直接涂片法" +
                ",临床微生物与寄生虫检验\t粪类圆线虫检验" +
                ",临床微生物与寄生虫检验\t分枝杆菌金胺O染色" +
                ",临床微生物与寄生虫检验\t分枝杆菌罗氏培养" +
                ",临床微生物与寄生虫检验\t分枝杆菌萋尼染色" +
                ",临床微生物与寄生虫检验\t分枝杆菌液体培养" +
                ",临床微生物与寄生虫检验\t弗朗西斯菌属检验" +
                ",临床微生物与寄生虫检验\t副流感病毒抗体测定" +
                ",临床微生物与寄生虫检验\t改良加藤厚涂片法" +
                ",临床微生物与寄生虫检验\t肛门拭子法与肛周蛲虫检查法" +
                ",临床微生物与寄生虫检验\t杆菌肽敏感试验" +
                ",临床微生物与寄生虫检验\t肝炎病毒检验" +
                ",临床微生物与寄生虫检验\t甘油复红试验" +
                ",临床微生物与寄生虫检验\t高危型人乳头瘤病毒（HPV）基因分型定量检测" +
                ",临床微生物与寄生虫检验\t革兰染色" +
                ",临床微生物与寄生虫检验\t革兰阳性无芽胞厌氧杆菌和放线菌属检验" +
                ",临床微生物与寄生虫检验\t革兰阴性杆菌葡萄糖代谢类型鉴别试验" +
                ",临床微生物与寄生虫检验\t汞碘醛离心沉淀法" +
                ",临床微生物与寄生虫检验\t弓形虫IgG检测" +
                ",临床微生物与寄生虫检验\t弓形虫检验" +
                ",临床微生物与寄生虫检验\t弓形虫筛查" +
                ",临床微生物与寄生虫检验\t弓形体抗体测定" +
                ",临床微生物与寄生虫检验\t钩虫相关检验" +
                ",临床微生物与寄生虫检验\t钩端螺旋体乳胶凝集试验" +
                ",临床微生物与寄生虫检验\t钩蚴培养法" +
                ",临床微生物与寄生虫检验\t枸橼酸盐利用试验" +
                ",临床微生物与寄生虫检验\t广州管圆线虫检验" +
                ",临床微生物与寄生虫检验\t关节液培养" +
                ",临床微生物与寄生虫检验\t灌洗液培养" +
                ",临床微生物与寄生虫检验\t汉坦病毒检验" +
                ",临床微生物与寄生虫检验\t红酵母属检验" +
                ",临床微生物与寄生虫检验\t环蚴沉淀试验" +
                ",临床微生物与寄生虫检验\t华支睾吸虫检验" +
                ",临床微生物与寄生虫检验\t蛔虫检验" +
                ",临床微生物与寄生虫检验\t弧菌属检验" +
                ",临床微生物与寄生虫检验\t霍乱红试验" +
                ",临床微生物与寄生虫检验\t霍乱弧菌检验" +
                ",临床微生物与寄生虫检验\t呼吸道标本细菌检验" +
                ",临床微生物与寄生虫检验\t呼吸道病毒培养和鉴定" +
                ",临床微生物与寄生虫检验\t呼吸道合胞病毒（RSV）核酸检测" +
                ",临床微生物与寄生虫检验\t假单胞菌属检验" +
                ",临床微生物与寄生虫检验\t贾第虫检验" +
                ",临床微生物与寄生虫检验\t甲基红试验" +
                ",临床微生物与寄生虫检验\t荚膜肿胀试验" +
                ",临床微生物与寄生虫检验\t浆膜腔积液寄生虫检查" +
                ",临床微生物与寄生虫检验\t姜片虫相关检验" +
                ",临床微生物与寄生虫检验\t间接法ELISA检测TORCH特异性IgG抗体" +
                ",临床微生物与寄生虫检验\t间接血凝试验" +
                ",临床微生物与寄生虫检验\t甲型H1N1流感病毒核酸检测" +
                ",临床微生物与寄生虫检验\t甲型肝炎病毒IgM抗体（HAV IgM）测定" +
                ",临床微生物与寄生虫检验\t甲型流感病毒抗原检测（胶体金法）" +
                ",临床微生物与寄生虫检验\t甲氧西林耐药葡萄球菌（MRS）检测" +
                ",临床微生物与寄生虫检验\t结核分枝杆菌RNA检测" +
                ",临床微生物与寄生虫检验\t结核分枝杆菌检验" +
                ",临床微生物与寄生虫检验\t结核分枝杆菌抗体检测" +
                ",临床微生物与寄生虫检验\t接合菌检验" +
                ",临床微生物与寄生虫检验\t疥螨检查" +
                ",临床微生物与寄生虫检验\t解脲脲原体RNA检测" +
                ",临床微生物与寄生虫检验\t解脲脲原体分离培养及鉴定" +
                ",临床微生物与寄生虫检验\t解脲支原体核酸检测" +
                ",临床微生物与寄生虫检验\t精氨酸双水解酶（ADH）试验" +
                ",临床微生物与寄生虫检验\t棘球蚴检验" +
                ",临床微生物与寄生虫检验\t寄生虫标本的采集、保存与鉴定" +
                ",临床微生物与寄生虫检验\t寄生虫肠黏膜活检" +
                ",临床微生物与寄生虫检验\t寄生虫单向琼脂扩散试验" +
                ",临床微生物与寄生虫检验\t寄生虫碘液染色法" +
                ",临床微生物与寄生虫检验\t寄生虫肺组织活检" +
                ",临床微生物与寄生虫检验\t寄生虫骨髓穿刺涂片活检" +
                ",临床微生物与寄生虫检验\t寄生虫间接血凝试验" +
                ",临床微生物与寄生虫检验\t寄生虫淋巴结穿刺涂片活检" +
                ",临床微生物与寄生虫检验\t寄生虫皮下及肌肉活组织检查" +
                ",临床微生物与寄生虫检验\t寄生虫乳胶凝集试验" +
                ",临床微生物与寄生虫检验\t寄生虫双向琼脂扩散试验" +
                ",临床微生物与寄生虫检验\t寄生虫痰液检查法" +
                ",临床微生物与寄生虫检验\t寄生虫血膜染色法" +
                ",临床微生物与寄生虫检验\t军团菌属检验" +
                ",临床微生物与寄生虫检验\t军团菌血清特异性抗体检测" +
                ",临床微生物与寄生虫检验\t巨细胞病毒检验" +
                ",临床微生物与寄生虫检验\t巨细胞病毒抗原血症（CMVpp65）检测" +
                ",临床微生物与寄生虫检验\t巨细胞病毒离心小瓶培养法" +
                ",临床微生物与寄生虫检验\t巨细胞病毒免疫检测" +
                ",临床微生物与寄生虫检验\t抗EB病毒衣壳抗原IgA测定" +
                ",临床微生物与寄生虫检验\t抗EB病毒衣壳抗原IgG抗体测定" +
                ",临床微生物与寄生虫检验\t抗EB病毒衣壳抗原IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗EB病毒早期抗原IgA抗体测定" +
                ",临床微生物与寄生虫检验\t抗HSV1型‐IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗HSV2型‐IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗单纯疱疹病毒1型IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗单纯疱疹病毒2型IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗风疹病毒IgG抗体测定" +
                ",临床微生物与寄生虫检验\t抗风疹病毒IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗弓形虫IgG抗体测定" +
                ",临床微生物与寄生虫检验\t抗弓形虫IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗巨细胞病毒IgG抗体测定" +
                ",临床微生物与寄生虫检验\t抗巨细胞病毒IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗麻疹病毒IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗沙眼衣原体、肺炎衣原体和鹦鹉热衣原体IgG和IgM抗体测定" +
                ",临床微生物与寄生虫检验\t抗嗜肺军团菌抗体IgG检测" +
                ",临床微生物与寄生虫检验\t抗嗜肺军团菌抗体IgM检测" +
                ",临床微生物与寄生虫检验\t抗体捕获法ELISA检测TORCH特异性IgM抗体" +
                ",临床微生物与寄生虫检验\t卡他莫拉菌检验" +
                ",临床微生物与寄生虫检验\t克雷伯菌属和拉乌尔菌属检验" +
                ",临床微生物与寄生虫检验\t柯萨奇病毒A16型（CA16）核酸检测" +
                ",临床微生物与寄生虫检验\t柯萨奇病毒中和试验" +
                ",临床微生物与寄生虫检验\t克氏双糖铁琼脂试验" +
                ",临床微生物与寄生虫检验\t快速血浆反应素环状卡片试验" +
                ",临床微生物与寄生虫检验\t快速β内酰胺酶试验" +
                ",临床微生物与寄生虫检验\t蓝氏贾第鞭毛虫培养" +
                ",临床微生物与寄生虫检验\t镰刀菌检验" +
                ",临床微生物与寄生虫检验\t链球菌属检验" +
                ",临床微生物与寄生虫检验\t立克次体属检验" +
                ",临床微生物与寄生虫检验\t淋病奈瑟菌检验" +
                ",临床微生物与寄生虫检验\t淋病奈瑟球菌显微镜检查" +
                ",临床微生物与寄生虫检验\t淋病双球菌镜检" +
                ",临床微生物与寄生虫检验\t淋病双球菌培养" +
                ",临床微生物与寄生虫检验\t临床常用的细菌抗原检测试验" +
                ",临床微生物与寄生虫检验\t淋球菌RNA检测" +
                ",临床微生物与寄生虫检验\t淋球菌的初步鉴定：氧化酶试验" +
                ",临床微生物与寄生虫检验\t淋球菌的分离培养" +
                ",临床微生物与寄生虫检验\t淋球菌的确认鉴定：糖发酵试验" +
                ",临床微生物与寄生虫检验\t淋球菌定性药敏检测（纸片扩散法）" +
                ",临床微生物与寄生虫检验\t淋球菌免疫学检查" +
                ",临床微生物与寄生虫检验\t利什曼原虫检验" +
                ",临床微生物与寄生虫检验\t利什曼原虫培养" +
                ",临床微生物与寄生虫检验\t李斯特菌属检验" +
                ",临床微生物与寄生虫检验\t六胺银染色" +
                ",临床微生物与寄生虫检验\t流感嗜血杆菌检验" +
                ",临床微生物与寄生虫检验\t硫化氢（H<sub>2</sub>S）试验" +
                ",临床微生物与寄生虫检验\t硫酸锌离心浮聚法" +
                ",临床微生物与寄生虫检验\t流行性出血热病毒血清学检查" +
                ",临床微生物与寄生虫检验\t流行性感冒病毒检验" +
                ",临床微生物与寄生虫检验\t留置性导管标本细菌检验" +
                ",临床微生物与寄生虫检验\t卵磷脂酶试验和聂格尔试验" +
                ",临床微生物与寄生虫检验\t轮状病毒离心小瓶培养法" +
                ",临床微生物与寄生虫检验\t轮状病毒试管培养法" +
                ",临床微生物与寄生虫检验\t轮状病毒实时定量PCR检测" +
                ",临床微生物与寄生虫检验\t螺杆菌属检验" +
                ",临床微生物与寄生虫检验\t螺旋体检验" +
                ",临床微生物与寄生虫检验\t螺旋体形态学检查" +
                ",临床微生物与寄生虫检验\t麻风杆菌皮肤涂片检查" +
                ",临床微生物与寄生虫检验\t马尿酸钠水解试验" +
                ",临床微生物与寄生虫检验\t毛孢子菌属检验" +
                ",临床微生物与寄生虫检验\t毛蚴孵化法" +
                ",临床微生物与寄生虫检验\t梅毒病原学（梅毒螺旋体）检测" +
                ",临床微生物与寄生虫检验\t梅毒螺旋体抗体检测" +
                ",临床微生物与寄生虫检验\t梅毒螺旋体明胶颗粒凝集试验" +
                ",临床微生物与寄生虫检验\t酶联免疫吸附试验" +
                ",临床微生物与寄生虫检验\t免疫荧光标记技术" +
                ",临床微生物与寄生虫检验\t明胶液化试验" +
                ",临床微生物与寄生虫检验\t泌尿生殖道分泌物培养" +
                ",临床微生物与寄生虫检验\t莫拉菌属检验" +
                ",临床微生物与寄生虫检验\t墨汁染色" +
                ",临床微生物与寄生虫检验\t难辨梭菌毒素A＋B检测" +
                ",临床微生物与寄生虫检验\t囊尾蚴检验" +
                ",临床微生物与寄生虫检验\t蛲虫检验" +
                ",临床微生物与寄生虫检验\t脑脊液标本细菌检验" +
                ",临床微生物与寄生虫检验\t脑脊液的微生物检验" +
                ",临床微生物与寄生虫检验\t脑脊液中寄生虫检查" +
                ",临床微生物与寄生虫检验\t脑膜炎奈瑟菌检验" +
                ",临床微生物与寄生虫检验\t黏丝试验" +
                ",临床微生物与寄生虫检验\t念珠菌属检验" +
                ",临床微生物与寄生虫检验\t念珠菌药物敏感性测定" +
                ",临床微生物与寄生虫检验\t尿素酶试验" +
                ",临床微生物与寄生虫检验\t尿液标本的微生物检验" +
                ",临床微生物与寄生虫检验\t尿液标本细菌检验" +
                ",临床微生物与寄生虫检验\t尿液真菌培养" +
                ",临床微生物与寄生虫检验\t尿液中寄生虫检查" +
                ",临床微生物与寄生虫检验\t拟杆菌属检验" +
                ",临床微生物与寄生虫检验\t尼龙袋集卵法" +
                ",临床微生物与寄生虫检验\t凝固酶试验" +
                ",临床微生物与寄生虫检验\t脓液及创伤感染分泌物细菌学检查" +
                ",临床微生物与寄生虫检验\t疟原虫检验" +
                ",临床微生物与寄生虫检验\t疟原虫培养" +
                ",临床微生物与寄生虫检验\t诺卡菌检验" +
                ",临床微生物与寄生虫检验\t诺卡菌属检验" +
                ",临床微生物与寄生虫检验\t疱疹病毒培养和鉴定" +
                ",临床微生物与寄生虫检验\t皮肤癣菌检验" +
                ",临床微生物与寄生虫检验\t葡萄球菌属检验" +
                ",临床微生物与寄生虫检验\t葡萄糖代谢类型鉴别试验" +
                ",临床微生物与寄生虫检验\t鞘膜积液寄生虫检查" +
                ",临床微生物与寄生虫检验\t气单胞菌科检验" +
                ",临床微生物与寄生虫检验\t氰化钾抑制试验" +
                ",临床微生物与寄生虫检验\t青霉素耐药肺炎链球菌（PRSP）检测" +
                ",临床微生物与寄生虫检验\t氢氧化钾拉丝试验" +
                ",临床微生物与寄生虫检验\t琼脂稀释法淋球菌药敏检测" +
                ",临床微生物与寄生虫检验\t其他肠杆菌科细菌检验" +
                ",临床微生物与寄生虫检验\t七叶苷水解试验" +
                ",临床微生物与寄生虫检验\t醛醚沉淀法" +
                ",临床微生物与寄生虫检验\t曲霉菌属检验" +
                ",临床微生物与寄生虫检验\t染料抑制试验" +
                ",临床微生物与寄生虫检验\t人类轮状病毒检验" +
                ",临床微生物与寄生虫检验\t人类免疫缺陷病毒抗体（HIV1＋2）及抗原（HIV1p24）联合检测" +
                ",临床微生物与寄生虫检验\t人类免疫缺陷性病毒检验" +
                ",临床微生物与寄生虫检验\t人类乳头状瘤病毒检验" +
                ",临床微生物与寄生虫检验\t人芽囊原虫培养" +
                ",临床微生物与寄生虫检验\t溶组织内阿米巴检验" +
                ",临床微生物与寄生虫检验\t溶组织内阿米巴培养" +
                ",临床微生物与寄生虫检验\t朊病毒检验" +
                ",临床微生物与寄生虫检验\t弱抗酸染色" +
                ",临床微生物与寄生虫检验\t蠕形螨检查" +
                ",临床微生物与寄生虫检验\t蠕形螨检验" +
                ",临床微生物与寄生虫检验\t赛多孢霉检验" +
                ",临床微生物与寄生虫检验\t沙门菌属检验" +
                ",临床微生物与寄生虫检验\t伤口、脓液及组织标本细菌检验" +
                ",临床微生物与寄生虫检验\t沙眼衣原体RNA检测" +
                ",临床微生物与寄生虫检验\t沙眼衣原体抗原检测" +
                ",临床微生物与寄生虫检验\t生殖道标本的微生物检验" +
                ",临床微生物与寄生虫检验\t生殖道标本细菌检验" +
                ",临床微生物与寄生虫检验\t生殖道标本真菌镜检" +
                ",临床微生物与寄生虫检验\t生殖道念珠菌培养鉴定法" +
                ",临床微生物与寄生虫检验\t生殖道念珠菌显微镜检法" +
                ",临床微生物与寄生虫检验\t生殖道沙眼衣原体细胞培养" +
                ",临床微生物与寄生虫检验\t生殖道支原体分离培养" +
                ",临床微生物与寄生虫检验\t生殖道支原体核酸检测" +
                ",临床微生物与寄生虫检验\t舌形虫检验" +
                ",临床微生物与寄生虫检验\t十二指肠液和胆汁检验寄生虫" +
                ",临床微生物与寄生虫检验\t试管凝集反应" +
                ",临床微生物与寄生虫检验\t噬菌体鉴定细菌" +
                ",临床微生物与寄生虫检验\t石蕊牛乳试验" +
                ",临床微生物与寄生虫检验\t实时定量PCR法检测TORCH特异性核酸" +
                ",临床微生物与寄生虫检验\t嗜血杆菌属检验" +
                ",临床微生物与寄生虫检验\t实验动物中卫氏并殖吸虫循环抗原（CAg）动力学观察" +
                ",临床微生物与寄生虫检验\t双相真菌检验" +
                ",临床微生物与寄生虫检验\t水洗沉淀法" +
                ",临床微生物与寄生虫检验\t术前八项" +
                ",临床微生物与寄生虫检验\t丝虫检验" +
                ",临床微生物与寄生虫检验\t梭菌属检验" +
                ",临床微生物与寄生虫检验\t糖（醇）类发酵试验" +
                ",临床微生物与寄生虫检验\t痰细菌培养的前处理" +
                ",临床微生物与寄生虫检验\t痰液病原体检查" +
                ",临床微生物与寄生虫检验\t梯度扩散法淋球菌药敏检测" +
                ",临床微生物与寄生虫检验\t铁苏木素染色法" +
                ",临床微生物与寄生虫检验\t体外抗生素敏感试验方法" +
                ",临床微生物与寄生虫检验\t体液标本细菌检验" +
                ",临床微生物与寄生虫检验\t透明胶带法丝状真菌镜检" +
                ",临床微生物与寄生虫检验\t透明质酸酶试验" +
                ",临床微生物与寄生虫检验\t土拉伦斯杆菌凝集试验" +
                ",临床微生物与寄生虫检验\t外斐反应" +
                ",临床微生物与寄生虫检验\t万古霉素耐药肠球菌（VRE）检测" +
                ",临床微生物与寄生虫检验\t弯曲菌属检验" +
                ",临床微生物与寄生虫检验\t微量肉汤稀释法淋球菌药敏检测" +
                ",临床微生物与寄生虫检验\t微球菌属检验" +
                ",临床微生物与寄生虫检验\t微丝蚴血检法" +
                ",临床微生物与寄生虫检验\t尾蚴膜反应" +
                ",临床微生物与寄生虫检验\t温度生长试验" +
                ",临床微生物与寄生虫检验\t戊型肝炎病毒IgG抗体测定" +
                ",临床微生物与寄生虫检验\t戊型肝炎病毒IgM抗体（HEV IgM）测定" +
                ",临床微生物与寄生虫检验\t无形体属检验" +
                ",临床微生物与寄生虫检验\t下呼吸道标本的处理和微生物检验" +
                ",临床微生物与寄生虫检验\t腺病毒检验" +
                ",临床微生物与寄生虫检验\t消化链球菌属检验" +
                ",临床微生物与寄生虫检验\t消化球菌属检验" +
                ",临床微生物与寄生虫检验\t硝酸盐还原试验" +
                ",临床微生物与寄生虫检验\t细菌感染性疾病的免疫检测" +
                ",临床微生物与寄生虫检验\t细菌内毒素检查法" +
                ",临床微生物与寄生虫检验\t细菌培养及鉴定" +
                ",临床微生物与寄生虫检验\t细菌性痢疾检验" +
                ",临床微生物与寄生虫检验\t性病性淋巴肉芽肿血清学检测" +
                ",临床微生物与寄生虫检验\t新生霉素敏感试验" +
                ",临床微生物与寄生虫检验\t新型冠状病毒核酸检测" +
                ",临床微生物与寄生虫检验\t旋毛虫检验" +
                ",临床微生物与寄生虫检验\t血培养" +
                ",临床微生物与寄生虫检验\t血吸虫环卵沉淀试验" +
                ",临床微生物与寄生虫检验\t血吸虫间接血凝试验" +
                ",临床微生物与寄生虫检验\t血吸虫检验" +
                ",临床微生物与寄生虫检验\t血吸虫毛蚴孵化法" +
                ",临床微生物与寄生虫检验\t血吸虫尼龙绢筛集卵法" +
                ",临床微生物与寄生虫检验\t血液及骨髓标本细菌检验" +
                ",临床微生物与寄生虫检验\t血液及骨髓细菌培养" +
                ",临床微生物与寄生虫检验\t血液寄生虫检查" +
                ",临床微生物与寄生虫检验\t芽胞杆菌属检验" +
                ",临床微生物与寄生虫检验\t氧化酶试验" +
                ",临床微生物与寄生虫检验\t氧化酶试验（改良法）" +
                ",临床微生物与寄生虫检验\t厌氧菌鉴定流程" +
                ",临床微生物与寄生虫检验\t厌氧菌检验" +
                ",临床微生物与寄生虫检验\t眼、耳标本细菌检验" +
                ",临床微生物与寄生虫检验\t耶尔森菌属检验" +
                ",临床微生物与寄生虫检验\t隐孢子虫检验" +
                ",临床微生物与寄生虫检验\t隐孢子虫卵囊染色检查" +
                ",临床微生物与寄生虫检验\t阴道加德纳菌检验" +
                ",临床微生物与寄生虫检验\t阴道毛滴虫培养" +
                ",临床微生物与寄生虫检验\t阴道、前列腺液分泌物寄生虫检查" +
                ",临床微生物与寄生虫检验\t吲哚试验" +
                ",临床微生物与寄生虫检验\t荧光梅毒螺旋体抗体吸收试验FTA‐ABS IgG" +
                ",临床微生物与寄生虫检验\t荧光梅毒螺旋体抗体吸收试验FTA‐ABS IgM" +
                ",临床微生物与寄生虫检验\t隐秘杆菌属检验" +
                ",临床微生物与寄生虫检验\t隐球菌抗原乳胶凝集试验" +
                ",临床微生物与寄生虫检验\t隐球菌属检验" +
                ",临床微生物与寄生虫检验\t阴虱检查" +
                ",临床微生物与寄生虫检验\t阴虱镜检" +
                ",临床微生物与寄生虫检验\t乙酰胺利用试验" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒表面抗体定量检测" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒表面抗原检测" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒e抗体测定" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒e抗原测定" +
                ",临床微生物与寄生虫检验\t乙型肝炎病毒核心抗体测定" +
                ",临床微生物与寄生虫检验\t医院环境卫生监测标本细菌检验" +
                ",临床微生物与寄生虫检验\t衣原体Giemsa染色" +
                ",临床微生物与寄生虫检验\t衣原体抗原测定" +
                ",临床微生物与寄生虫检验\t衣原体抗原快速金标检测" +
                ",临床微生物与寄生虫检验\t衣原体酶免疫检测" +
                ",临床微生物与寄生虫检验\t衣原体属检验" +
                ",临床微生物与寄生虫检验\t衣原体微量免疫荧光试验" +
                ",临床微生物与寄生虫检验\t衣原体直接免疫荧光染色" +
                ",临床微生物与寄生虫检验\t幽门螺旋杆菌（Hp）核酸检测" +
                ",临床微生物与寄生虫检验\t窄食单胞菌属检验" +
                ",临床微生物与寄生虫检验\t真菌Etest药敏试验" +
                ",临床微生物与寄生虫检验\t真菌KOH压片" +
                ",临床微生物与寄生虫检验\t真菌分离培养" +
                ",临床微生物与寄生虫检验\t真菌感染的实验室检查" +
                ",临床微生物与寄生虫检验\t真菌鉴定流程" +
                ",临床微生物与寄生虫检验\t真菌检验" +
                ",临床微生物与寄生虫检验\t真菌染色标本镜检" +
                ",临床微生物与寄生虫检验\t真菌小培养" +
                ",临床微生物与寄生虫检验\t真菌直接标本镜检" +
                ",临床微生物与寄生虫检验\t制动试验" +
                ",临床微生物与寄生虫检验\t志贺菌属检验" +
                ",临床微生物与寄生虫检验\t志贺菌属（痢疾杆菌属）的系统鉴定方法" +
                ",临床微生物与寄生虫检验\t脂酶试验" +
                ",临床微生物与寄生虫检验\t纸片扩散法药敏试验" +
                ",临床微生物与寄生虫检验\t支原体培养及药敏" +
                ",临床微生物与寄生虫检验\t支原体属和脲原体属检验" +
                ",临床微生物与寄生虫检验\t锥虫检验" +
                ",临床微生物与寄生虫检验\t猪囊虫抗体测定" +
                ",临床微生物与寄生虫检验\t组织标本的微生物检验" +
                ",临床核酸和基因检验\t5，10-亚甲基四氢叶酸还原酶基因检测" +
                ",临床核酸和基因检验\tα-珠蛋白生成障碍性贫血基因检测" +
                ",临床核酸和基因检验\tβ-珠蛋白生成障碍性贫血基因检测" +
                ",临床核酸和基因检验\tAlport综合征产前诊断" +
                ",临床核酸和基因检验\tAngelman综合征产前诊断" +
                ",临床核酸和基因检验\tBCR/ABL基因检测" +
                ",临床核酸和基因检验\tBCR/ABL融合基因FISH检测" +
                ",临床核酸和基因检验\tBRAF基因检测" +
                ",临床核酸和基因检验\tCYP2C19和CYP2D6基因检测" +
                ",临床核酸和基因检验\tCYP2C9* 3等位基因焦磷酸测序检测" +
                ",临床核酸和基因检验\tDNA测序技术" +
                ",临床核酸和基因检验\tEB病毒核酸PCR检测" +
                ",临床核酸和基因检验\tFabry病产前诊断" +
                ",临床核酸和基因检验\tHLA细胞学分型测定" +
                ",临床核酸和基因检验\tHLA血清学分型测定" +
                ",临床核酸和基因检验\tHuntington舞蹈症产前诊断" +
                ",临床核酸和基因检验\tKRAS基因检测" +
                ",临床核酸和基因检验\tKallmann综合征产前诊断" +
                ",临床核酸和基因检验\tNoonan综合征产前诊断" +
                ",临床核酸和基因检验\tPrader‐Willi综合征产前诊断" +
                ",临床核酸和基因检验\tRett综合征产前诊断" +
                ",临床核酸和基因检验\tWilliams综合征产前诊断" +
                ",临床核酸和基因检验\tY染色质检验" +
                ",临床核酸和基因检验\t白血病融合基因RT-PCR检测" +
                ",临床核酸和基因检验\t斑点杂交或狭缝杂交技术" +
                ",临床核酸和基因检验\t苯丙酮尿症产前诊断" +
                ",临床核酸和基因检验\t表皮生长因子受体EGFR基因检测" +
                ",临床核酸和基因检验\t别嘌呤醇毒副作用基因检测" +
                ",临床核酸和基因检验\t丙型肝炎病毒核酸RT-PCR检测" +
                ",临床核酸和基因检验\t薄基底膜肾病产前诊断" +
                ",临床核酸和基因检验\t常染色体显性多囊肾产前诊断" +
                ",临床核酸和基因检验\t常染色体显性小脑性共济失调产前诊断" +
                ",临床核酸和基因检验\t脆性X综合征产前诊断" +
                ",临床核酸和基因检验\t脆性X综合征基因检测" +
                ",临床核酸和基因检验\t单纯疱疹病毒核酸PCR检测" +
                ",临床核酸和基因检验\t大前庭水管综合征产前诊断" +
                ",临床核酸和基因检验\t地中海贫血产前诊断" +
                ",临床核酸和基因检验\t多发性家族性息肉病产前诊断" +
                ",临床核酸和基因检验\t多发性神经纤维瘤1产前诊断" +
                ",临床核酸和基因检验\t杜氏进行性肌营养不良产前诊断" +
                ",临床核酸和基因检验\t肺炎支原体核酸PCR检测" +
                ",临床核酸和基因检验\t芬兰型先天性肾病综合征产前诊断" +
                ",临床核酸和基因检验\t伏立康唑临床药物基因组学研究" +
                ",临床核酸和基因检验\t肝豆状核变性产前诊断" +
                ",临床核酸和基因检验\t高苯丙氨酸血症筛查技术" +
                ",临床核酸和基因检验\t骨髓增殖性肿瘤AK2 V617F突变基因检测" +
                ",临床核酸和基因检验\t骨髓增殖性肿瘤JAK2 exon12基因突变检测" +
                ",临床核酸和基因检验\t骨髓增殖性肿瘤MPL W515L/K突变基因检测" +
                ",临床核酸和基因检验\t黑斑息肉病产前诊断" +
                ",临床核酸和基因检验\t华法林的基因检测与个体化治疗" +
                ",临床核酸和基因检验\t家族性高胆固醇血症产前诊断" +
                ",临床核酸和基因检验\t结核分枝杆菌耐药相关基因检测" +
                ",临床核酸和基因检验\t结核杆菌DNA定量测定" +
                ",临床核酸和基因检验\t结节性硬化症产前诊断" +
                ",临床核酸和基因检验\t脊髓性肌萎缩产前诊断" +
                ",临床核酸和基因检验\t基因突变检测技术" +
                ",临床核酸和基因检验\t基因芯片技术" +
                ",临床核酸和基因检验\t聚丙烯酰胺凝胶电泳" +
                ",临床核酸和基因检验\t聚合酶链反应和相关技术" +
                ",临床核酸和基因检验\t淋巴瘤融合基因t (11; 14) ( q13; q32) (IGH/CCND1)检测" +
                ",临床核酸和基因检验\t淋巴瘤融合基因t (14; 18) (q32; q21) (IGH/ BCL2)检测" +
                ",临床核酸和基因检验\t淋病奈瑟菌核酸PCR检测" +
                ",临床核酸和基因检验\t磷酸肌醇3-激酶CA基因检测" +
                ",临床核酸和基因检验\t流感病毒核酸RT-PCR检测" +
                ",临床核酸和基因检验\t马方综合征产前诊断" +
                ",临床核酸和基因检验\t脉冲电场凝胶电泳" +
                ",临床核酸和基因检验\t弥漫大B细胞淋巴瘤的分子诊断" +
                ",临床核酸和基因检验\t耐甲氧西林金黄色葡萄球菌相关基因检测" +
                ",临床核酸和基因检验\t黏多糖贮积症Ⅱ型产前诊断" +
                ",临床核酸和基因检验\t尿苷二磷酸葡糖醛酸转移酶1A1基因检测" +
                ",临床核酸和基因检验\t凝血因子Ⅺ缺乏症产前诊断" +
                ",临床核酸和基因检验\t器官移植配型HLA基因分型检测" +
                ",临床核酸和基因检验\t琼脂糖凝胶电泳" +
                ",临床核酸和基因检验\t其他PCR技术" +
                ",临床核酸和基因检验\t染色体分析" +
                ",临床核酸和基因检验\t人表皮生长因子受体-2基因检测" +
                ",临床核酸和基因检验\t人巨细胞病毒核酸PCR检测" +
                ",临床核酸和基因检验\t人类免疫缺陷病毒1型核酸RT-PCR检测" +
                ",临床核酸和基因检验\t沙眼衣原体核酸PCR检测" +
                ",临床核酸和基因检验\t生物素酶缺乏症与全羧化酶合成酶缺乏症筛查" +
                ",临床核酸和基因检验\t神经母细胞瘤N-myc基因扩增FISH检测" +
                ",临床核酸和基因检验\t实时荧光PCR技术" +
                ",临床核酸和基因检验\t手足口病肠道病毒核酸RT-PCR检测" +
                ",临床核酸和基因检验\t四氢生物蝶呤缺乏症产前诊断" +
                ",临床核酸和基因检验\t锁骨颅骨发育不全综合征产前诊断" +
                ",临床核酸和基因检验\t唐氏综合征和性染色体多倍体疾病检测" +
                ",临床核酸和基因检验\t糖原累积病Ⅰ型产前诊断" +
                ",临床核酸和基因检验\t先天性耳聋产前诊断" +
                ",临床核酸和基因检验\t先天性软骨发育不全" +
                ",临床核酸和基因检验\t先天性肾上腺皮质增生症产前诊断" +
                ",临床核酸和基因检验\t先天性眼球震颤产前诊断" +
                ",临床核酸和基因检验\t先天性眼外肌纤维化产前诊断" +
                ",临床核酸和基因检验\t细胞色素P450——CYP2C9基因芯片检测" +
                ",临床核酸和基因检验\t性染色体测定" +
                ",临床核酸和基因检验\t性染色质测定" +
                ",临床核酸和基因检验\t新生儿Citrin蛋白缺乏症筛查" +
                ",临床核酸和基因检验\t新生儿苯丙酮尿症筛查" +
                ",临床核酸和基因检验\t新生儿丙酸血症筛查" +
                ",临床核酸和基因检验\t新生儿多种酰基辅酶A脱氢酶缺乏症筛查" +
                ",临床核酸和基因检验\t新生儿枫糖尿症筛查" +
                ",临床核酸和基因检验\t新生儿甲基丙二酸血症筛查" +
                ",临床核酸和基因检验\t新生儿极长链酰基辅酶A脱氢酶缺乏症筛查" +
                ",临床核酸和基因检验\t新生儿酪氨酸血症筛查" +
                ",临床核酸和基因检验\t新生儿尿素循环障碍筛查" +
                ",临床核酸和基因检验\t新生儿葡萄糖-6-磷酸脱氢酶缺乏症筛查" +
                ",临床核酸和基因检验\t新生儿四氢生物蝶呤缺乏症筛查" +
                ",临床核酸和基因检验\t新生儿同型半胱氨酸尿症筛查" +
                ",临床核酸和基因检验\t新生儿戊二酸尿症Ⅰ型筛查" +
                ",临床核酸和基因检验\t新生儿先天性肾上腺皮质增生症筛查" +
                ",临床核酸和基因检验\t新生儿异戊酸血症筛查" +
                ",临床核酸和基因检验\t新生儿原发性肉碱缺乏症筛查" +
                ",临床核酸和基因检验\t新生儿中链酰基辅酶A脱氢酶缺乏症筛查" +
                ",临床核酸和基因检验\t血友病A产前基因诊断" +
                ",临床核酸和基因检验\t眼皮肤白化病产前诊断" +
                ",临床核酸和基因检验\t叶酸代谢酶的基因多态性" +
                ",临床核酸和基因检验\t遗传性多发性外生型骨疣产前诊断" +
                ",临床核酸和基因检验\t遗传性耳聋基因检测" +
                ",临床核酸和基因检验\t遗传性腓骨肌萎缩症产前诊断" +
                ",临床核酸和基因检验\t荧光原位杂交技术" +
                ",临床核酸和基因检验\t印迹杂交技术" +
                ",临床核酸和基因检验\t乙型肝炎病毒核酸PCR检测" +
                ",临床核酸和基因检验\t乙型肝炎病毒基因分型检测" +
                ",临床核酸和基因检验\t乙型肝炎病毒耐药相关基因检测" +
                ",临床核酸和基因检验\t幽门螺杆菌核酸PCR检测" +
                ",临床核酸和基因检验\t尤因肉瘤EWS基因检测" +
                ",临床核酸和基因检验\t孕妇外周血胎儿游离DNA产前筛查" +
                ",临床核酸和基因检验\t质子泵抑制剂的代谢与药物基因组学特点" +
                ",有毒物质检验\t血液、血浆、血清铝测定" +
                ",有毒物质检验\t安非他明筛查试验" +
                ",有毒物质检验\t安眠酮确认试验" +
                ",有毒物质检验\t阿片类药物确认试验" +
                ",有毒物质检验\t苯巴比妥筛查试验" +
                ",有毒物质检验\t苯二氮䓬类筛查试验" +
                ",有毒物质检验\t苯环己哌啶确认试验" +
                ",有毒物质检验\t丙氧酚确认试验" +
                ",有毒物质检验\t大麻素确认试验" +
                ",有毒物质检验\t丁丙诺啡筛查试验" +
                ",有毒物质检验\t二亚甲基双氧苯丙胺筛查试验" +
                ",有毒物质检验\t可卡因筛查试验" +
                ",有毒物质检验\t美沙酮确认试验" +
                ",有毒物质检验\t尿液苯乙烯测定" +
                ",有毒物质检验\t尿液扁桃酸测定" +
                ",有毒物质检验\t尿液二甲苯测定" +
                ",有毒物质检验\t尿液二甲基乙酰胺测定" +
                ",有毒物质检验\t尿液芬太尼测定" +
                ",有毒物质检验\t尿液甲基马尿酸测定" +
                ",有毒物质检验\t尿液邻甲酚测定" +
                ",有毒物质检验\t尿液铝测定" +
                ",有毒物质检验\t尿液麦角酸二乙基酰胺测定" +
                ",有毒物质检验\t尿液四氢大麻酚测定" +
                ",有毒物质检验\t尿液乙苯测定" +
                ",有毒物质检验\t尿液游离和结合苯酚测定" +
                ",有毒物质检验\t尿液、血液甲苯测定" +
                ",有毒物质检验\t羟氢可待酮确认试验" +
                ",有毒物质检验\t三氯乙烯、三氯乙醇测定" +
                ",有毒物质检验\t砷测定方法" +
                ",有毒物质检验\t血浆蝰蛇毒时间测定" +
                ",有毒物质检验\t血清狄氏剂测定" +
                ",有毒物质检验\t血清多氯联苯测定" +
                ",有毒物质检验\t血清二氯苯二氯乙烷测定" +
                ",有毒物质检验\t血清二氯苯二氯乙烯测定" +
                ",有毒物质检验\t血清二氯苯三氯乙烷测定" +
                ",有毒物质检验\t血清六氯苯测定" +
                ",有毒物质检验\t血清六氯化苯测定" +
                ",有毒物质检验\t血清氯丹测定" +
                ",有毒物质检验\t血清七氯、环氧七氯测定" +
                ",有毒物质检验\t血清四氯乙烯测定" +
                ",有毒物质检验\t血清糖缺失性转铁蛋白测定" +
                ",有毒物质检验\t血清异狄氏剂测定" +
                ",有毒物质检验\t血清、血浆咖啡因测定" +
                ",有毒物质检验\t血清、血浆、尿液铊测定" +
                ",有毒物质检验\t血液甲醇测定" +
                ",有毒物质检验\t血液甲基异丁基甲酮测定" +
                ",有毒物质检验\t血液甲基正丁酮测定" +
                ",有毒物质检验\t血液吗啡测定" +
                ",有毒物质检验\t血液有机磷农药测定" +
                ",有毒物质检验\t血液、尿液百草枯测定" +
                ",有毒物质检验\t血液、尿液苯测定" +
                ",有毒物质检验\t血液、尿液铋测定" +
                ",有毒物质检验\t血液、尿液丁酮测定" +
                ",有毒物质检验\t血液、尿液鹅膏毒肽" +
                ",有毒物质检验\t血液、尿液镉测定" +
                ",有毒物质检验\t血液、尿液汞测定" +
                ",有毒物质检验\t血液、尿液尼古丁测定" +
                ",有毒物质检验\t血液、尿液铅测定" +
                ",有毒物质检验\t血液、尿液氰化物测定" +
                ",有毒物质检验\t血液、尿液乙醇测定" +
                ",有毒物质检验\t血液、血浆、血清、尿液砷测定" +
                ",有毒物质检验\t氧吗啡酮确认试验" +
                ",其他\t角膜荧光素染色" +
                ",其他\t基础代谢功能检测组合" +
                ",其他\t快速石蜡切片" +
                ",其他\t颞动脉活检" +
                ",其他\t全套代谢功能检测组合" +
                ",其他\t唾液流量测定" +
                ",其他\t肿瘤筛查");

        jianyanSplit = jianyan.toString().split(",");
        for(String temp: jianyanSplit) {
            String[] ele = temp.split("\t");
            jianyanMap.put(ele[1],ele[0]);
        }
    }

}

